(* ::Package:: *)

(*
Author:

    Frank Tabakin, Email : tabakin@pitt.edu,
    Department of Physics & Astronomy,
    University of Pittsburgh, Pittsburgh, PA 15260

version 2.0, May 26, 2015
*)

    
 
BeginPackage["QDENSITY`BTSystem`"];
rcsRevision2 = StringReplace["$Revision: 2.00 $", "$" -> ""];
rcsDate2     = StringReplace["$Date: 5-26-2015 1:00:00  $", "$" -> ""];

Print["Type \"?introBT\", \"?QDENSITY`BTSystem`*\" or \"?aboutBT\ for help."];

$PrePrint = MatrixForm; 
Off[General::"spell1"]

(* Begins USAGE description of Commands *)
aboutBT::usage = 
" Qdensity is a Mathematica package to simulate a quantum computer 
\n on a classical one. 

BTSystem is an additional feature added to QDensity to allow for quntum systems
that are composed of a mixture of qubits and qutrits.  The qubits have 2 degrees
 of freedom and are denoted as Binary or B; whereas qutrits have three degrees of
freedom and are denotes as Triplet or T.  The BT system is defined by the array QA.
For example, QA={B,B,B,T} designates a BT system consisting of
 3 qubits plut one qutrit.

We acknowledge the stimulating questions 
posed by Kapil Sharma. ";

introBT::usage =
"The quantum states and density matix of quantum systems which consist of
mixtures of qubits and qutrits are provided in this notebook.
Partial trace, and othe commands are also provided 
and are illustrated."
 
(*  ================================================================= *)
(*  START USAGE DOCUMENTATION  *)

KetT::usage=
"KetT[0], KetT[1] KetT[2],
\n one qutrit |0\[RightAngleBracket] |1> and |2\[RightAngleBracket]  triplet states, respectively.";

BraT::usage=
"BraT[0], BraT[1],
\n one qutrit \[LeftAngleBracket]0|, \[LeftAngleBracket]1|, state, respectively.";

\[ScriptCapitalP]T::usage=
"\!\(\*SubscriptBox[\(\[ScriptCapitalP]T\), \(0\)]\) , \!\(\*SubscriptBox[\(\[ScriptCapitalP]T\), \(\(1\)\(\\\ \)\(,\)\(\\\ \)\)]\)\!\(\*SubscriptBox[\(\[ScriptCapitalP]T\), \(2\)]\) ,
\n  qutrit projection operators.";



PartialTraceBT::usage=
"PTrBT[QA,TO, \[CapitalOmega]2] 
\n
\n PTrBT[QA,traceout,\[CapitalOmega]] returns the partial 
\n trace of a general operator \[CapitalOmega] for a mixed BT system defined by the
array QA, the array traceout or TO, select the qubits or qutrits. to be traced out.
For example, PTrBT[{B,B,T,T},{2,3,4}, \[Rho]] traces out the qubit 2, and the qutrits 3 and 4
for the B B T T system described by the density matrix \[Rho].
\n see also PTr";

PartialTransposeBT::usage=
"PartialTransposeBT{QA,BL,\[CapitalOmega]] 
\n 
\n For \!\(\*
StyleBox[\"qubit\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"(\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"binary\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"B\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\")\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"plus\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"qutrit\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"(\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"triplet\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"T\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\")\",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\" \",\nFontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"systems\",\nFontSlant->\"Italic\"]\)  the command
\n PartialTransposeBT{QA,BL,\[CapitalOmega]] returns the Partial Transpose of the operator \[CapitalOmega],
\n for mixed BT density matrices, where the array QA stipulates the BT blend 
\n and BL defines which qubits and qutrits are in subsystem Bs.
\n  For example, QA={B,B,T,T} is a (qubit_1 X qubit_2 X qutrit_3 X qutrit_4) system
and BL={1,4} defines a subsystem (qubit_1 X qutrit_4) (subsystem Bs)
\n  and its complement (qubit_2 X qutrit_3) subsystem As
\n see also PartialTranspose";

w::usage=
"w[i] with i={0,1,2,3,...8}
\n Returns the spin 1  matrices.";
wT::usage=
"wT[i] with i={0,1,2,3,...8}
 Returns the spin 1  matrices with the rank 2 part in spherical tensor form.";

Gellmann::usage= " see wG for Gell-mann SU(3) matrices.";

wG::usage=
"wG[ig] with ig={0,1,2,3,...8}
 Returns the  9 SU(3) spin 1  Gell-mann matrices.";

SPBTA::usage=
"SPBTA[Length[V],QA, compts]
\n SPBTA returns the tensor or spin product of 
\n a BT system with Length[V] qubits+qutrits, as specified by the array QA
 with the components specified by the array compts. 
For example,SPBTA[3,{T,B,B},{i,j,k}] forms the T B B system
\!\(\*
StyleBox[\"(\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"qutrit\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"X\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"qubit\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"X\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"qubit\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\")\",\nFontWeight->\"Bold\"]\)with components i,j,k.";

SPT::usage=
"SPT[Length[V], compts]
\n SPT returns the tensor or spin product of 
\n a purely qutrit system with Length[V]qutris, 
 with the components specified by the array compts. 
For example,SPT[3,{i,j,k}] forms the T T T system
\!\(\*
StyleBox[\"(\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"qutrit\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"X\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"qutrit\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"X\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\" \",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\"qutrit\",\nFontWeight->\"Bold\"]\)\!\(\*
StyleBox[\")\",\nFontWeight->\"Bold\"]\)with components i,j,k.
see SPBTA for mixed B and T states";
 
 

DFormT::usage=
"DFormT[state]
 gives output of triplet Ket states in Dirac form.
Use DFormA for triplet Bra states";

DFormTA::usage=
"DFormTA[state]
Use DFormTA for triplet Bra states.
 Gives output of state in Dirac form.
Use DFormT for triplet Ket states";

DFormBT::usage=
"DFormBT[BTarray,state]
 gives output of mixed binary-triplet(BT)
 Ket states in Dirac form.  BTarray species the nature of
the BT mixed states; for example, BTarray={B,B,T} stipulates 
that we have a qubit X qubit X qutrit state.
Use DFormBTA for BT Bra states";

DFormBTA::usage=
"DFormBTA[BTarray,state]
Use DFormBTA for mixed binary-triplet(BT) Bra states.
 Gives output of state in Dirac form.
Use DFormBT for triplet Ket states";

RandomBTKet::usage=
"RandomBTKet[QA], 
\n\t returns a random BT system Ket,
where the array stipulates the type of BT qubits(B) and qutrits(T).
For example, QA={B,B,T} is a qubitXqubitXqutrit state";

RandomBTBra::usage=
"RandomBTBra[QA], 
\n\t returns a random BT system Bra,
where the array stipulates the type of BT qubits(B) and qutroits(T).
For example, QA={B,B,T} is a qubitXqubitXqutrit state";

RandomBT::usage=
"RandomBT[QA], 
\n\t returns a random BT system density matrix,
where the array stipulates the type of BT qubits(B) and qutrits(T).
For example, QA={B,B,T} is a qubitXqubitXqutrit state";

RandomQutrit::usage=
"RandomQutrit[nT], 
\n\t returns a random qutrit density matrix for nT qutrits";

KetVBT::usage=
"KetVBT[L,QA, QS],
\n\t  KetVBT[L,QA,QS] is a BT generalization to KetV; L= number of qubits+qutrits,
QA is array of type {B,B,T....} and QS=an array for the Ket states, which is 0 or 1 for qubits
or 0,1,2 for qutrits ";

PartnerT1::usage=
"PartnerT1[isT],
\n\t For one-qutrit operators acting on qutrit isT ,PartnerT1 is used determine the  nonet sets.  
PartnerT1[isT].   ";

PickT1::usage=
"PickT1[[isT icase,iv],
\n\t For one-qutrit operators acting on qutrit isT , PickT1 is used to pick out the members of each nonet set labelled by icase;  icase 
runs from 1 to 3**nqt/9.  iv runs from 1 to 9.
PickT1[isI,icase,iv].   ";

PartnerT2::usage=
"PartnerT2[isT1,isT2],
\n\t For two-qutrit operators acting on qutrits isT1  and isT2, PartnerT2 is used determine the  nonet sets.  
PartnerT2[isT1,isT2].   ";

PickT2::usage=
"PickT2[[is1,is2, icase,iv],
\n\t For two-qutrit operators acting on qutrits is1  and is2 , PickT2 is used to
 pick out the members of each nonet set labelled by icase;  icase 
runs from 1 to 3**nqt/9.  iv runs from 1 to 9.
PickT2[is1,is2,icase,iv].   ";

OpT1::usage=
"OpT1[\[CapitalOmega], isT,state],
\n\t One-qutrit operator \[CapitalOmega] acts on qutrit isT   in multi-qutrit state state; Using QCMPI method 
OpT1[\[CapitalOmega],isT,state]. Using QCMPI method";

OpT2::usage=
"OpT2[\[CapitalOmega], isT1,isT2,state],
\n\t Two-qutrit operator \[CapitalOmega] acts on qutrits isT1  and isT2   in multi-qutrit state state; Using QCMPI method 
OpT2[\[CapitalOmega], isT1,isT2,state]. Using QCMPI method  ";

DM::usage=
"DM[p,q],
\n\t Generalized  d x d  Pauli Operators  DM[p,q]= Z^p.X^q. For d=3 yields qutrit basis ";

wP::usage=
"wP[k],
\n\t  Define nine Pauli operators wP[0]...wP[8] for  qutrits.  ";

PartnerBT1::usage=
" PartnerBT1[QA,  isBT],
\n\t Generates the partners for qubit/qutrit isBT for a BT system stipulated by QA array-using 
QCMPI-type procedure.  For example, PartnerBT1[{B,T,B},  1]=\!\(\*TagBox[TemplateBox[{\"1\",\"7\",\"2\",\"8\",\"3\",\"9\",\"4\",\"10\",\"5\",\"11\",\"6\",\"12\"},\n\"RowDefault\"],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\) ";

PickBT1::usage=
"PickBT1[QA,is,icaseBT,ip],

{PickBT1[{B,T,B},1,1,1],PickBT1[{B,T,B},1,1,2],PickBT1[{B,T,B},1,1,3]}=172 ";

PartnerBT2::usage=
" PartnerBT2[QA,  isBT1,isBT2],
\n\t Generates the partners for a BT system stipulated by QA array,
 for qubit qutrit pair  isBT1,isBT2  ";


PickBT2::usage=
"PickBT2[QA,is1,is2,icaseBT,ip],
For BT  qubit qutrit pair is1,is2  , PickBT2 
 picks out the partner set labelled by icaseBT;  icaseBT 
runs from 1 to 2^nB 3^nT/int,  where int=2 (for a qubit) or 3 (for a  qutrit),
 and nB=number of qubits and nT= number of qutrits.
 Also, ip runs from 1 to int. ";

DToMR::usage=
"DToMR[n,QD],
DToMR is a truncated version of  Seth  J. Chandler's Wolfram `Demonstration Mixed Radix Number Representations.`  \[IndentingNewLine]\n\t MR\[Equal]Mixed Radix is stipulated by the BT array QA and  n denotes a base 10 digit.
\n\t For example, Row[DToMR[5,{2,3,3}]]=0 1 2  , for a BT system QA={B,T,T}. see MRToD  ";

MRToD::usage=
"MRToD[n,QD],
MRToD is a truncated version of  Seth  J. Chandler's Wolfram Demonstration `Mixed Radix Number Representations.`  \[IndentingNewLine]\n\t MR\[Equal]Mixed Radix is stipulated by the BT array QA and  n denotes a BT base digit.
MRToD returns a base 10 digit.
\n\t For example, MRToD[{0,1,2},{2,3,3}]=5  , for a BT system QA={B,T,T}.   see DToMR ";

BTbit::usage=
"BTbit[isBT,npBT,QD],
\n\t BTbit returns the BT bit for qubit/qutrit isBT for BT system labelled by
decimal array QD. For example: for QA={B,T,T}, QD={2,3,3}. 
and  {BTbit[1,6,{2,3,3}],BTbit[2,6,{2,3,3}],BTbit[3,6,{2,3,3}]}=\!\(\*TagBox[TemplateBox[{\"0\",\"1\",\"2\"},\n\"RowDefault\"],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\) ";

BTOp1::usage=
"BTOp1[QA_,\[CapitalOmega]c, isc],
\n\t BTOp1 constructs the BT system operator for  a one qubit/qutrit operator \[CapitalOmega]c  acting on qubit/qutrit `is`.  ";

BTOp2::usage=
"BTOp2[QA,,\[CapitalOmega]c2 isc1, isc2] ,
\n\t BTOp2 constructs the BT system operator for  a two qubit/qutrit operator \[CapitalOmega]c2  acting on qubit/qutrit pair
 `isc1,isc2 `. ";

\[ScriptCapitalH]T::usage=
"\[ScriptCapitalH]T,
\n\t \[ScriptCapitalH]T is the one-qutrit Hadamard operator operator.  Also denoted as F, hadT, and w[10]";

F::usage=
"F,
\n\t F= \[ScriptCapitalH]T is the one-qutrit Hadamard operator operator.  Also denoted as F, hadT, and w[10]";

hadT::usage=
"hadT,
\n\t hadT= \[ScriptCapitalH]T is the one-qutrit Hadamard operator operator.  Also denoted as F, hadT, and w[10]";

MutualBT::usage=
"MutualBT[\[CapitalOmega]],
\n\t returns the SA,SB and SAB and the mutual entropy for the density matrix \[CapitalOmega] for BT systems, with QA stipulated before useage.";


(*  END USAGE DOCUMENTATION  *)
(*  ================================================================= *)

(*  START DEFINITIONS *)

(*  Define: Adj2 as Adj same  as given in QDensity*)
 
Adj2[x2_?MatrixQ] := Transpose[Conjugate[x2]]; 

(*  Define qutrit  T arrays *)

KetT[0]:={{1},{0},{0}};
KetT[1]:={{0},{1},{0}};
KetT[2]:={{0},{0},{1}};
BraT[0]:=Adj2[KetT[0]];
BraT[1]:=Adj2[KetT[1]];
BraT[2]:=Adj2[KetT[2]];

(*  Define: qutrit projection operators  *)

Subscript[\[ScriptCapitalP]T, ipr_]:=AF2[KetT[ipr]\[TensorProduct]BraT[ipr]];

(*  Define: Four Pauli Matrices  same as in Qdensity  *) 

sB[0]:={{1,0},{0,1}};
sB[1]:={{0,1},{1,0}};
sB[2]:={{0,-I},{I,0}};
sB[3]:={{1,0},{0,-1}};
sB[10]=(sB[1] + sB[3])/Sqrt[2]  ;
sB[11]=sB[1]

Subscript[\[ScriptCapitalP]B, 0]=(sB[0] +   sB[3])/2;
Subscript[\[ScriptCapitalP]B, 1]=(sB[0] -  sB[3])/2;
sB[12]=Subscript[\[ScriptCapitalP]B, 0];sB[13]=Subscript[\[ScriptCapitalP]B, 1];
sB[33]=sB[3]

(*  Define: Spin 1 Matrices  *) 
(*    CREATE  I, Sx, Sy, Sy operators for spin 1 system    *)

S0 := {{1, 0, 0},{0, 1, 0}, {0, 0, 1}};
Sx  :=1/Sqrt[2] *({{0, 1, 0},{1, 0, 1},{0, 1, 0}});
Sy  :=1/Sqrt[2] *({{0, -I, 0},{I, 0, -I},{0, I, 0}});
Sz  :=({{1, 0, 0},{0, 0, 0},{0, 0, -1}});
iden := S0 ;

(*    CREATE  Tij rank 2  Cartesian    *)
txx  :=   Sx.Sx - (2/3) iden ;
tyy  :=   Sy.Sy - (2/3) iden ;
tzz  :=   Sz.Sz - (2/3) iden ;
txy  :=   (Sx.Sy + Sy.Sx)/2 ;
txz  :=   (Sx.Sz + Sz.Sx)/2 ;
tyz  :=   (Sy.Sz + Sz.Sy)/2 ;

(* Alternate rank 2 spherical tensor  *)
t20  :=   Sqrt[3/2] tzz ;
t21  :=   -txz - I tyz ;
t2m1  :=   +txz - I tyz ;
t22  :=   (txx - tyy)/2 + I txy; 
t2m2  :=   (txx - tyy)/2 - I txy; 

(*     CREATE nine spin-1  operator basis  w (0)  --  w(8)    *)

w[0]  :=   iden  ;
w[1]  :=   Sqrt[3/2] Sx  ;
w[2]  :=   Sqrt[3/2] Sy; 
w[3]  :=   Sqrt[3/2] Sz;  
w[4]  :=   Sqrt[6] txy   ;
w[5]  :=   Sqrt[6] txz  ;
w[6]  :=   Sqrt[6] tyz  ;
w[7]  :=   Sqrt[3]/2 (+(txx - tyy) - Sqrt[3] tzz)  ;
w[8]  :=   Sqrt[3]/2 (-(txx - tyy) - Sqrt[3] tzz)  ;


(* Rank 2 spherical tensor *)
wT[0] :=  w[0];
wT[1] :=  w[1];
wT[2] :=  w[2];
wT[3] :=  w[3];
wT[4]  :=  Sqrt[3] t20  ;
wT[5]  :=  Sqrt[3] t21 ;
wT[6]  :=  Sqrt[3] t2m1  ;
wT[7]  :=  Sqrt[3] t22  ;
wT[8]  :=  Sqrt[3] t2m2  ;

(* The nine 3 X 3 SU(3) Gellman matrices *)
wG[0]:= Sqrt[2/3]*{{1, 0, 0},{0, 1, 0}, {0, 0, 1}};
wG[1]:={{0, 1, 0},{1, 0, 0}, {0, 0, 0}};
wG[2]:={{0, -I, 0},{I, 0, 0},{0, 0, 0}};
wG[3]:={{1, 0, 0}, {0, -1, 0},{0, 0, 0}};
wG[4]:={{0, 0, 1},{0, 0, 0},{1, 0, 0}};
wG[5]:={{0, 0, -I}, {0, 0, 0},{I, 0, 0}};
wG[6]:={{0, 0, 0}, {0, 0, 1},{0, 1, 0}};
wG[7]:={{0, 0, 0},{0, 0, -I},{0, I, 0}};
wG[8]:=1/Sqrt[3] *({ {1, 0, 0},{0, 1, 0},{0, 0, -2}}); ;

(*  ================================================================= *)
 
(* -- Define:  Tensor Product  special case  *)
TProd[xq_?MatrixQ,yq_?MatrixQ]:=ArrayFlatten[Outer[Times,xq,yq]]; 

 
(* -- Define:  Tensor Products for BT systems *)
(* SPBTA is a generalization of above SP.
 Now:
 QA={ B,T,B,T etc} type of spin stipulation ,
and QS={ 2,4,1,3 etc} component stipulation.
SPBTA can be used to build BTBTBB...type  product states *)
 
 BTop[T, icv_]:=w[ icv];
 BTop[B, icv_]:=sB[ icv];
SPBTA[L_,QAA_, QSS_] := SPBTA[L,QAA, QSS] = If [L > 1, TProd[SPBTA[L - 1,Drop[QAA,-1],Drop[QSS,-1]],BTop[QAA[[L]],QSS[[L]]] ],BTop[QAA[[1]],QSS[[1]]]];

(* KetVBT[L,QA,QS] is a BT generalization to KetV; L= number of qubits+qutrits,
QA is defines as above and QS=an array for the Ket states, which is 0 or 1 for qubits
or 0,1,2 for qutrits *)

 BTop2[T, icv_]:=KetT[ icv];
 BTop2[B, icv_]:=Ket[ icv];
KetVBT[L_,QAA_, QSS_] := KetVBT[L,QAA, QSS] = If [L > 1, TProd[KetVBT[L - 1,Drop[QAA,-1],Drop[QSS,-1]],BTop2[QAA[[L]],QSS[[L]]] ],BTop2[QAA[[1]],QSS[[1]]]];

(* SPT is a generalization of SP Now L=number of Triplet states.
SPT can be used to build triplet product states w[1]....w[9] *)

SPT[L_, Qz_] := SPT[L, Qz] = If [L > 1, TProd[SPT[L - 1,Drop[Qz,-1]],w[Qz[[L]]] ],w[Qz[[1]]]];

(* General Dirac Form command for hybrid BT states *)

DFormBT[QA3_, \[CapitalOmega]3_ ]:=Module[{stV3,QDx,nf3, nB,nT,ns3,coef3,size3,plist3,pS,pST,V,V2,wx,dx,X3,qnos},
If[VectorQ[\[CapitalOmega]3],
stV3=Flatten[\[CapitalOmega]3],stV3=\[CapitalOmega]3];
If[VectorQ[stV3],
coef3[js3_]:=stV3[[js3]],coef3[js3_]:=stV3[[js3]][[1]]  ];
size3=Length[stV3];
QDx =QA3 /.{B->2,T->3};
nB=Length[Cases[QA3,B]];
nT=Length[Cases[QA3,T]];
nf3=Length[QA3];
dx[i3x_] :=QDx[[i3x]];
V = Array[{{wx[#], 0, dx[#]-1}} &, nf3];
  Do[{V = MapAt[Apply[Sequence, #] &, V, {i3}]}, {i3, 1, nf3 }];
   V = FlattenAt[V, nf3];
   V = Partition[Flatten[V], 3];
V2 = Array[{{wx[#]}} &, nf3];
  Do[{V2 = MapAt[Apply[Sequence, #] &, V2, {i3}]}, {i3, 1, nf3 }];
   V2 = FlattenAt[V2, nf3];
   V2 = Partition[Flatten[V2], nf3];
X3=Row[Flatten[ Table[V2, Evaluate[Sequence @@ V]],nf3]];
plist3={};
pST[x3_]:=Style[x3,Blue,Italic,Bold];
pS[y3_,x3_]:=If[y3==2,Style[x3,Black,Bold],Style[x3,Blue,Italic,Bold]];
qnos[nsx_,isx_]:=Subscript[X3[[1]][[nsx]][[isx]],QDx[[isx]]];
Do[If[coef3[ns3]==0," ",
AppendTo[plist3,{" + ","(",coef3[ns3],")"," |",Row[Flatten[Array[{pS[QDx[[#]],qnos[ns3,#]]} &, Length[QDx]],Length[QDx]]]," >"}],
AppendTo[plist3,{" + ","(",coef3[ns3],")"," |",Row[Flatten[Array[{pS[QDx[[#]],qnos[ns3,#]]} &, Length[QDx]],Length[QDx]]]," >"}]],
{ns3,1,size3}];
 Row[Flatten[plist3]]];

DFormBTA[QA3_, \[CapitalOmega]3_ ]:=Module[{stV3,QDx,nf3, nB,nT,ns3,coef3,size3,plist3,pS,pST,V,V2,wx,dx,X3,qnos},
If[VectorQ[\[CapitalOmega]3],
stV3=Flatten[Transpose[\[CapitalOmega]3]],stV3=Transpose[\[CapitalOmega]3]];
If[VectorQ[stV3],
coef3[js3_]:=stV3[[js3]],coef3[js3_]:=stV3[[js3]][[1]]  ];
size3=Length[stV3];
QDx =QA3 /.{B->2,T->3};
nB=Length[Cases[QA3,B]];
nT=Length[Cases[QA3,T]];
nf3=Length[QA3];
dx[i3x_] :=QDx[[i3x]];
V = Array[{{wx[#], 0, dx[#]-1}} &, nf3];
  Do[{V = MapAt[Apply[Sequence, #] &, V, {i3}]}, {i3, 1, nf3 }];
   V = FlattenAt[V, nf3];
   V = Partition[Flatten[V], 3];
V2 = Array[{{wx[#]}} &, nf3];
  Do[{V2 = MapAt[Apply[Sequence, #] &, V2, {i3}]}, {i3, 1, nf3 }];
   V2 = FlattenAt[V2, nf3];
   V2 = Partition[Flatten[V2], nf3];
X3=Row[Flatten[ Table[V2, Evaluate[Sequence @@ V]],nf3]];
plist3={};
pST[x3_]:=Style[x3,Blue,Italic,Bold];
pS[y3_,x3_]:=If[y3==2,Style[x3,Black,Bold],Style[x3,Blue,Italic,Bold]];
qnos[nsx_,isx_]:=Subscript[X3[[1]][[nsx]][[isx]],QDx[[isx]]];
Do[If[coef3[ns3]==0," ",
AppendTo[plist3,{" + ","(",coef3[ns3],")"," < ",Row[Flatten[Array[{pS[QDx[[#]],qnos[ns3,#]]} &, Length[QDx]],Length[QDx]]]," |"}],
AppendTo[plist3,{" + ","(",coef3[ns3],")"," < ",Row[Flatten[Array[{pS[QDx[[#]],qnos[ns3,#]]} &, Length[QDx]],Length[QDx]]]," |"}]],
{ns3,1,size3}];
 Row[Flatten[plist3]]];

(* Some Dirac Form commands for pure triplet states:  based on DFormBT setup *)

DFormT[statV_] :=DFormBT[Table[T,{IntegerPart[Log[3,Length[statV]]]}],statV]
DFormTA[statV_] :=DFormBTA[Table[T,{IntegerPart[Log[3,Length[Transpose[statV]]]]}],statV]

 (* Partial  Trace for BT systems *)

 PTrBT[QA_ /; VectorQ[QA],TO_ /; VectorQ[TO], \[CapitalOmega]2_ /; MatrixQ[\[CapitalOmega]2]] := 
Module[{nto,QTO,ntoB,ntoT,nf, nB,nT,nfB,nfT,dim,dimf,dx, W2, wx, VOP, WP, V, V2,X2,zp,QAf},
nto = Length[TO];
QTO=Table[QA[[TO[[i2]]]],{i2,1,nto}];
ntoB=Length[Cases[QTO,B]];
ntoT=Length[Cases[QTO,T]];
nf=Length[QA];
nB=Length[Cases[QA,B]];
nT=Length[Cases[QA,T]];
nfB=nB-ntoB;
nfT=nT-ntoT;
dim=If[nT==0,2^nB ,2^nB 3^nT   ] ;  
dim=If[nB==0,3^nT ,2^nB 3^nT   ];
dimf=If[nfT==0,2^nfB ,2^nfB 3^nfT   ];   
dimf=If[nfB==0,3^nfT ,2^nfB 3^nfT   ];
dx[it_]:= If[QA[[it]]===B,3,8];
QAf= DeleteCases[ReplacePart[QA, zp, Split[TO]], zp];
If[Length[\[CapitalOmega]2]==dim," " ,Print["Error in QA or in \[CapitalOmega]"]];
If[nto == nf, X2 = Tr[\[CapitalOmega]2],
W2 = Array[wx, nf];
VOP = DeleteCases[ReplacePart[Array[wx, nf], zp, Split[TO]], zp];
WP = ReplacePart[W2, 0, Split[TO]];
V = Array[{{wx[#], 0, dx[#]}} &, nf];
        Do[{V = MapAt[Apply[Sequence, #] &, V, {i3}]}, {i3, 1, nf - nto}];
        V = FlattenAt[V, nf];
        V2 = DeleteCases[ReplacePart[V, zp, Split[TO]], zp];
        V2 = Partition[Flatten[V2], 3];
        X2 = Sum[((2^ntoB *3^ntoT)/dim)*Tr[\[CapitalOmega]2.Adj2[SPBTA[nf,QA,WP]]]*SPBTA[nf - nto,QAf, VOP], Evaluate[Sequence @@ V2]]];
 Simplify[X2]];

 (*  Specify QA and get a corresponding random BT state Ket *)
RandomBTKet[QA_]:= Module[{ket4,nv,nB,nT,d,dim4},nB=Length[Cases[QA,B]];
nT=Length[Cases[QA,T]];
dim4=If[nT==0,2^nB ,2^nB 3^nT   ];   
dim4=If[nB==0,3^nT ,2^nB 3^nT   ];d=dim4; ket4=RandomComplex[{-1-I,1+I},{d}];nv=Sqrt[Chop[Conjugate[ket4].ket4]];
Chop[ket4/nv]];

(*  Specify QA and get a corresponding random BT state Bra *)
RandomBTBra[QA_]:= Module[{bra4,nv,nB,nT,d,dim4},nB=Length[Cases[QA,B]];
nT=Length[Cases[QA,T]];
dim4=If[nT==0,2^nB ,2^nB 3^nT   ];   
dim4=If[nB==0,3^nT ,2^nB 3^nT   ];d=dim4; bra4=RandomComplex[{-1-I,1+I},{d}];nv=Sqrt[Chop[Conjugate[bra4].bra4]];
Transpose[Partition[Chop[bra4/nv],1]]];

(*  Specify QA and get a corresponding random density matrix *)
(* Useful for testing entanglement *)
RandomBT[QA_]:= Module[{rh1,rhoA,nB,nT,d,dim5},
nB=Length[Cases[QA,B]];
nT=Length[Cases[QA,T]];
dim5=If[nT==0,2^nB ,2^nB 3^nT   ];   
dim5=If[nB==0,3^nT ,2^nB 3^nT   ];d=dim5;
 rh1=RandomComplex[{-1-I,1+I},{d,d}];rhoA=(rh1+Adj2[rh1])/2.;Chop[(rhoA.rhoA)/Tr[rhoA.rhoA]]];

(* Generate Random qutrit density matrix for n qutrits *)
RandomQutrit[nr_]:= Module[{rh1,rhoA,d},d=3^nr; rh1=RandomComplex[{-1-I,1+I},{d,d}];rhoA=(rh1+Adj2[rh1])/2.;Chop[(rhoA.rhoA)/Tr[rhoA.rhoA]]]
 

(*  =================================================================  *)

YFlip2[BL_ /; VectorQ[BL],W5_ /; VectorQ[W5]] :=
Module[{nBL, WP, ic5,i5,sign}, nBL =Length[BL];ic5=1;
WP=W5; Do[If[WP[[BL[[i5]]  ]]==2 ||  WP[[BL[[i5]]  ]]==4 || WP[[BL[[i5]]  ]]==6,
ic5=-ic5,ic5=ic5],{i5,1,nBL}];sign=ic5   ; sign];

(* Partial Transpose for mixed BT density matrices  called as:  PartialTransposeBT[QA BL , \[CapitalOmega] ] *)

  PartialTransposeBT[QA_ /; VectorQ[QA],BL_ /; VectorQ[BL], \[CapitalOmega]2_ /; MatrixQ[\[CapitalOmega]2]] := 
      Module[{i5,i6,nB ,nT, n\[CapitalOmega],dim, WP,V,V2,qd,qa,nBL,X5},
qa=QA;qd=qa /. {B->2,T->3};
 nB=Length[Cases[qa,B]];
nT=Length[Cases[qa,T]];
 n\[CapitalOmega]=nT+nB;
dim=If[nT==0,2^nB ,2^nB 3^nT   ] ;  
dim=If[nB==0,3^nT ,2^nB 3^nT   ];
      nr[i4_]:= If[qd[[i4]]==2,3,8];
nBL=Length[BL];
      If[nBL == n\[CapitalOmega], X5 = Transpose[\[CapitalOmega]2],
V=Array[{{u[#-1], 0, nr[#]}} &, n\[CapitalOmega]];
         Do[{V = MapAt[Apply[Sequence, #] &, V, {i5}]}, {i5, 1, n\[CapitalOmega] }];
        V2 = Partition[Flatten[V], 3];
WP=Array[{{u[#-1]}} &, n\[CapitalOmega]];
Do[{WP= ReplacePart[WP,{ u[i6-1]},{ i6}]},{i6,1, n\[CapitalOmega]}];
WP=Flatten[WP];
        X5 = Sum[2^( -nB)*3^( -nT)*Tr[\[CapitalOmega]2.SPBTA[Length[qa],qa,WP] ]*SPBTA[Length[qa],qa,WP]*YFlip2[BL,WP]  , 
        Evaluate[Sequence @@ V2]]]; Simplify[Chop[X5]]];  

Smax[QA_ /; VectorQ[QA]]:= Module[{nB,nT,qa},qa=QA; nB=Length[Cases[qa,B]];
nT=Length[Cases[qa,T]]; res=nB+ nT*N[Log[2,3]];res]

(*  =================================================================  *)

(*  Following  generates the partners for a one-qutrit operator-using 
QCMPI-type procedure. One qutrit operator acts on qutrit "is" *)

PartnerT1[ isT_] :=
 Module[{nqt1,jpT=isT,strideT,tempT,npT,yp1T},nqt1=nqt;strideT=3^{nqt1-jpT};tempT={};
Do[If[IntegerDigits[npT-1,3,nqt1][[jpT]]==0,tempT={tempT,{npT,npT+strideT[[1]],npT+2*strideT[[1]]}},yp1T=1]
,{npT,1,3^nqt1}];Flatten[tempT]];

PickT1[isT_,icaseT_,ip1T_]:=PartnerT1[isT][[3*(icaseT-1)+ip1T]] 

(*  =================================================================  *)

(*  Following  generates the 9 partners for a two-qutrit operator-
using QCMPI-type procedure. 
Two qutrit operator acts on qutrits isT1 and isT2.
isT1<isT2 *) 

PartnerT2[ isT1_,isT2_] :=
 Module[{nqs2T,j1T=isT1,j2T=isT2,strideT1,strideT2,tempT,nT,yT},nqs2T=nqt; 
hitT1[nT_]:=IntegerDigits[nT-1,3,nqs2T][[isT1]];hitT2[nT_]:=IntegerDigits[nT-1,3,nqs2T][[isT2]];
strideT1=3^{nqs2T-j1T};strideT2=3^{nqs2T-j2T};
tempT={};
Do[If[hitT1[nT]==0&&hitT2[nT]==0,
tempT={tempT,{{nT,nT+strideT2[[1]],nT+2strideT2[[1]],nT+strideT1[[1]],nT+strideT1[[1]]+strideT2[[1]],nT+strideT1[[1]]+2 strideT2[[1]],nT+ 2 strideT1[[1]],nT+ 2 strideT1[[1]]+strideT2[[1]],nT+ 2 strideT1[[1]]+ 2 strideT2[[1]]
}}},
yT=1 ],{nT,1,2^nqs2T}]; Flatten[tempT] ];

PickT2[isT1_,isT2_,icaseT_,ivT_]:=PartnerT2[isT1,isT2][[9*icaseT-8+ivT-1]]

(*  PickT2  isT used to pick out the members of each nonet set labelled by icaseT;  icaseT runs from 1 to 3**nqx/9.  ivT runs from 1 to 9.
PickT2[isT1,isT2,icaseT,ivT].  *)




(*  =================================================================  *)

(*  One-qutrit operator \[CapitalOmega] acts on qutrit "isT"  in multi-qutrit state "stateT";
  OpT1[\[CapitalOmega],isT,stateT] ;Using QCMPI method *)

  OpT1[\[CapitalOmega]T_?MatrixQ, isT_,stateT_] :=
 Module[{wfT,tempT,icasT,nquT,ivT1,iccT,ik2},nquT=Log[3,Length[stateT]];
wfT=stateT;tempT=wfT;ivT1[icv_,ipv2_]:=PickT1[isT,icv,ipv2];
Do[ Do[tempT[[ivT1[icasT,ik2]]]=Sum[ \[CapitalOmega]T[[ik2]][[iccT]]*wfT[[ivT1[icasT,iccT]]],{iccT,1,3}],{ik2,1,3}],
{icasT,1,3^nquT/3}];tempT];

(*  =================================================================  *)

(*  Two-qutrit operator \[CapitalOmega]T acts on qutrits "isT1" and "isT2"  in state "stateT";  OpT2[\[CapitalOmega]T, isT1,isT2,stateT] ;Using QCMPI method   *)

   
 OpT2[\[CapitalOmega]T_?MatrixQ, isT1_,isT2_,stateT_] :=
 Module[{wfT,tempT,i1,i2,i3,i4,i5,i6,i7,ii8,i9,icaseT,nquT},wfT=stateT;tempT=wfT;
nquT=Log[3,Length[stateT]];
Do[
i1=PickT2[isT1,isT2,icaseT,1];i2=PickT2[isT1,isT2,icaseT,2];i3=PickT2[isT1,isT2,icaseT,3];
i4=PickT2[isT1,isT2,icaseT,4];i5=PickT2[isT1,isT2,icaseT,5];i6=PickT2[isT1,isT2,icaseT,6];
i7=PickT2[isT1,isT2,icaseT,7];ii8=PickT2[isT1,isT2,icaseT,8];i9=PickT2[isT1,isT2,icaseT,9];
        tempT[[i1]]= \[CapitalOmega]T[[1]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[1]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[1]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[1]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[1]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[1]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[1]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[1]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[1]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i2]]= \[CapitalOmega]T[[2]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[2]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[2]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[2]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[2]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[2]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[2]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[2]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[2]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i3]]= \[CapitalOmega]T[[3]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[3]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[3]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[3]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[3]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[3]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[3]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[3]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[3]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i4]]= \[CapitalOmega]T[[4]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[4]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[4]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[4]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[4]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[4]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[4]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[4]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[4]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i5]]= \[CapitalOmega]T[[5]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[5]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[5]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[5]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[5]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[5]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[5]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[5]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[5]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i6]]= \[CapitalOmega]T[[6]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[6]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[6]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[6]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[6]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[6]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[6]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[6]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[6]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i7]]= \[CapitalOmega]T[[7]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[7]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[7]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[7]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[7]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[7]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[7]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[7]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[7]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[ii8]]= \[CapitalOmega]T[[8]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[8]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[8]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[8]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[8]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[8]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[8]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[8]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[8]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
tempT[[i9]]= \[CapitalOmega]T[[9]][[1]]*wfT[[i1]]+\[CapitalOmega]T[[9]][[2]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[9]][[3]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[9]][[4]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i4", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+ \[CapitalOmega]T[[9]][[5]]*wfT[[i5]]+\[CapitalOmega]T[[9]][[6]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[9]][[7]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[9]][[8]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "ii8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega]T[[9]][[9]]*\!\(\*
TagBox[
RowBox[{"wfT", "[", 
RowBox[{"[", "i9", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\),
{icaseT,1,3^nquT/9}]; tempT];     

(*  =================================================================  *)
(* Generalized  d x d  Pauli Operators  Z^p.X^q *)
(* D_{p q}  = sum_n w^(pq) sum(0,d-1)  w^(p n)  |n+q> <n|   with w = psi[1]  *)

  DM[p_,q_]:=Module[{vec,psi},psi[kk_]:=Exp[ 2  kk I Pi/3];
vec[mm_]:=Partition[Table[KroneckerDelta[ii,mm+1],{ii,3}],1];dd= (psi[1])^(p*q)*Sum[  (psi[1])^(p*nn)(FixedPoint[ArrayFlatten,vec[Mod[nn+q,3]]\[TensorProduct]Transpose[vec[nn]]]),{nn,0,2}];dd]; 

(*  =================================================================  *)

(*  Define Generalized Hadamard for qutrits  *)

 F=Table[Exp[- 2  ss tt I Pi/3],{ss,0,2},{tt,0,2}]/Sqrt[3];
hadT=F;
\[ScriptCapitalH]T=F;
w[10]=F;


(*  Define Pauli operators X and Z for qutrits  *)
\[DoubleStruckCapitalX]= wP[1]
\[DoubleStruckCapitalZ]= wP[3]
\[DoubleStruckCapitalY]= wP[2]


(*  =================================================================  *)

(*  Define nine Pauli operators wP[0]...wP[8] for  qutrits;
set wP[10} as the hadamard--for later use *) 

wP[0]=DM[0,0];
wP[1]=DM[0,1];
wP[4]=DM[0,2];

wP[3]=DM[1,0];
wP[2]=DM[1,1];
wP[5]=DM[1,2];

wP[6]=DM[2,0];
wP[7]=DM[2,1];
wP[8]=DM[2,2];
\[DoubleStruckCapitalX]=wP[1];
\[DoubleStruckCapitalZ]=wP[3];
(*  set wP[10 as the hadamard w[11] as X,  w[12-14] as  projections
w[33} as Z and w[34} as Z.Z  w--for later use *)
wP[10]=F;
wP[11]=wP[1];
wP[33]=wP[3];
wP[34]=wP[3].wP[3];
w[11] = wP[1];
w[12]=Subscript[\[ScriptCapitalP]T, 0];w[13]=Subscript[\[ScriptCapitalP]T, 1];w[14]=Subscript[\[ScriptCapitalP]T, 2];
w[33]=wP[3];w[34]=wP[34];
w[16]=\[DoubleStruckCapitalX].\[DoubleStruckCapitalX]
(*  Used in general BT CNOTS  w[44]-w[46] are for swap CNOTH cases *)
w[44]=Simplify[Adj2[F] .w[0]. Adj2[F]]
w[45]=Simplify[Adj2[F] .\[DoubleStruckCapitalZ]. Adj2[F]]
w[46]=Simplify[Adj2[F] .\[DoubleStruckCapitalZ].\[DoubleStruckCapitalZ]. Adj2[F]]

(*  Now setup BT cases *)
DToMR[nv_Integer,radixlist_List]:=Module[{bv=Reverse@FoldList[Times,1,Reverse@radixlist],rep},
rep=Rest@Last@Last@Reap[Fold[
(Sow[Quotient[#1,#2]];Mod[#1,#2])&,nv,bv]];
rep=MapThread[Subscript,{rep,Take[radixlist,-Length[rep]]}];
rep];

MRToD[nMR_List,radixlist_List]:=nMR.Reverse@Most@FoldList[Times,1,Reverse@radixlist];

BTbit[isb_,nbb_,QDv_]:=DToMR[nbb-1,QDv][[isb]][[1]]
strideA[QDV_]:=Reverse@Most@FoldList[Times,1,Reverse@QDV];
strideBT[isb_,QDv_]:=strideA[QDv][[isb]];

PartnerBT1[QA3_,  isBT_] :=
 Module[{QDx,nB,nT,nf3,jpBT=isBT,tempBT,npBT,dimBT,dum},
QDx =QA3 /.{B->2,T->3};
nB=Length[Cases[QA3,B]];
nT=Length[Cases[QA3,T]];
nf3=Length[QA3];
dimBT=If[nT==0,2^nB ,2^nB 3^nT   ] ;  
dimBT=If[nB==0,3^nT ,2^nB 3^nT   ];
tempBT={};
Do[
If[BTbit[jpBT,npBT,QDx]==0,
       If[QDx[[jpBT]]==2, tempBT={tempBT,{npBT,npBT+strideBT[jpBT,QDx]}},
      tempBT={tempBT,{npBT,npBT+strideBT[jpBT,QDx] , npBT+2*strideBT[jpBT,QDx]}}],dum=99        
   ],
{npBT,1,dimBT}];
Flatten[tempBT]];

PickBT1[QA3_,iss_,icaseBT_,ip3_]:= Module[{QDx,a},
QDx =QA3 /.{B->2,T->3};
If[QDx[[iss]]==2,a=2*(icaseBT-1),a=3*(icaseBT-1)];
PartnerBT1[QA3,iss][[a+ip3]]  ]; 

 PartnerBT2[QA3_,  isBT1_,isBT2_] :=
 Module[{QDx,nB,nT,nf3,tempBT,npBT,dimBT,st1,st2,dum},
QDx =QA3 /.{B->2,T->3};nB=Length[Cases[QA3,B]];nT=Length[Cases[QA3,T]];
nf3=Length[QA3];
dimBT=If[nT==0,2^nB ,2^nB 3^nT   ] ;  dimBT=If[nB==0,3^nT ,2^nB 3^nT   ];
st1=strideBT[isBT1,QDx];   st2=strideBT[isBT2,QDx];
tempBT={};
Do[
If[BTbit[isBT1,npBT,QDx]==0&&BTbit[isBT2,npBT,QDx]==0,
Which[QDx[[isBT1]]==2&&QDx[[isBT2]]==2,
                    tempBT={tempBT,{{npBT,npBT+st2,npBT+st1,npBT+st1+st2}}},
QDx[[isBT1]]==2&&QDx[[isBT2]]==3,                             tempBT={tempBT,{{npBT,npBT+st2,npBT+2st2,npBT+st1,npBT+st1+st2,npBT+st1+2 st2}}},
QDx[[isBT1]]==3&&QDx[[isBT2]]==2,     tempBT={tempBT,{{npBT,npBT+st2,npBT+st1,npBT+st1+st2,npBT+ 2 st1,npBT+ 2 st1+st2}}},
QDx[[isBT1]]==3&&QDx[[isBT2]]==3,     tempBT={tempBT,{{npBT,npBT+st2,npBT+2st2,npBT+st1,npBT+st1+st2,npBT+st1+2 st2,npBT+ 2 st1,npBT+ 2 st1+st2,npBT+ 2 st1+ 2 st2
                  }}}],dum=1 
  ],
{npBT,1,dimBT}];
Flatten[tempBT]];

PickBT2[QA33_,iss1_,iss2_,icaseBT_,ip3_]:= Module[{QDx},
QDx =QA33 /.{B->2,T->3};PartnerBT2[QA33,iss1,iss2][[QDx[[iss1 ]]*QDx[[iss2 ]]*(icaseBT-1)+ip3]]  ]; 






OpBT1[QA3_,\[CapitalOmega]BT_?MatrixQ, isBT_,stateBT_] :=
 Module[{wfBT,tempBT,QDx,nB,nT,nqt,icasBT,ivBT, iccBT,ik,nopBT,dimBT},QDx =QA3 /.{B->2,T->3};
nB=Length[Cases[QA3,B]];
nT=Length[Cases[QA3,T]];
nqt=Length[QA3];dimBT=If[nB==0,3^nT,2^nB*3^nT];dimBT=If[nT==0,2^nB,2^nB*3^nT];
If[QDx[[isBT]]==2,nopBT=2,nopBT=3];
wfBT=stateBT;tempBT=wfBT;ivBT[ic2_,ipv_]:=PickBT1[QA3,isBT,ic2,ipv];
Do[ Do[tempBT[[ivBT[icasBT,ik]]]=Sum[ \[CapitalOmega]BT[[ik]][[iccBT]]*wfBT[[ivBT[icasBT,iccBT]]],{iccBT,1,nopBT}],{ik,1,nopBT}],
{icasBT,1,dimBT/nopBT}];tempBT]; 


OpBT2[QA3_,\[CapitalOmega]BT_?MatrixQ, isBT1_,isBT2_,stateBT_] :=
 Module[{wfBT,tempBT,QDx,nB,nT,icasBT,ivBT2, iccBT2,ik,nopBT,dimBT},QDx =QA3 /.{B->2,T->3};
nB=Length[Cases[QA3,B]];
nT=Length[Cases[QA3,T]];
dimBT=If[nB==0,3^nT,2^nB*3^nT];dimBT=If[nT==0,2^nB,2^nB*3^nT];
nopBT=QDx[[isBT1]]*QDx[[isBT2]];
wfBT=stateBT;tempBT=wfBT;ivBT2[icT2_,ipv_]:=PickBT2[QA3,isBT1,isBT2,icT2,ipv];
Do[ Do[tempBT[[ivBT2[icasBT,ik]]]=Sum[ \[CapitalOmega]BT[[ik]][[iccBT2]]*wfBT[[ivBT2[icasBT,iccBT2]]],{iccBT2,1,nopBT}],{ik,1,nopBT}],
{icasBT,1,dimBT/nopBT}];tempBT]; 


AF2[sta_]:=FixedPoint[ArrayFlatten,sta]
BTOp1[Qa3_,\[CapitalOmega]c_?MatrixQ, isc_] :=
 Module[{QDx,nB,nT,nqt,dim,ps,Opmat,iss},iss=isc;
QDx =Qa3 /.{B->2,T->3};
nB=Length[Cases[Qa3,B]];nT=Length[Cases[Qa3,T]];nqt=Length[Qa3];
dim=If[nT==0,2^nB ,2^nB 3^nT   ] ;  dim=If[nB==0,3^nT ,2^nB 3^nT   ];
res[jvc_]:=Table[DToMR[jvc,QDx][[ivc]][[1]],{ivc,1,nqt}];
ps[ivvc_]:=AF2[KetVBT[nqt,Qa3, res[ivvc]]];
Opmat=Transpose[Table[ArrayFlatten[OpBT1[Qa3,\[CapitalOmega]c,iss,ps[if]],1],{if,0,dim-1}]];AF2[Opmat]]

BTOp2[Qa3_,\[CapitalOmega]c2_?MatrixQ, isc1_, isc2_] :=
 Module[{QDx,nB,nT,nqt,dim,ps,Opmat,iss1,iss2},iss1=isc1;iss2=isc2;
QDx =Qa3 /.{B->2,T->3};
nB=Length[Cases[Qa3,B]];nT=Length[Cases[Qa3,T]];nqt=Length[Qa3];
dim=If[nT==0,2^nB ,2^nB 3^nT   ] ;  dim=If[nB==0,3^nT ,2^nB 3^nT   ];
res[jcc_]:=Table[DToMR[jcc,QDx][[iac]][[1]],{iac,1,nqt}];
ps[itt_]:=ArrayFlatten[KetVBT[nqt,Qa3, res[itt]]];
Opmat=Transpose[Table[ArrayFlatten[OpBT2[Qa3,\[CapitalOmega]c2,iss1,iss2,ps[ity]],1],{ity,0,dim-1}]];Opmat]


(* Hadamard  "HadBT[L,QA,Q]"   acts on  all members of set Q={q1,q2,...} of the L qubits *)
HadBT[L_,QAq_, QP_] :=HadBT[L,QAq, QP]=SPBTA[L,QAq,QP]

(* Define Hadamard acting on one qubit q out of L qubits *)

hadBT[L_,QAq_,q_]:= SPBTA[L,QAq, ReplacePart[Table[0,{L}],10,q]]

(* Define Hadamard acting on all of BT system *)

 HALLBT[L_,QAq_] :=HadBT[L,QAq, ConstantArray[10,Length[QAq]]]

(* Define NOT acting on one qubit/qutrit q out of L qubits + qutrits *)

NOTBT[QAq_,q_]:=Module[{LG,qq },LG=Length[QAq];qq=q;
res=SPBTA[LG,QAq, ReplacePart[Table[0,{LG}],11,qq] ];res];

(* Define Projectors acting on one qubit or qutrit q out of L qubits +qutrits *)

ProjBT[nvp_,QAq_,q_]:=Module[{LG,qq, QDq },LG=Length[QAq];qq=q;QDq=QAq  /.{B->2,T->3};
res=SPBTA[LG,QAq, ReplacePart[Table[0,{LG}],12+nvp,qq]];res];



OneOpBT[L_,QAq_, q1_, opBT1_] := SPBTA[L,QAq, ReplacePart[Table[0,{L}], opBT1, q1]]
TwoOpBT[L_,QAq_,qBT1_,qBT2_,opBT1_,opBT2_] :=SPBTA[L,QAq, ReplacePart[Table[0,{L}],{qBT1->opBT1,qBT2->opBT2}]]
ThreeOpBT[L_,QAq_,qBT1_,qBT2_,qBT3_,opBT1_,opBT2_,opBT3_] :=SPBTA[L,QAq, ReplacePart[Table[0,{L}],{qBT1->opBT1,qBT2->opBT2,qBT3->opBT3}]]


AF2[st_]:=FixedPoint[ArrayFlatten,st]

CNOT1BT[QAq_,q1_,q2_]:=Module[{QDq,Lvv  },QDq=QAq  /.{B->2,T->3};Lvv=Length[QAq];  
res6=  If[QDq[[q1]]==3&&QDq[[q2]]==3,
TwoOpBT[Lvv,QAq,q1,q2,12,0]+TwoOpBT[Lvv,QAq,q1,q2,13,11]+TwoOpBT[Lvv,QAq,q1,q2,14,16],res6];
res6=If[QDq[[q1]]==3&&QDq[[q2]]==2,
TwoOpBT[Lvv,QAq,q1,q2,12,0]+TwoOpBT[Lvv,QAq,q1,q2,13,11]+TwoOpBT[Lvv,QAq,q1,q2,14,0],res6];
 res6=  If[QDq[[q1]]==2&&QDq[[q2]]==3,
TwoOpBT[Lvv,QAq,q1,q2,12,0]+TwoOpBT[Lvv,QAq,q1,q2,13,11],res6];
res6=  If[QDq[[q1]]==2&&QDq[[q2]]==2,
TwoOpBT[Lvv,QAq,q1,q2,12,0]+TwoOpBT[Lvv,QAq,q1,q2,13,11]  ,res6];AF2[res6]   ]



CNOTH1BT[QAq_,q1_,q2_]:=Module[{QDq,Lv  },QDq=QAq  /.{B->2,T->3};Lv=Length[QAq]; 
res9=  If[QDq[[q1]]==3&&QDq[[q2]]==3,
TwoOpBT[Lv,QAq,q1,q2,12,44]+TwoOpBT[Lv,QAq,q1,q2,13,45]+TwoOpBT[Lv,QAq,q1,q2,14,46],res9];
res9=If[QDq[[q1]]==3&&QDq[[q2]]==2,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33]+TwoOpBT[Lv,QAq,q1,q2,14,0],res9];
res9=  If[QDq[[q1]]==2&&QDq[[q2]]==3,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33],res9];
res9=  If[QDq[[q1]]==2&&QDq[[q2]]==2,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33]  ,res9];AF2[res9]   ]



CZ1BT[QAq_,q1_,q2_]:=Module[{QDq,Lv  },QDq=QAq  /.{B->2,T->3};Lv=Length[QAq];  
                                                          res6=  If[QDq[[q1]]==3&&QDq[[q2]]==3,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33]+TwoOpBT[Lv,QAq,q1,q2,14,34],res6];
                                                       res6=If[QDq[[q1]]==3&&QDq[[q2]]==2,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33]+TwoOpBT[Lv,QAq,q1,q2,14,0],res6];
                                                       res6=  If[QDq[[q1]]==2&&QDq[[q2]]==3,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33],res6];
                                                  res6=  If[QDq[[q1]]==2&&QDq[[q2]]==2,
TwoOpBT[Lv,QAq,q1,q2,12,0]+TwoOpBT[Lv,QAq,q1,q2,13,33]  ,res6];AF2[res6]   ]


ToffoliBT[QAq_,q1_,q2_,q3_]:=Module[{QDq ,imax,jmax,iset,Lv},QDq=QAq  /.{B->2,T->3};resTT=0; imax=QDq[[q1]];jmax=QDq[[q2]] ;Lv=Length[QDq];
iset[iT_,jT_]:=If[iT==imax&&jT==jmax,1,0];
  resTT= Sum[ThreeOpBT[Lv,QAq,q1,q2,q3,11+iT,11+jT,iset[iT,jT]],{iT,1,imax},{jT,1,jmax}];
        AF2[resTT]   ];


EnTropy3[O_?MatrixQ]:= Module[{M3,S},
M3 = Chop[Eigenvalues[O]];
S=-M3.Log[M3]/Log[2];
S]


MutualBT[ \[CapitalOmega]9_ /; MatrixQ[\[CapitalOmega]9]] := 
      Module[{rhoAB,rhoA,rhoB,SSAB,SSA,SSB,\[ScriptCapitalI]},
rhoAB=\[CapitalOmega]9;rhoA=PTrBT[QA,{2},rhoAB];rhoB=PTrBT[QA,{1},rhoAB];
SSAB=EnTropy3[rhoAB];SSA=EnTropy3[rhoA];SSB=EnTropy3[rhoB];
\[ScriptCapitalI] = Chop[SSA+SSB-SSAB];{ SSA,SSB,SSAB,\[ScriptCapitalI]}]


EndPackage[ ]
