(* ::Package:: *)

(* Nov 14, 2014   Frank Tabakin *)
BeginPackage["QDENSITY`Circuits`"]


(*   Package for presentation of circuit diagrams
 and also display of states in Dirac form . *)


Circuit::usage=
"Circuit[mat,meters] generates a quantum
 circuit diagram with operators stipulated by the
circuit  matrix 'mat' and the selection of measurements 
at the end.  For example. Circuit[mat,{1,6}] places
meters at end of qubit 1 and 6 lines."

CircuitW::usage=
"CircuitW[mat,meters,wires] generates a quantum
 circuit diagram with operators stipulated by the
circuit  matrix 'mat' and
 the selection of measurements (meters) 
at the end. For example. CircuitW[mat,{1,6},3,4] places
meters at end of qubit 1 and 6 lines and includes only the top
3 and bottom 4 wires."




(* Specify default sizes *)

boxsize0=.1;
boxsize=boxsize0;
fontscale0=0.5*boxsize0;
fontscale=fontscale0;
(* diagramscale *)
ds0=3*boxsize;
ds=ds0;
(* circuitlength *)
clength[matr_]:=3*Map[Length,matr ][[1]]*boxsize
cl0=2;
cl=cl0;

(* control dot size=cds  Default size =cds0  *)
cds0=.4  boxsize0;
cds=cds0

(* control wire thickness=cwt  Default size =cwt0. *)

cwt0=.007;
cwt=cwt0

(*  picture size *)
picsize=500


(*  One Qubit Operators *)

(*   General Form of One Qubit Operators =OQO *)
 
OQO[irow_,icol_,op_]:=Module[{xs,ys},xs=ds*(1-irow);
ys=ds*(1-icol);
Graphics[{
EdgeForm[Black],FaceForm[LightBlue],
Rectangle[{boxsize -ys,-boxsize+xs},{boxsize+2 boxsize -ys,boxsize+xs}],
Text[
Style[op,FontFamily -> "Times",
FontSize->Scaled[fontscale],FontWeight -> "Bold"],{ 2 boxsize-ys,+xs}]}]
];

(*   Following Special cases might be useful *)

HS[irow_,icol_]  :=OQO[irow,icol,"H"];
XS[irow_,icol_]  :=OQO[irow,icol,"X"];
YS[irow_,icol_]  :=OQO[irow,icol,"Y"];
ZS[irow_,icol_]  :=OQO[irow,icol,"Z"];
SMS[irow_,icol_]:=OQO[irow,icol,"\!\(\*SubscriptBox[\"\[Sigma]\", \"-\"]\)"];
SPS[irow_,icol_]:=OQO[irow,icol,"\!\(\*SubscriptBox[\"\[Sigma]\", \"+\"]\)"];
SXS[irow_,icol_]:=OQO[irow,icol,"\!\(\*SubscriptBox[\"\[Sigma]\", \"x\"]\)"];
SYS[irow_,icol_]:=OQO[irow,icol,"\!\(\*SubscriptBox[\"\[Sigma]\", \"y\"]\)"];
SZS[irow_,icol_]:=OQO[irow,icol,"\!\(\*SubscriptBox[\"\[Sigma]\", \"z\"]\)"];
HT[irow_,icol_]  :=OQO[irow,icol,"HT"];
(* ======================================================================== *)


(*  Two Qubit Operators *)
(*   General Form of Two Qubit Control Operators = CNTRG  *)
(*   Located in column icol with control at row iictl, 
and operator located at row iact. *)

CNTRG[icol_,ictl_,iact_,op_]:=Module[
{zs,ys,cs},zs=ds*(1-ictl);ys=ds*(1-icol);cs=(iact-ictl) ds;
Graphics[{
PointSize[cds],Point[ { 2 boxsize-ys,zs}],
Text[Style[op,FontFamily -> "Times",FontSize->Scaled[ fontscale],FontWeight -> "Bold"],
{ 2 boxsize-ys,+zs-cs}],
Thick,Blue,Circle[{2 boxsize-ys,zs-cs},boxsize],
Thick,Black,Line[{{2 boxsize-ys,+zs},{2 boxsize-ys,zs-cs+Sign[cs]*.8*boxsize}}]}] 
 ];

(*   Following Special cases might be useful *)

CNTRX[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"X"];
CNTRY[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"Y"];
CNTRZ[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"Z"];
CNTRSP[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"\[Sigma]\", \"+\"]\)"];
CNTRSM[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"\[Sigma]\", \"-\"]\)"];
CNPHI[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\[CapitalPhi]"];

CNTRR2[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"R\", \"2\"]\)"];
CNTRR3[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"R\", \"3\"]\)"];
CNTRRn[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"R\", \"n\"]\)"];
CNTRRnm1[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"R\", RowBox[{\"n\", \"-\", \"1\"}]]\)"];
CNTRRnm2[icol_,ictl_,iact_]:=CNTRG[icol,ictl,iact,"\!\(\*SubscriptBox[\"R\", RowBox[{\"n\", \"-\", \"2\"}]]\)"];


(*  Following CNOT is for use within Circuit commands *)

CNOTC[icol_,ictl_,iact_,xop_]:=Module[
{zs,ys,cs},zs=ds*(1-ictl);ys=ds*(1-icol);cs=(iact-ictl) ds;
Graphics[{
PointSize[cds],Point[ { 2 boxsize-ys,zs}],
Blue,
Text[Style["\[CirclePlus]",FontFamily -> "Symbol",FontSize -> 34,FontWeight -> "Bold"],
{ 2 boxsize-ys,-cs+zs}],
Thick,Black,Line[{{2 boxsize-ys,+zs},{2 boxsize-ys,-cs+zs}}]}]
];

(*  Following CNOT is for use as a stand alone CNOT gate setup *)

CNOTS[icol_,ictl_,iact_]:=Module[
{zs,ys,cs},zs=ds*(1-ictl);ys=ds*(1-icol);cs=(iact-ictl) ds;
Graphics[{
PointSize[cds],Point[ { 2 boxsize-ys,zs}],
Blue,Inset[Style["\[CirclePlus]",FontFamily -> "Symbol",FontSize -> 34,FontWeight -> "Bold"],{ 2 boxsize-ys,-cs+zs}],
Thick,Black,Line[{{2 boxsize-ys,+zs},{2 boxsize-ys,-cs+zs}}]}]
];

(* ======================================================================== *)


(*  Three Qubit Operators *)

Toff[icol_,ictl_,ict2_,iact_]:=Module[{zs1,zs2,ys,cs},zs1=ds*(1-ictl);zs2=ds*(1-ict2);
ys=ds*(1-icol);cs=(iact-ictl) ds;
Graphics[{
PointSize[cds],Point[ { 2 boxsize-ys,zs1}],Point[ { 2 boxsize-ys,zs2}],
Blue,
Inset[Style["\[CirclePlus]",FontFamily -> "Symbol",FontSize -> 34,FontWeight -> "Bold"],{ 2 boxsize-ys,-cs+zs1}],
Thick,Black,
Line[{{2 boxsize-ys,+zs1},{2 boxsize-ys,-cs+zs1}}],Line[{{2 boxsize-ys,+zs2},{2 boxsize-ys,-cs+zs1}}]}]];

(* ======================================================================== *) 


(*  Wire Grid *)
(*  Following  places a wire in row ir of length cls *)

Wires[ir_,matr_]:=Module[{ cls },cls=clength[matr];
Graphics[{
Thickness[cwt],Black,
Line[{{0,+ds*(1-ir)},{cls+boxsize,+ds*(1-ir)}}]}]];

 (*  Wire Grid  White-Out (WO) wire at location ir,ic *)
(*  Enter WO in Circuit control matrix where white-out is requested *)

WiresWO[ir_,ic_,matr_]:=Module[{ cls ,ys},ys=ds*(1-ic);
Graphics[{Thickness[.01],White,Line[{{  boxsize-ys,+ds*(1-ir)},{1.9 boxsize-ys+boxsize,+ds*(1-ir)}}]}]];

(* Special Wire Grid : Gap in wires starts at tnos and ends at bnos *)

W[matr_]:=Table[Wires[i,matr],{i,1,Length[matr]}];
SWires[matr_,t_,b_]:={Take[W[matr],t],Take[W[matr],-b]}

(* ======================================================================== *)



(*  Insert a Double Bar (DB) for Classical Communication Down *)

DBAR[ictl_,icol_]:=Module[{zs,ys,cs},iact=ictl-1;zs=ds*(1-ictl);
ys=ds*(1-icol);cs=-(iact-ictl) ds;
Graphics[{Black,
Inset[Style["\[CenterDot]",FontFamily -> "Symbol",FontSize ->Scaled[4 fontscale],FontWeight -> "Bold"],
{ 2 boxsize-ys,-.05+zs}],
Thickness[fontscale/3],Black,
Line[{{2 boxsize-ys,+zs-.01},{2 boxsize-ys,-2.6boxsize+zs}}]}]]

(*  Insert a Double Bar (DBU) for Classical Communication Up *)

DBAR2[ictl_,icol_]:=Module[{zs,ys,cs},iact=ictl-1;zs=ds*(1-ictl);ys=ds*(1-icol);
cs=-(iact-ictl) ds;
Graphics[{Black,
Inset[Style["\[CenterDot]",FontFamily -> "Symbol",FontSize ->Scaled[4 fontscale],FontWeight -> "Bold"],
{ 2 boxsize-ys,-.05+zs}],
Thickness[fontscale/3],Black,Line[{{2 boxsize-ys,+zs-.01},{2 boxsize-ys,+2.6  boxsize+zs}}]}]]

(*  Insert a Boxed Message Receiver (BM) for Classical Communication *)


BMR[irow_,icol_]:=Module[{xs,ys},xs=ds*(1-irow);ys=ds*(1-icol);
Graphics[
{Thick,LightRed,Rectangle[{boxsize -ys,-2 boxsize+xs},{boxsize+2 boxsize -ys,boxsize+xs}],
Black,
Text[
Style[" ",FontFamily -> "Times",FontSize->Scaled[0.7 fontscale],FontWeight -> "Bold"],
{2 boxsize -ys,-0.5*boxsize+xs},Automatic,Automatic,{0,1}]}]];
(*   This is for insert of O (M) label *)

BML[irow_,icol_]:=Module[{xs,ys},xs=ds*(1-irow);ys=ds*(1-icol);
Graphics[
{Thick,LightRed,Rectangle[{boxsize -ys,-2 boxsize+xs},{boxsize+2 boxsize -ys,boxsize+xs}],
Black,
Text[
Style["O(M)",FontFamily -> "Times",FontSize->Scaled[0.7 fontscale],FontWeight -> "Bold"],
{2 boxsize -ys,-0.5*boxsize+xs},Automatic,Automatic,{0,1}]}]];

 


(*  Insert a Comment Box *)

WBOX[ir_,ic_,comment_]:=Module[{cls,ys,xs},cls=clength[mat];ys=ds*(1-ic);xs=ds*(1-ir);
    If[comment==G,  comm=" "," "];
If[comment==GG,comm="GROVER"," "];
If[comment==GR,comm="  "," "];
If[comment==GO,comm=" "," "];
If[comment==GV,comm="  "," "];
If[comment==WB00,comm="|Bell>"," "];
If[comment==WB,comm="  "," "];
           If[comment==O,  comm=" "," " ];
       If[comment==OO,comm="ORACLE"," " ];
       If[comment==OR,comm=" "," " ];
       If[comment==OA,comm=" "," " ];
       If[comment==OC,comm=" "," " ];
    If[comment==QQ,comm="QFT"," " ];
    If[comment==QF,comm="  "," " ];
    If[comment==QT,comm="  "," " ];
    If[comment==Q , comm=" "," " ];
If[comment==GHZ,comm="GHZ"," "];
If[comment==GZ,comm="  "," "];
If[comment==GZ,comm=" "," "];
If[comment==GZ,comm="  "," "];
If[comment=="=",comm="||"," " ];
Graphics[{LightOrange,
Opacity[0.9],Rectangle[{  boxsize-ys,- 1 boxsize+xs},{  3 boxsize-ys,2 boxsize+xs}],
Black,
Text[Style[comm,FontFamily -> "Times",FontSize -> Scaled[0.6*boxsize],
FontWeight -> "Bold"],
{   2 boxsize-ys,+3*boxsize+xs},Automatic,Automatic,{0,1}]
}]   ]


BOXGET[matr_,op_]:=Module[{locOp,loc},locOp[i_]:=
Position[matr,op][[i]];loc[i_]:=Append[locOp[i],op];
getB=Table[loc[ib],{ib,1,Length[Position[matr,op]] }] ];

box[matr_]:=Join[BOXGET[matr,QF],BOXGET[matr,QT],BOXGET[matr,QQ],
                                    BOXGET[matr,GR],BOXGET[matr,GO],BOXGET[matr,GV],  BOXGET[matr,GG],  
                                    BOXGET[matr,OR],BOXGET[matr,OA],BOXGET[matr,OC],  BOXGET[matr,OO],
                                    BOXGET[matr,GZ],BOXGET[matr,GHZ], 
                                    BOXGET[matr,WB],BOXGET[matr,WB00]
] 


(* Initial and Final Kets *)

(* Default choice *)
initialKetsq[matr_]:=Table[Row[{"|",Subscript[Style["q",Italic],j],"\[RightAngleBracket]    "}],{j,Length[matr]}];
(*  Another choice--set all initial qubits to zero *)
initialKets0[matr_]:=Table[Row[{"|","0","\[RightAngleBracket]"}],{jin,Length[matr]}];

(*  Another choice--set  initial nqsest qubits to |q>,  rest to zero *)
nqset=1
initialKetsA[matr_]:=Table[If[j\[LessSlantEqual]nqset,A=Subscript[Style["q",Italic],j],
A=Subscript[Style["0",Bold],j]];Row[{"|",A,"\[RightAngleBracket]    "}],{j,Length[matr]}];
(*  Another choice--set  initial nqsest qubits to |\[Psi]>,  rest to zero *)
nqset=1
initialKetsB[matr_]:=Table[If[j\[LessSlantEqual]nqset,A=Subscript[Style["\[Psi]",Bold],j],
A=Subscript[Style["0",Bold],j]];Row[{"|",A,"\[RightAngleBracket]    "}],{j,Length[matr]}];

nqset=1
initialKetsB[matr_]:=Table[If[j\[LessSlantEqual]nqset,A=Subscript[Style["\[Psi]",Bold],j],
A=Subscript[Style["0",Bold],j]];Row[{"|",A,"\[RightAngleBracket]    "}],{j,Length[matr]}];

(*  Another choice--set  initial   qubit nqset  to |\[Psi]> =a|0> + b|1> ,  rest to zero *)
nqset=1
initialKetsC[matr_]:=Table[If[j==nqset,A="|\[Psi] >= a |0\[RightAngleBracket]+ b |1>                              ",
A=Row[{"|",Subscript[Style["0",Bold],j],"\[RightAngleBracket]  "}]];A,{j,Length[matr]}];

(*  Another choice--qutrot set  initial   qutrit nqset  to |\[Psi]> =a|0> + b|1>+ c|2> ,  rest to zero *)
nqset=1
initialKetsD[matr_]:=Table[If[j==nqset,A="|\[Psi]>= a|0\[RightAngleBracket]+b|1>+c|2>                             ",
A=Row[{"|",Subscript[Style["0",Bold],j],"\[RightAngleBracket]  "}]];A,{j,Length[matr]}];
  

(* Set to default *)

initialKets[matr_]:=initialKetsq[matr];

Initstate[irow_,matr_]:=
Graphics[{
Black,
Inset[Style[initialKets[matr][[irow]],FontFamily -> "Times",FontSize->Scaled[fontscale],FontWeight -> "Bold"],
{ -boxsize,+ds*(1-irow)}]}];


initialKetsq[matr_]:=Table[Row[{"|",Subscript[Style["q",Italic],j],"\[RightAngleBracket]    "}],{j,Length[matr]}];
 
(* Use   initialKets[matr_]:=initialKetsq[matr];
  to select this case *)


(* Use space to shift final kets to right *)

space="             " ;
finalKets[matr_]:=
Table[Row[{space,"|",Subscript[Style["b",Italic],jf],"\[RightAngleBracket]"}],{jf,Length[matr]}];

Finalstate[irow_,matr_]:=
Graphics[{
Black,
Inset[Style[finalKets[matr][[irow]],FontFamily ->
 "Times",FontSize->Scaled[fontscale],FontWeight -> "Bold"],
{clength[matr]+3 boxsize ,+ds*(1-irow)}]}];

(* ======================================================================== *)


(*  Measurement Operators *)

meter[irow_,icol_]:=Module[{xs,ys},xs=ds*(1-irow);ys=ds*(1-icol);
Graphics[{
Thick,Red,Circle[{4 boxsize-ys,xs},boxsize,{-90 Degree,90 Degree}] ,
Thick,Red,Line[{{4 boxsize-ys,+xs-boxsize},{4 boxsize-ys,xs+boxsize}}],
Thick,Red,Arrow[{{4 boxsize-ys,xs},{4 boxsize-ys+.1,xs+.1}}],
Dotted,Red,Line[{{3 boxsize-ys,+xs },{4 boxsize-ys,xs }}]
 }]];


(*  Insert a Measurement Box *)

meter2[irow_,icol_]:=Module[{xs,ys},xs=ds*(1-irow);ys=ds*(1-icol);
Graphics[{
Thick,Red,Circle[{2boxsize-ys,xs-.5 boxsize},.7 boxsize,{0 Degree,180 Degree}] ,
Thick,Red,Arrowheads[.15  boxsize],Arrow[{{2 boxsize-ys,xs-.02-.5 boxsize},{2boxsize-ys+.3 boxsize,xs+.75 boxsize}}],
Black,
Opacity[0.2],Rectangle[{  boxsize-ys,-boxsize+xs},{  3 boxsize-ys,boxsize+xs}]
 }]];



(*   General Form of Two Qubit   Operators =TWOG  *)

TWOG[icol_,ictl_,iact_,op1_,op2_]:=Module[{zs1,zs2,ys,cs},zs=ds*(1-ictl);ys=ds*(1-icol);cs=(iact-ictl) ds;
Graphics[{
Inset[Style[op1,FontFamily ->  "Times",FontSize->Scaled[fontscale],FontWeight -> "Bold"],{ 2 boxsize-ys,-.01+zs}],
Black,
Inset[Style[op2,FontFamily -> "Times",FontSize->Scaled[fontscale],FontWeight -> "Bold"],{ 2 boxsize-ys,-cs+zs}],
Thick,Red,
Circle[{2 boxsize-ys,-cs+zs},boxsize],Thick,Red,Circle[{2 boxsize-ys,-.01+zs},boxsize],Thick,Black}]];

(* Single qubit operators-  get locations in mat  and graph all *)
(*  ONEGET determines the location of an operator "'op" in circuit matrix "matr." *)

ONEGET[matrix_,op_]:=Module[{mat,locOp,loc,ig,igg},mat=matrix;
locOp=Position[mat,op];
loc[ig_]:=Append[locOp[[ig]],op];
getone=Table[loc[igg],{igg,1,Length[locOp]}]];

(* get1 lists all one operator locations along with its name *)

get1[matr_]:=Join[ONEGET[matr,H],ONEGET[matr,HT],ONEGET[matr,X],ONEGET[matr,Y],
ONEGET[matr,Z],ONEGET[matr,SA],ONEGET[matr,SM],ONEGET[matr,SX],
ONEGET[matr,SY],ONEGET[matr,SPL],
ONEGET[matr,SZ]]/.{ SM->Subscript["\[Sigma]", "-"],SPL->Subscript["\[Sigma]", "+"],SA->Subscript["\[Sigma]", "+"]
 ,SX->Subscript["\[Sigma]", x] ,SY->Subscript["\[Sigma]", "y"] ,SZ->Subscript["\[Sigma]", "z"]};

 (* Apply OQO@@@get1[matr]  to draw these gates *)




(*  Now do two qubit operators *)
(*  TWOGET determines the location of two qubit operators "'opa" and "opb"
 in circuit matrix "matr." *)

TWOGET[matrix_,opa_,opb_]:=Module[
{matr,locOp1,locOp2,opr1,opr2,Bb,b},matr=matrix;
opr1=opa;opr2=opb;
locOp1=Position[matr,opr1];
locOp2=Position[matr,opr2];
Do[
If[
locOp1[[i]][[2]]==locOp2[[j]][[2]],
b[j]={locOp1[[i]][[2]],locOp1[[i]][[1]],locOp2[[j]][[1]],opr2}," " ],
{i,1,Length[locOp1]},{j,1,Length[locOp2]}];
Bb=Array[b,Length[locOp2]]];

(* get2 lists the control-y and control-z two qubit operator locations along with its name *)
(* get2X lists the control-x or CNOT two qubit operator locations along with its name *)

get2[matr_]:=Join[TWOGET[matr,C,CY],TWOGET[matr,C,CZ]] /.{CY->"Y",CZ->"Z"} ;
get2X[matr_]:=TWOGET[matr,C,CX] ;
get2X2[matr_]:=TWOGET[matr,C2,C2X] ;
get2X3[matr_]:=TWOGET[matr,C3,C3X] ;


(* Apply CNTRG@@@get2[matr] and CNOTC@@@get2X[matr] to draw these gates *)


(*  Now do three qubit operators   In "mat" use "T" to mark two control qubits and "TX" 
 to mark qubit acted on *)

(*  THREEGET determines the location of three qubit operators "'opa" 'opb," and "opc " *)

THREEGET[matrix_,opa_,opb_,opc_]:=Module[{matr,locOp1,locOp2,locOp3,opr1,opr2,opr3,Bb,b},
matr=matrix;
opr1=opa;opr2=opb;opr3=opc;
locOp1=Position[matr,opr1];
locOp2=Position[matr,opr2];
locOp3=Position[matr,opr3];
Do[
If[
locOp1[[i]][[2]]==locOp2[[j]][[2]]==locOp3[[k]][[2]]&&j>i&&locOp1[[i]][[1]] != locOp2[[j]][[1]],
b[k]={locOp1[[i]][[2]],locOp1[[i]][[1]],locOp2[[j]][[1]],locOp3[[k]][[1]]}," " ],
{i,1,Length[locOp1]},{j,1,Length[locOp2]},{k,1,Length[locOp3]}];
Bb=Array[b,Length[locOp3]]];

(* get3 lists the control-control-X or Toffoli three qubit operator locations along with its name *)
 
get3[matr_]:= THREEGET[matr,T1,T1,TX] ;
get32[matr_]:= THREEGET[matr,T2,T2,T2X] ;
get33[matr_]:= THREEGET[matr,T3,T3,T3X] ;


(* Apply Toff@@@get3[matr]  to draw these gates *)




(*   Circuit[mat,meterarray]  produces circuit diagram for  steps specified by matrix="matr"
and includes meters at end on qubits specified bt array {q1,q2...} *)
(* For example  Circuit[mat,{1,5}]  put meters at qubit 1 and 5--see below  *)

Circuit[matr_,pickmeters_]:=Module[{cl,mtrs,nqc,fwo},mtrs=pickmeters;
nqc=Map[Length,matr ][[1]];
cl=3*Map[Length,matr ][[1]]*boxsize;
fwo[ir_,ic_]:=WiresWO[ir,ic,matr];
Style[Show[
Table[Initstate[icc,matr],{icc,1,Length[matr]}],
Table[Wires[icw,matr],{icw,1,Length[matr]}],
fwo@@@Position[matr ,WO],
OQO@@@get1[matr],
CNTRG@@@get2[matr],
CNOTC@@@get2X[matr],
CNOTC@@@get2X2[matr],
CNOTC@@@get2X3[matr],
WBOX@@@box[matr],
Toff@@@get3[matr],
Toff@@@get32[matr],
Toff@@@get33[matr],
Table[Finalstate[ifs,matr],{ifs,1,Length[matr]}],
meter2@@@Position[matr,M],
DBAR@@@Position[matr,DB],
DBAR2@@@Position[matr,DBU],
BMR@@@Position[matr,BM],
BML@@@Position[matr,BML],
Table[meter[mtrs[[ime]],nqc],{ime,1,Length[mtrs]}],Background->White,ImageSize->picsize]]] ;


CircuitW[matr_,pickmeters_,top_,bot_]:=Block[{cl,mtrs,nqc},mtrs=pickmeters;nqc=Map[Length,matr ][[1]];
cl=3*Map[Length,matr ][[1]]*boxsize;
Show[
Table[Initstate[i,matr],{i,1,Length[matr]}],
SWires[matr,top,bot],
OQO@@@get1[matr],
CNTRG@@@get2[matr],
CNOTC@@@get2X[matr],
WBOX@@@box[matr],
Toff@@@get3[matr],
Table[Finalstate[i,matr],{i,1,Length[matr]}],
meter2@@@Position[matr,M],
DBAR@@@Position[matr,DB],
DBAR2@@@Position[matr,DBU],
BMR@@@Position[matr,BM],
BML@@@Position[matr,BML],
Table[meter[mtrs[[i]],nqc],{i,1,Length[mtrs]}],Background->White]]





EQUAL= 
Graphics[{
Text[Style["==",FontFamily -> "Times",FontSize->14,FontWeight -> "Bold"]]
}];


EndPackage[];

