(* ::Package:: *)

(*  July 15,2014 BeginPackage["QCWave`"] *)
BeginPackage["QDENSITY`QCWave`"]

(*  Usage Definitions  *)
DForm::usage=
"DForm[state]
 gives output of state in Dirac form.
Use DFormA for Bra states";

DFormA::usage=
"DFormA[state]
Use DFormA for Bra states.
 Gives output of state in Dirac form.
Use DForm for Ket states";

Partner1::usage=
"Partner1 generates the partners for a one-qubit
 operator-used to do QCMPI-type
 procedure. One qubit operator acts on qubit is.
Called as Partner1[ is ]";

Pick1::usage=
"Pick1  is used to pick out the members of each pair labelled by icase.
icase runs from 1 to 2**nqx/2  and ip1 =1,2.
Pick1[is_,icase_,ip1_]:=Pick1[is,icase,ip1]=Partner1[is][[2*icase-1+ip1-1]].
";

Partner2::usage=
"Partner2 generates the quartet for a two-qubit
 operator-used to do QCMPI-type
 procedure. Two qubit operator acts on qubits is1 and is2,
with is1<is2.
Called as Partner2[ is1,is2 ] ";

Pick2::usage=
"Pick2  is used to pick out the members of each quartet set labelled by 
icase;icase runs from 1 to 2**nqx/4.  iv runs from 1 to 4.
Pick2[is1,is2,icase,iv].
";

Partner3::usage=
"Partner3 generates the octet for a three-qubit
 operator-used to do QCMPI-type
 procedure. Three qubit operator acts on qubits is1,is3 and is3.
Called as Partner3[ is1,is2,is3 ] ";

Pick3::usage=
"Pick3  is used to pick out the members of each octet set labelled by 
icase;icase runs from 1 to 2**nqx/8.  iv runs from 1 to 8.
Pick1[is1,is2,is3,icase,iv]
";



Op1::usage=
"Op1 yields result of a one-body operator \[CapitalOmega] acting 
on qubit is  in state state0;the result is the final state.
Called as: Op1[ \[CapitalOmega],is,state0];
 see also state1f ; Using QCMPI method.
op1C[\[CapitalOmega], is,state] is the compiled version.
An equivalent notation is:
\[CapitalOmega](1) \[VerticalSeparator] psi  >
where \[CapitalOmega](1) is operator \[CapitalOmega] acting on qubit 1.
 ";

Op2::usage=
"Op2 Two-body operator \[CapitalOmega] acts on qubits is1 and is2 
 in state state0;  the result is the final state:
Called as: Op2[\[CapitalOmega], is1,is2,state0];
 see also state2f ; ; Using QCMPI method.
An equivalent notation is:
\[CapitalOmega](1,3) \[VerticalSeparator] psi  >
where \[CapitalOmega](1,3) is operator \[CapitalOmega] acting on qubits 1 and 3.
";

Op3::usage=
"Op3 Three-body operator \[CapitalOmega] acts on qubits is1, is2 and is3 
 in state state0;  the result is the final state:
  Using QCMPI method.
";
 

state1f::usage=
"state1f: Generates final state from initial state0
when one-qubit operator 'oper' acts on qubit 'is.'
Also prints out final state in Dirac form.
Called as:
 state1f[oper,is,state0] ; Using QCMPI method.
";

state2f::usage=
"stat2f: Generates final state from initial state0
when  two-qubit operator 'oper' acts on qubits 'is1' and 'is2.'
Also prints out final state in Dirac form.
Called as:
 state2f[oper,is1,is2,state0] ; Using QCMPI method.
";

\[CapitalOmega]SET::usage=
"\[CapitalOmega]SET[opset,state0]: Generates final state from initial 'state0'
when set of  one-qubit operators 'opset' acts on selected qubits.
For example,for three qubits opset={A,1,T} is equivalent to
'( (A \!\(\*
StyleBox[\"\[CircleTimes]\",\nScriptLevel->0]\)(s[0]) \!\(\*
StyleBox[\"\[CircleTimes]\",\nScriptLevel->0]\) T). state0.'  Other examples are in the tutorial notebook
QCwaveTutorial
";
 
\[CapitalOmega]ALL::usage=
"\[CapitalOmega]ALL[oper,state0]: Generates final state from initial state0
when  one-qubit operator 'oper' acts on all qubits.
Use to generate HALLW, which is equivaleny to HALL[nqx].state0.
";

MeterGraph::usage=
"MeterGraph[wav]: Generates Argand plot for each amplitude in state 'wav'.
";

AmpPlots::usage=
"AmpPlots[wav]: Generates  plots for magnitude and phase of
 each amplitude in state 'wav'.
";




(*  END USAGE DOCUMENTATION  *)
(*  ================================================================= *)


DForm[stateV_] :=
 Module[{nqD,size,ns,plist,stV,coef},
If[VectorQ[stateV],
stV=Flatten[stateV],stV=stateV];
If[VectorQ[stV],
coef[js_]:=stV[[js]],coef[js_]:=stV[[js]][[1]]  ];
nqD=IntegerPart[Log[2,Length[stV]]];
size= Length[stV]; plist={};
Do[
 If[
coef[ns]==0," " ,
AppendTo[plist,{ " + ","(",coef[ns],")"," |",IntegerString[ns-1,2,nqD]," >"}],
AppendTo[plist,{ " + ","(",coef[ns],")"," |",IntegerString[ns-1,2,nqD]," >"}]],
{ns,1,size}];Row[Flatten[plist]  ]];

DFormA[stateV_] :=
 Module[{nqD,js,size,ns,plist,stV,coef}, 
 If[VectorQ[stateV],
stV=Flatten[Transpose[stateV]],stV=Transpose[stateV]];
If[VectorQ[stV],
coef[js_]:=Part[stV,js],coef[js_]:=Part[Part[stV,js],1]];nqD=IntegerPart[Log[2,Length[stV]]];
size= Length[stV]; plist={};
Do[
 If[
coef[ns]==0," " ,
AppendTo[plist,{ " + "," ( ",coef[ns]," ) "," < ",IntegerString[ns-1,2,nqD]," |"}],
AppendTo[plist,{ " + "," ( ",coef[ns]," ) "," < ",IntegerString[ns-1,2,nqD]," |"}]],
{ns,1,size}];Row[Flatten[plist]  ]];


(*
"QCWave`"
*)

(*  This generates the partners for a one-qubit operator-used 
to do QCMPI-type procedure. One qubit operator acts on qubit "is" *)
Partner1[ is_] :=
 Module[{nqs1,jp=is,stride,temp,np,yp1},nqs1=nqx;stride=2^{nqs1-jp};temp={};
Do[If[IntegerDigits[np-1,2,nqs1][[jp]]==0,temp={temp,{np,np+stride[[1]]}},yp1=1]
,{np,1,2^nqs1}];Flatten[temp]];

Pick1[is_,icase_,ip1_]:=Partner1[is][[2*icase-1+ip1-1]];


(*  This generates the quartet for a two-qubit operator-
used to do QCMPI-type procedure. 
Two qubit operator acts on qubits is1 and is2.
is1<is2 *) 
Partner2[ is1_,is2_] :=
 Module[{nqs2=nqx,j1=is1,j2=is2,stride1,stride2,temp,n,y}, 
hit1[n_]:=IntegerDigits[n-1,2,nqs2][[is1]];
hit2[n_]:=IntegerDigits[n-1,2,nqs2][[is2]];
stride1=2^{nqs2-j1};stride2=2^{nqs2-j2};
temp={};
Do[If[hit1[n]==0&&hit2[n]==0,
temp={temp,{{n,n+stride2[[1]],n+stride1[[1]],n+stride1[[1]]+stride2[[1]]}}},
y=1 ],{n,1,2^nqs2}]; Flatten[temp] ];

Pick2[is1_,is2_,icase_,iv_]:=Partner2[is1,is2][[4*icase-3+iv-1]]


(*  This generates the octet for a three-qubit operator-
used to do QCMPI-type procedure. 
Three qubit operator acts on qubits is1, is2 and is3.
is1<is2 *) 
Partner3[ is1_,is2_,is3_] :=
 Module[{nqs2=nqx,j1=is1,j2=is2,j3=is3,stride1,stride2,stride3,temp,n,y}, 
 hit1[n_]:=IntegerDigits[n-1,2,nqs2][[is1]];
 hit2[n_]:=IntegerDigits[n-1,2,nqs2][[is2]];
 hit3[n_]:=IntegerDigits[n-1,2,nqs2][[is3]];
stride1=2^{nqs2-j1};
stride2=2^{nqs2-j2};
stride3=2^{nqs2-j3};
temp={};
Do[If[hit1[n]==0&&hit2[n]==0&&hit3[n]==0,
temp={temp,{{n,n+stride3[[1]],n+stride2[[1]],
n+stride3[[1]]+stride2[[1]],n+stride1[[1]],
n+stride3[[1]]+stride1[[1]],
n+stride2[[1]]+stride1[[1]],
n+stride1[[1]]+stride2[[1]]+stride3[[1]]}}},
y=1 ],{n,1,2^nqs2}]; Flatten[temp] ];

Pick3[is1_,is2_,is3_,icase_,iv3_]:=Partner3[is1,is2,is3][[8*icase-7+iv3-1]] 

(*  One-body operator \[CapitalOmega] acts on qubit "is"  in state "state";
  Op1[ is,\[CapitalOmega],state] ;Using QCMPI method *)

  Op1[\[CapitalOmega]_?MatrixQ, is_,state_] :=
 Module[{wf,temp,iv1,j1,icas,nqu},nqx=Log[2,Length[state]];nqu=nqx;
wf=state;temp=wf;
Do[iv1=Pick1[is,icas,1];j1=Pick1[is,icas,2];
temp[[iv1]]= \[CapitalOmega][[1]][[1]]*wf[[iv1]]+\[CapitalOmega][[1]][[2]]*wf[[j1]];
temp[[j1]]= \[CapitalOmega][[2]][[1]]*wf[[iv1]]+\[CapitalOmega][[2]][[2]]*wf[[j1]],
{icas,1,2^nqu/2}];Partition[temp,1]]; 

(*  op1C  is the compiled version ot Op1
  op1C[\[CapitalOmega], is,state]  *)

op1C=Compile[{{\[CapitalOmega],_Complex,2},{is,_Integer},{state,_Complex,1}}, 
 Op1[\[CapitalOmega], is,state] ,{{ Op1[\[CapitalOmega], is,state],_Complex,1}}];

(*  Two-body operator \[CapitalOmega] acts on qubits "is1" and "is2"  in state "state";  Op2[ is1,is2,\[CapitalOmega],state] ;Using QCMPI method*)
   Op2[\[CapitalOmega]_?MatrixQ, is1_,is2_,state_] :=
 Module[{wf,temp,iv2,j2,k2,m2,icase,nqu},wf=state;temp=wf;
nqx=Log[2,Length[state]];nqu=nqx;
Do[
iv2=Pick2[is1,is2,icase,1];
j2=Pick2[is1,is2,icase,2];
k2=Pick2[is1,is2,icase,3];
m2=Pick2[is1,is2,icase,4];
temp[[iv2]]= \[CapitalOmega][[1]][[1]]*wf[[iv2]]+\[CapitalOmega][[1]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "j2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[1]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "k2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[1]][[4]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "m2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[j2]]= \[CapitalOmega][[2]][[1]]*wf[[iv2]]+\[CapitalOmega][[2]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "j2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[2]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "k2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[2]][[4]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "m2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[k2]]= \[CapitalOmega][[3]][[1]]*wf[[iv2]]+\[CapitalOmega][[3]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "j2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[3]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "k2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[3]][[4]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "m2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[m2]]= \[CapitalOmega][[4]][[1]]*wf[[iv2]]+\[CapitalOmega][[4]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "j2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[4]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "k2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[4]][[4]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "m2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\),
{icase,1,2^nqu/4}]; temp];   

(*  Three-body operator \[CapitalOmega] acts on qubits "is1", "is2" and "is3"  in state "state";  Op3[ is1,is2,is3,\[CapitalOmega],state] ;Using QCMPI method*)
   Op3[\[CapitalOmega]_?MatrixQ, is1_,is2_,is3_,state_] :=
 Module[{wf,temp,iv1,iv2,iv3,iv4,iv5,iv6,iv7,iv8,icase,nqu},
wf=state;temp=wf;nqx=Log[2,Length[state]];nqu=nqx;
Do[
iv1=Pick3[is1,is2,is3,icase,1];iv2=Pick3[is1,is2,is3,icase,2];
iv3=Pick3[is1,is2,is3,icase,3];iv4=Pick3[is1,is2,is3,icase,4];
iv5=Pick3[is1,is2,is3,icase,5];iv6=Pick3[is1,is2,is3,icase,6];
iv7=Pick3[is1,is2,is3,icase,7];iv8=Pick3[is1,is2,is3,icase,8];
temp[[iv1]]= \[CapitalOmega][[1]][[1]]*wf[[iv1]]+\[CapitalOmega][[1]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
           +\[CapitalOmega][[1]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[1]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)           +\[CapitalOmega][[1]][[5]]*wf[[iv5]]+\[CapitalOmega][[1]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
           +\[CapitalOmega][[1]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[1]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv2]]= \[CapitalOmega][[2]][[1]]*wf[[iv1]]+\[CapitalOmega][[2]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
    +       \[CapitalOmega][[2]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[2]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)    +       \[CapitalOmega][[2]][[5]]*wf[[iv5]]+\[CapitalOmega][[2]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
    +       \[CapitalOmega][[2]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[2]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv3]]= \[CapitalOmega][[3]][[1]]*wf[[iv1]]+\[CapitalOmega][[3]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
    +       \[CapitalOmega][[3]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[3]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)    +       \[CapitalOmega][[3]][[5]]*wf[[iv5]]+\[CapitalOmega][[3]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
    +       \[CapitalOmega][[3]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[3]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv4]]= \[CapitalOmega][[4]][[1]]*wf[[iv1]]+\[CapitalOmega][[4]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
    +       \[CapitalOmega][[4]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[4]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)    +       \[CapitalOmega][[4]][[5]]*wf[[iv5]]+\[CapitalOmega][[4]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
    +       \[CapitalOmega][[4]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[4]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv5]]= \[CapitalOmega][[5]][[1]]*wf[[iv1]]+\[CapitalOmega][[5]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +         \[CapitalOmega][[5]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[5]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\) +         \[CapitalOmega][[5]][[5]]*wf[[iv5]]+\[CapitalOmega][[5]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +         \[CapitalOmega][[5]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[5]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv6]]= \[CapitalOmega][[6]][[1]]*wf[[iv1]]+\[CapitalOmega][[6]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +       \[CapitalOmega][[6]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[6]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)  +       \[CapitalOmega][[6]][[5]]*wf[[iv5]]+\[CapitalOmega][[6]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +       \[CapitalOmega][[6]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[6]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv7]]= \[CapitalOmega][[7]][[1]]*wf[[iv1]]+\[CapitalOmega][[7]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +       \[CapitalOmega][[7]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[7]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)  +       \[CapitalOmega][[7]][[5]]*wf[[iv5]]+\[CapitalOmega][[7]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +       \[CapitalOmega][[7]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[7]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\);
temp[[iv8]]= \[CapitalOmega][[8]][[1]]*wf[[iv1]]+\[CapitalOmega][[8]][[2]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv2", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +       \[CapitalOmega][[8]][[3]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv3", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[8]][[4]]*\!\(\*
TagBox[
RowBox[{
RowBox[{"wf", "[", 
RowBox[{"[", "iv4", "]"}], "]"}], " "}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)  +       \[CapitalOmega][[8]][[5]]*wf[[iv5]]+\[CapitalOmega][[8]][[6]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv6", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)
  +       \[CapitalOmega][[8]][[7]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv7", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\)+\[CapitalOmega][[8]][[8]]*\!\(\*
TagBox[
RowBox[{"wf", "[", 
RowBox[{"[", "iv8", "]"}], "]"}],
Function[BoxForm`e$, MatrixForm[BoxForm`e$]]]\),
{icase,1,2^nqu/8}];Partition[temp,1]];  

state1f[oper_,is_,state_]:=Module[{psif},
psif=Op1[oper,is,state];Print[DForm[Flatten[psif]]];psif];

state2f[oper2_,is1_,is2_,statev_]:=
Module[{psif},
psif=Op2[oper2,is1,is2,statev];Print[DForm[Flatten[psif]]];psif];

\[CapitalOmega]ALL[opz_,state_]:=\[CapitalOmega]ALL[opz,state]=
Flatten[Fold[Op1[opz,#2,#1]&,state,  Range[1,Log[2,Length[state]]]], Log[2,Length[state]]];

\[CapitalOmega]SET[opset_,state_]:=\[CapitalOmega]SET[opset,state]=
Fold[Op1[opset[[#2]],#2, #1]&,state,  Range[1,Log[2,Length[state]] ] ];

MeterGraph[wav_]:=Module[{nsz,npartdis,nimage},nsz=Length[wav];If[nsz>7,npartdis=8,npartdis=nsz];
If[nsz>7,nimage=1,nimage=2];pcase[ks_]:=Graphics[{{
{Blue,Thick,Arrowheads[.07],Arrow[{{0,0},{Re[wav[[ks]]],Im[wav[[ks]]]}}]},
Red,Dashing[{.04}],Circle[{0,0},1.]
}}];
GraphicsGrid[Partition[Table[pcase[kt],{kt,1,nsz}],npartdis],ImageSize->1000/nimage]];

AmpPlots[wav_]:=Module[{nsz,p1,p2},nsz=Length[wav];
 phase[kv_]:=Chop[ArcTan[.000000001+Re[wav[[kv]]],Im[wav[[kv]]]]];
p1:=DiscretePlot[Abs[wav[[k1]]],{k1,1,nsz},
FillingStyle->Thickness[0.01],PlotLabel->"Amplitudes-Magnitude",Joined->False];
If[Flatten[Im[wav]]==Table[0,{isz,1,nsz}],
p2:=DiscretePlot[phase[k2],{k2,1,nsz},
FillingStyle->Thickness[0.01],PlotLabel->"  Phases",Joined->False],
p2:=DiscretePlot[phase[k2],{k2,1,nsz},
FillingStyle->Thickness[0.01],PlotLabel->"Phases",Joined->False]];
GraphicsGrid[{{p1},{p2}},ImageSize->750]];




EndPackage[];
