(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    167771,       4556]
NotebookOptionsPosition[    151683,       4036]
NotebookOutlinePosition[    160542,       4304]
CellTagsIndexPosition[    160223,       4289]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Quantum Fourier Transform", "Title",
 CellTags->"c:1"],

Cell["Implementation of the QFT", "Subtitle"],

Cell[TextData[{
 "Version:November-2014--For ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 9.0-10.3"
}], "Abstract",
 CellChangeTimes->{{3.395090887981901*^9, 3.395090925377634*^9}, {
   3.512204933064251*^9, 3.5122049705968847`*^9}, 3.5122120582929077`*^9, {
   3.625184952631393*^9, 3.625184969493586*^9}, {3.6554033899441137`*^9, 
   3.655403394914819*^9}}],

Cell[TextData[ButtonBox["Click here to get the 2014 palette: QDENSpalette.nb",
 BaseStyle->"Hyperlink",
 ButtonData:>{"QDENSpalette14.nb", None}]], "Text"],

Cell[CellGroupData[{

Cell["\<\
Quantum Fourier Transform                                                     \
                        To open double click at down \
arrow---------------------------->>\
\>", "Section",
 CellChangeTimes->{{3.512204977901103*^9, 3.512205004434519*^9}},
 FontFamily->"Times",
 CellTags->"c:2"],

Cell[CellGroupData[{

Cell["a) Quantum Fourier Transform using vectors", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:3", "b:2.0.0"}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[GridBox[{
     {
      RowBox[{"References", ":"}], 
      RowBox[{
       RowBox[{
        RowBox[{"[", "1", "]"}], " ", "Quantum", " ", "Computation", " ", 
        "and", " ", "Quantum", " ", "Information"}], ",", " ", 
       RowBox[{"Nielsen", " ", "and", " ", "Chuang"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
      "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], TraditionalForm]], 
  "DefinitionBox",
  GridBoxOptions->{
  GridBoxDividers->{
   "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
    "RowsIndexed" -> {}},
  GridBoxSpacings->{"Columns" -> {
      Offset[0.27999999999999997`], {
       Offset[0.5599999999999999]}, 
      Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
      Offset[0.2], {
       Offset[0.16]}, 
      Offset[0.2]}, "RowsIndexed" -> {}}},
  CellTags->"S5.1.1"],
 "\nLast Version: 25-Nov-2005"
}], "Abstract",
 CellTags->"b:2.0.0"],

Cell["\<\
The Quantum Fourier Transform is equivalent to the Discrete Fourier Transform 
We present here two ways of writing down algorithms that perform the QFT; the \
first one deals with state vectors, while the second one uses the density \
matrix approach. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:2.0.0"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{3.616263653001302*^9, 3.624968783089155*^9, 
  3.6251849777319403`*^9, 3.625185564170611*^9, 3.655403399506936*^9, 
  3.657377222693737*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["QDENSITY`Qdensity`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["QDENSITY`Qdensity`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["about",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "about", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Discord",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Discord", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mc$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "mc$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBra1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomBra1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Sigma1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ws",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ws", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Adj",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Adj", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["EnTropy",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "EnTropy", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MeasureKet",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "MeasureKet", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBra2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomBra2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Sigma2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["xb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "xb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["AF",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "AF", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["EnTropy2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "EnTropy2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MultiProjCBasis",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "MultiProjCBasis", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBraN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomBraN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma3",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Sigma3", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Xb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Xb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["at",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "at", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Fidelity",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Fidelity", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MultiQProj",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "MultiQProj", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKet1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomKet1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Snos",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Snos", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["xd",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "xd", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Bra1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Bra1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["FidelityB",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "FidelityB", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Mutual",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Mutual", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKet2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomKet2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["SP",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "SP", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Xd",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Xd", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "BraV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["for",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "for", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "nb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKetN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomKetN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Spin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Spin", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ZeroMatrix",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ZeroMatrix", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "BraX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["had",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "had", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["neg",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "neg", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubit1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomQubit1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["SpinObs",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "SpinObs", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ZeroT",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ZeroT", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "BraY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Had",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Had", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nneg",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "nneg", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubit2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomQubit2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["stA",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "stA", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[ScriptCapitalH]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CNOT",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "CNOT", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Hadamard",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Hadamard", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nq",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "nq", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubitN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RandomQubitN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["stB",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "stB", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[ScriptCapitalP]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Coeff",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Coeff", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["HALL",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "HALL", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Observables",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Observables", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["rcsDate",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "rcsDate", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Swap",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Swap", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[DoubleStruckCapitalI]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Comm",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Comm", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["i8",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "i8", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["OneOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "OneOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["rcsRevision",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "rcsRevision", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["s$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "s$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Rho]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Rho]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CompMeasureKet",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "CompMeasureKet", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["intro",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "intro", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Oracle",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Oracle", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "res1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["TensorProductQ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "TensorProductQ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Rho]$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Rho]$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CompPTr",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "CompPTr", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["j8",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "j8", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTrace",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PartialTrace", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res1$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "res1$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ThreeOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ThreeOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Sigma]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Sigma]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ControlledX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Ket1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Ket1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTraceGen",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PartialTraceGen", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "res2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["to",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "to", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]A1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Phi]A1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ControlledY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "KetV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTranspose",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PartialTranspose", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res2$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "res2$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Toffoli",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Toffoli", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]A2",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Phi]A2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledZ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ControlledZ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "KetX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ProjG",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ProjG", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Rotqbit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Rotqbit", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ts",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ts", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]B1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Phi]B1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CPHASE",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "CPHASE", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "KetY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PTr",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PTr", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RotX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ts$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "ts$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]B2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[Phi]B2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CPHASEV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "CPHASEV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["look",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "look", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PTrGen",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PTrGen", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RotY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["TwoOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "TwoOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[CapitalOmega]s",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "\[CapitalOmega]s", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["cRQN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "cRQN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["matrices",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "matrices", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Purity",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Purity", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotZ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "RotZ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["U",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "U", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["Definef",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Definef", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mc",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "mc", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["s",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "s", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["v",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "v", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["density",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "density", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mcl",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "mcl", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "PY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Schmidt",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Schmidt", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Vs",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Vs", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["DForm2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "DForm2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mcl$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "mcl$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["qubit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "qubit", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma0",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "Sigma0", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["v$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-259546c1-2103-4fc0-bcb3-332fe4832060", {
            "v$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.15833333333333333`]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.657377223292841*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"figure", "=", 
   RowBox[{"Import", "[", 
    RowBox[{"\"\<QDENSITY/drawings/QFT.png\>\"", ",", "\"\<PNG\>\""}], 
    "]"}]}], ";"}], "\n", 
 RowBox[{"Show", "[", 
  RowBox[{"figure", ",", 
   RowBox[{"ImageSize", "\[Rule]", "900"}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[RasterBox[CompressedData["
1:eJztnT2O4zYUx4kgRVp1aaMrKMW0CVRsvQsD2wW7QIzpgmwCCHsDX0FX8BXU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    "], {{0, 0}, {564, 95}}, {0, 255},
    ColorFunction->GrayLevel],
   ImageSize->900,
   PlotRange->{{0, 564}, {0, 95}}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699724847*^9, 3.615987582471958*^9, 
  3.616263653151511*^9, 3.624968783266714*^9, 3.625184977994959*^9, 
  3.655403400467297*^9, 3.657377223372933*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
The quantum Fourier transform of a vector written in its binary \
representation has a very nice expression (we follow the notation of Ref [1]) \
:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 FormBox[GridBox[{
    {Cell[BoxData[
      FormBox[
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", 
         RowBox[{
          SubscriptBox["j", "1"], ",", 
          SubscriptBox["j", "2"], ",", "..", ",", 
          SubscriptBox["j", "n"]}], "]"}], "=", " "}], TraditionalForm]]], 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"|", "0"}]}], "\[RightAngleBracket]"}], "+", 
                    SuperscriptBox["e", 
                    RowBox[{"\[ImaginaryI]2\[Pi]0", ".", 
                    SubscriptBox["j", "n"]}]]}], "|", "1"}], 
                   "\[RightAngleBracket]"}], " ", ")"}], "\[CircleTimes]", 
                 RowBox[{
                  RowBox[{"(", " ", 
                   RowBox[{"|", "0"}]}], "\[RightAngleBracket]"}]}], "+", 
                SuperscriptBox["e", 
                 RowBox[{
                  RowBox[{"\[ImaginaryI]2\[Pi]0", ".", 
                   SubscriptBox["j", 
                    RowBox[{"n", "-", "1"}]]}], 
                  SubscriptBox["j", "n"]}]]}], "|", "1"}], 
              "\[RightAngleBracket]"}], " ", ")"}], "..."}], "\[CircleTimes]", 
           RowBox[{
            RowBox[{"(", " ", 
             RowBox[{"|", "0"}]}], "\[RightAngleBracket]"}]}], "+", 
          SuperscriptBox["e", 
           RowBox[{
            RowBox[{
             RowBox[{"\[ImaginaryI]2\[Pi]0", ".", 
              SubscriptBox["j", "1"]}], ".."}], 
            SubscriptBox["j", "n"]}]]}], "|", "1"}], "\[RightAngleBracket]"}],
        ")"}], "\n", "\n", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"0.", 
          SubscriptBox["j", "m"]}], ".."}], 
        SubscriptBox["j", "L"]}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           SubscriptBox["j", "m"], "/", "2"}], "+", 
          RowBox[{
           SubscriptBox["j", 
            RowBox[{"m", "+", "1"}]], "/", 
           SuperscriptBox["2", "2"]}], " ", "+"}], " ", "..."}], "+", 
        RowBox[{
         SubscriptBox["j", "L"], "/", 
         SuperscriptBox["2", 
          RowBox[{"L", "-", "m", "+", "1"}]]}]}]}]}]}
   },
   GridBoxAlignment->{
    "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
     "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], 
  TraditionalForm]], "DefinitionBox",
 GridBoxOptions->{GridBoxSpacings->{"Columns" -> {
     Offset[0.27999999999999997`], {
      Offset[0.5599999999999999]}, 
     Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
     Offset[0.2], {
      Offset[0.16]}, 
     Offset[0.2]}, "RowsIndexed" -> {}}},
 CellTags->{"S5.1.1", "b:2.0.0"}],

Cell[TextData[{
 "\nThe binary representation of the base vectors is:",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"|", 
      SubscriptBox["j", "1"]}],
     TraditionalForm], ",", 
    SubscriptBox["j", "2"]}], TraditionalForm]]],
 ", ..., ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "N"], TraditionalForm]]],
 "\[RightAngleBracket].  A general vector maybe decomposed as:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[TextData[{
 " |\[Psi]\[RightAngleBracket] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Sum]", " "}], TraditionalForm]]],
 "[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 "  * KetV[ IntegerDigits[k,2,N]] , {k,1,2^N}]\nThe function \
IntegerDigits[k,2,N] returns the number k in base 2 with N figures, which is \
the binary representation of the vector k."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[TextData[{
 "Let's ",
 StyleBox["start ",
  FontWeight->"Bold"],
 "by defining the operation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0.", 
    SubscriptBox["j", "m"]}], TraditionalForm]]],
 " ... ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "L"], TraditionalForm]]],
 ", which enters in the exponents of the Fourier Transform, we call it, \
Zeroff:\nThe symbol  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0.", 
    SubscriptBox["j", "m"]}], TraditionalForm]]],
 " ... ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "L"], TraditionalForm]]],
 " denotes the series:     ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "m"], TraditionalForm]]],
 " /  2  + ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", 
    RowBox[{"m", "+", "1"}]], TraditionalForm]]],
 "/ 4 +  ......+ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "L"], TraditionalForm]]],
 " / ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    RowBox[{"L", "-", "m", "+", "1"}]], TraditionalForm]]],
 "= Zeroff.\nThe notation used is that vac is the vector in binary notation \
that we are going to Fourier Transform and the array R  stores the indices m \
to j ."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Zeroff", "[", 
   RowBox[{"vac_", ",", "R_"}], "]"}], ":=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vac", "[", 
      RowBox[{"[", 
       RowBox[{"R", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "/", 
     RowBox[{"2", "^", "i"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "R", "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"b:2.0.0"],

Cell["\<\
Then the Quantum Fourier Transform can be readily written  using tensor \
products of Kets:  (and its inverse);  this corresponds to Eq. 5.4 in Ref.[1].\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"QuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"Zeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[TensorProduct]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.61417552792661*^9, 3.6141755367555523`*^9}},
 CellTags->"b:2.0.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"IQuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"Zeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{
            RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[TensorProduct]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", 
       RowBox[{"\[CapitalOmega]", "/", 
        RowBox[{"2", "^", 
         RowBox[{"(", 
          RowBox[{"L", "/", "2"}], ")"}]}]}]}]}], "]"}]}], ";"}], 
  "  "}]], "Input",
 CellChangeTimes->{{3.614175459439846*^9, 3.6141754763796263`*^9}},
 CellTags->"b:2.0.0"],

Cell[TextData[{
 "For example, let us compute the QFT of state vector  | 0,1,1 \
\[RightAngleBracket]  ( In the full computational base, this vector would \
correspond to  the matrix (00010000) that is since binary 011 is decimal 3,\n\
the matrix representing the state has a 1 in location four.\n NOTE that \
|0,0,1\[RightAngleBracket]+|1,0,0\[RightAngleBracket] \[NotEqual] |1,0,1\
\[RightAngleBracket] !!  ) \n Now consider the QFT of the state state vector  \
| 0,1,1 \[RightAngleBracket]  which is of length ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "3"], TraditionalForm]]]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QFT011", "=", 
  RowBox[{"QuantumFourierTransformV", "[", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], "]"}]}]], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox[
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"3", " ", "\[ImaginaryI]", " ", "\[Pi]"}], "4"]], 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      RowBox[{"-", 
       FractionBox["\[ImaginaryI]", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      RowBox[{"-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
         SuperscriptBox["\[ExponentialE]", 
          FractionBox[
           RowBox[{"3", " ", "\[ImaginaryI]", " ", "\[Pi]"}], "4"]]}], 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         FractionBox[
          RowBox[{"3", " ", "\[ImaginaryI]", " ", "\[Pi]"}], "4"]], 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      FractionBox["\[ImaginaryI]", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[ExponentialE]", 
         FractionBox[
          RowBox[{"3", " ", "\[ImaginaryI]", " ", "\[Pi]"}], "4"]]}], 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6141756997525253`*^9, 3.615987582507421*^9, 
  3.61626365319088*^9, 3.624968783306286*^9, 3.625184978072769*^9, 
  3.6554034005565853`*^9, 3.657377223434729*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell[TextData[{
 "As can be seen the output is a list of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "N"], TraditionalForm]]],
 " numbers, where N is the dimension of our vector in the binary \
representation."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell["We can check whether it is normalized:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Norm", "[", "QFT011", "]"}]], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699782647*^9, 3.6159875825199223`*^9, 
  3.61626365320549*^9, 3.624968783337113*^9, 3.6251849782019873`*^9, 
  3.655403400621978*^9, 3.657377223457348*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "A ",
 StyleBox["more general random vector (with N=4)",
  FontWeight->"Bold"],
 " could be:"
}], "Subsubsection",
 FontFamily->"Helvetica",
 CellTags->{"c:4", "b:2.0.0"}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"RanVector", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"RandomReal", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", "2"}], "}"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       SuperscriptBox["2", "4"]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"RanVector", "=", 
  FractionBox["RanVector", 
   RowBox[{"Norm", "[", "RanVector", "]"}]]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"RanVector", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0.176381589791502`"},
      {"0.15843805058869076`"},
      {"0.1891623411653159`"},
      {"0.0734591284687692`"},
      {"0.14131976985597092`"},
      {"0.2499995992164066`"},
      {"0.180986044491715`"},
      {"0.3263080513670856`"},
      {"0.38303862528833765`"},
      {"0.2868737757819492`"},
      {"0.25399843575196074`"},
      {"0.009974872289780652`"},
      {"0.3004339309505211`"},
      {"0.35698140807512124`"},
      {"0.028665193038914894`"},
      {"0.4107998215482136`"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699793559*^9, 3.6159875825405607`*^9, 
  3.616263653241548*^9, 3.62496878337036*^9, 3.6251849782160387`*^9, 
  3.655403400671817*^9, 3.657377223474161*^9},
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.012833333333333334`], AbsoluteThickness[1.6], 
      LineBox[{{1., 0.176381589791502}, {2., 0.15843805058869076`}, {3., 
       0.1891623411653159}, {4., 0.0734591284687692}, {5., 
       0.14131976985597092`}, {6., 0.2499995992164066}, {7., 
       0.180986044491715}, {8., 0.3263080513670856}, {9., 
       0.38303862528833765`}, {10., 0.2868737757819492}, {11., 
       0.25399843575196074`}, {12., 0.009974872289780652}, {13., 
       0.3004339309505211}, {14., 0.35698140807512124`}, {15., 
       0.028665193038914894`}, {16., 0.4107998215482136}}]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 16.}, {0, 0.4107998215482136}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699793559*^9, 3.6159875825405607`*^9, 
  3.616263653241548*^9, 3.62496878337036*^9, 3.6251849782160387`*^9, 
  3.655403400671817*^9, 3.657377223541786*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
we apply the quantum fourier transform: (NOTE we compute the QFT of the base \
vectors)\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QFTVec", "=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RanVector", "[", 
      RowBox[{"[", 
       RowBox[{"i", "+", "1"}], "]"}], "]"}], 
     RowBox[{"QuantumFourierTransformV", "[", 
      RowBox[{"IntegerDigits", "[", 
       RowBox[{"i", ",", "2", ",", "4"}], "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", 
      RowBox[{
       RowBox[{"2", "^", "4"}], "-", "1"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.8817051594175638`"},
     {
      RowBox[{
       RowBox[{"-", "0.08389364159149837`"}], "-", 
       RowBox[{"0.05473085491848448`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.11639159195545475`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.08575626560947341`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.05685265459559845`"}], "+", 
       RowBox[{"0.010225415949684138`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.08709047535960629`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.057937739997079685`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.030300711410307022`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.10026219701284833`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.057558334818780865`"}], "-", 
       RowBox[{"0.2025110353027968`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.09621145072004583`"}], "-", 
       RowBox[{"0.006104306786466743`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"-", "0.054712194250444664`"}]},
     {
      RowBox[{
       RowBox[{"-", "0.09621145072004579`"}], "+", 
       RowBox[{"0.006104306786466729`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.057558334818780914`"}], "+", 
       RowBox[{"0.2025110353027968`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.030300711410306998`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.10026219701284836`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.08709047535960629`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.057937739997079685`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.05685265459559852`"}], "-", 
       RowBox[{"0.010225415949684138`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.1163915919554547`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.08575626560947343`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.08389364159149841`"}], "+", 
       RowBox[{"0.05473085491848445`", " ", "\[ImaginaryI]"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699851631*^9, 3.6159875826082277`*^9, 
  3.616263653376255*^9, 3.6249687834250183`*^9, 3.625184978314081*^9, 
  3.655403400856141*^9, 3.657377223573956*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
An we can then use the inverse and compare with the original vector:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"IQFTVec", "=", 
  RowBox[{"Chop", "[", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"QFTVec", "[", 
        RowBox[{"[", 
         RowBox[{"i", "+", "1"}], "]"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], "*", 
      RowBox[{"IQuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "4"}], "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "4"}], "-", "1"}]}], "}"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"IQFTVec", "=", 
   RowBox[{"Flatten", "[", "IQFTVec", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Chop", "[", 
  RowBox[{"RanVector", "-", "IQFTVec"}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.176381589791502`"},
     {"0.15843805058869076`"},
     {"0.1891623411653159`"},
     {"0.07345912846876917`"},
     {"0.14131976985597092`"},
     {"0.24999959921640658`"},
     {"0.18098604449171501`"},
     {"0.32630805136708557`"},
     {"0.3830386252883377`"},
     {"0.2868737757819492`"},
     {"0.25399843575196074`"},
     {"0.00997487228978064`"},
     {"0.3004339309505211`"},
     {"0.35698140807512124`"},
     {"0.028665193038914877`"},
     {"0.4107998215482136`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699862845*^9, 3.61598758262361*^9, 
  3.616263653518167*^9, 3.6249687834527073`*^9, 3.625184978328233*^9, 
  3.655403401203623*^9, 3.657377223607676*^9},
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699862845*^9, 3.61598758262361*^9, 
  3.616263653518167*^9, 3.6249687834527073`*^9, 3.625184978328233*^9, 
  3.655403401203623*^9, 3.657377223611753*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
The  inverse of the QFT and  the original vector prove to be the same, which \
is a check on the procedures.\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QFT of the Sin[x]/x function", "Subsubsection",
 FontFamily->"Helvetica",
 CellTags->{"c:5", "b:2.0.0"}],

Cell["\<\
In this example, we construct our vector out of a known function, Sin[x/a]/x. \
We make use of 6 qubits. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SinVector", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     FractionBox[
      RowBox[{"Sin", "[", 
       FractionBox[
        RowBox[{"i", " ", "4", " ", "\[Pi]"}], 
        SuperscriptBox["2", "6"]], "]"}], "i"], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       SuperscriptBox["2", "6"]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"SinVector", "=", 
   FractionBox["SinVector", 
    RowBox[{"Norm", "[", "SinVector", "]"}]]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"SinVector", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Sin[Number]\>\""}], "}"}]}]}], 
  "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd0n0s1HEcwPFftanEEnIejkO4kkiOcHfuwznn3KG72mhrdmP0NGVK9HD8
ijw02bXdOTkPl6e7LhmyGhVSWDQqp9qU9TBMD5TKH2aVdh9t+m6//fb6vb/7
fvf97ueSeFySvJogCFh6/r6N4ytnsaL20kzRSw5+gLCpJi93m2Wvg5S4NSLX
+BdoCzBTx9K4+0bR1tAtHKjPsjCgbWHswFk7J8VzNBUcs3P0vVNP0c4w4U1Y
LWwbQrvCBbvA9RTRANoNgln94jhhH9oDynYMf89j96C3wt0h65z9affQnpBe
rdHRNG1oL2j/0K85kahHe4N56BHqdYer6J1ArBi+UJNhaZdJL+0yehcEZ6ml
py7Wof3gzpOx5mNBjWgG1G5uaPW3aEH7ww/mROVbh1voAEjqgS+y8Db0brCl
TatUimUHwsxjvdmiw7KDIMEybcR+thUdDOODAdu5HsvrM8F0eC6qtP0mmgW9
o5tui6Q6NBv6VRuOTslq0CHQOO+spWvK0BzgF7GtxKJCNKw8PwlwWCL4tHck
H+8nFFYxmHGyDqXRZCi0tEdLHtRVYQ8DGSPpt/xnDfYln5x1+dZcj50LbLkr
syNTi50L4hSzBlaEDns45LkTrxUuaDIcpD7nsuwpOJ/gQTGP5ihn4nokD6QS
/TuqHvcjIqAyhGfjZ1KNPQJsmU0fT4+qsPOBkmDad3CjHDsfnPKVlIcmJPbI
/84fCYX+qYbyzmy8HwEcKqEnLhZfNpoUgBvDcli7Vok9CgydzRUSKMceBbmp
99d5CqqwC0H8iLGgy9VgFwL9ffKZjIBr2EWQwBFMlrDQpAh4Xee1r9Q4n4iG
+BuTz3y7cT0yGhS/6HNtDDX2GPBJf1PG78b/lYyBzwXkgI3yCvZY2FIxXzho
KMAeC77m1HHFdCb2Pf/O/wdS3hSV
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Sin[Number]\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.08039233645206101, 0.3677936891458129}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175699887211*^9, 3.6159875826574306`*^9, 
  3.6162636535657473`*^9, 3.624968783488043*^9, 3.625184978363805*^9, 
  3.655403401314183*^9, 3.6573772236413*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
we apply the quantum Fourier transform: (NOTE we compute the QFT of the base \
vectors)\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTSin", "=", 
   RowBox[{"N", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], 
      RowBox[{
       SuperscriptBox["2", "6"], "-", "1"}]], 
     RowBox[{
      RowBox[{"SinVector", "\[LeftDoubleBracket]", 
       RowBox[{"i", "+", "1"}], "\[RightDoubleBracket]"}], " ", 
      RowBox[{"QuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "6"}], "]"}], "]"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Chop", "[", "QFTSin", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"QFTSinplotRe", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Re", "[", 
     RowBox[{"N", "[", "QFTSin", "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"QFTSinplotIM", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Im", "[", "QFTSin", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "QFTSinplotRe", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Real part of QFT\>\""}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.05`"}], ",", "0.1`"}], "}"}]}]}], "]"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "QFTSinplotIM", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Imaginary part QFT\>\""}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.2`"}], ",", "0.2`"}], "}"}]}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGCwBWIQLereyPXvGK/DrJkgsNOeAQwEHOTPpbTP8V8O5Ys4
iJ3zbJz4bgGUL+GQ0d67x7RxLpQv4+Czwsg/J2c2lK/gYP3ER6DBYRaUr+RQ
5FLz8VDrTChfxWGS4hRPE04YX83hAg9P99WNM6B8DYc/qQ9NVYtgfC2HgIUd
vF5OML6Ow42ufX0FsjC+nsOGOo+jtQwwvoFDwqqIy3Evp0P5hg4CdzzYl12D
8Y0c/GRsDgYch/GNHS4I/Pi4eBeMb+LwLdlklegGGN/UwX++a4PdchjfzIGT
6+yF8/NhfHMHGdGV4r0zYXwLh4LDR/zcp8D4lg4t662W3+yH8a0c5Lb/1FPv
gfGtHZyjFKUlO2F8G4cQRZ/2uW0wvq3DiTPVnctaYHw7hyMlk1o0m2F8e4cH
C9VXyTXB+A4OYsI+CxsaofwGB4ffW0y8fWB8BkcHARR5R4dHKPqdUM1vcHI4
hWK/M6r7GpwdHFDc74LqvwYXhzoU/7s6FCKHT4OrgzxK+Lk5sCCHb4MbWvi7
O3xBjp8Gd4dzKPHn4eCEHL8NHmjx7+kQhZw+Gjwd1qCkHy/U9NXg5eCHkv68
Hf4hp88Gb4c7KOnXx6EPOX03+DhkoKR/XwdD5PzR4Otgg5J//FDzV4MfWv7z
d5CA5s/+1+HyE5T94fkXAMqKDAs=
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Real part of QFT\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.05, 0.1}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700051386*^9, 3.615987582828485*^9, 
  3.6162636537448063`*^9, 3.624968783654296*^9, 3.625184978523404*^9, 
  3.6554034015417147`*^9, 3.657377223813603*^9},
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], 
      LineBox[{{1., 0.}, {2., 0.09447748651390223}, {2.3365206931086835`, 
       0.2}}], LineBox[CompressedData["
1:eJxVk11IU2EYxw+jIBTpg4rAjE0rRsRY2nRWtv/m2dTNj2nguoguFkql0odG
IyIHkyEUK5FMTbBcgTVBQVsF6eiDVQhBESRG5IUfjbqwiBFdlT3/c9ELh8OP
H+e87/s8/8fgP1XXoFMUpeTvs/I+Fus4M+hdi/6+lfXapvxb63Br6NtyXvZL
8ka8d636Pvo8Qd6ChZupobbjj8hbYRjI+HDl7ThZD0OBLlITHSPnYtI8dejx
tRHydjQXfd3gSw2Td+JH79G48fddshGuWNuMKxIl74IubOz1hW6TdyMv4jyc
2TFINuHF/a6WZ60DZDPGNgfr05N95D048SW1fs57g5yPxtX24vS26+QCPLRN
1MZzusl70dNav/RO7SJbMJfIN6nZV8mFeJXVtNi8cJlchHA066PxcyfZikD7
9NO0EiYXAyPdEwE1RN6H2Uu/fJY7QfJ+PEla1TeFF8kHcKQl51PMc55cgoDZ
op5uP0s+iE7H9IWGniayDfpM045NtX4y8DPeOH/PWSccBJT/lh3D4hPi7fDK
98KKQ/s/vUPbn74UVjkffSlm5Pz0KhblfvSqdn96J85JfeidCEn96F1ISn3p
XZiV+tOXoV/6Q1+GcekffTn80l/6cq3/9BWISz7oK5CU/NC7YZB80bu1/NF7
4JZ80nuwLPmlr8RJyTd9JaYk//RV0Mt80FchV+aHvhrzMl/01dr80ddo85lw
P1hak86o0eY38QeRmh93
       "]], 
      LineBox[{{63.66347930689132, -0.2}, {64., -0.09447748651390223}}]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Imaginary part QFT\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.2, 0.2}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700051386*^9, 3.615987582828485*^9, 
  3.6162636537448063`*^9, 3.624968783654296*^9, 3.625184978523404*^9, 
  3.6554034015417147`*^9, 3.657377223841136*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["And we invert it to recover the original function:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"IQFTSin", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], 
      RowBox[{
       SuperscriptBox["2", "6"], "-", "1"}]], 
     RowBox[{
      RowBox[{
       RowBox[{"QFTSin", "\[LeftDoubleBracket]", 
        RowBox[{"i", "+", "1"}], "\[RightDoubleBracket]"}], 
       "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], " ", 
      RowBox[{"IQuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "6"}], "]"}], "]"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"IQFTSin", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"N", "[", "IQFTSin", "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "IQFTSin", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd0n8s1HEcx/HvtEnFkp/51WFxJZQccXfcy2/HIcdGK2aMtpqyJEr4pvKj
Vbs2v3Likh/XJZPM5lZIxaKoUG3S+jFMPyiNP8xS7d7a9Nm+++7xfX72+ezz
2dcm8ag0WYthGPx5/r4147toqerGhZni1yL6AJ+pZkc7kxXrICVmjcQ29hVZ
H7ryCI5f9CjZCN2h/fVZ+iPkzRg7kG22peQl2RJWuXmqx1PPydaYcGYMF7cP
km2Rb+axzlTST94KvrAvMia0l2yPCqehn+e8esjboB40ytuXdo/sgPQahZKj
aCM7Qv2pT5GeqCI7Q8/nkOVNi6vkXWBWDRcoMgzMMrllXRrvhjBLnnDifB3Z
FW1Px1qOeDaReVAaN7S66d8hu2FeMHHtvcVdsjuSevAtx7+NvAfmnOny8pIV
e2DmiUp3yWLFnogzSBs2n20l8zE+4L7Dz35lfQG0huZCyjpuk4XoGd3ULklQ
kr3QV77h8FROLdkb7QvWjVxFBVmE6GIvw0hJERmrz88CWVLxl6jhArofHyy7
CmJy1KUasz5o6giTPqirpu6LHF7Ssmy+lrovso/P2vxoqafuB77MVqDObKTu
h4gU3QZhoJK6P1g75m2JDZn1R/zO01nmpjSfCUBRAMdKJqD12ADsl6o+WKpo
PyYQtd4BJq7aNdQDYSxo/nxytJx6EIzj1/ce3CijHgRuQanpQ22WevB/5w9G
vlvqSGVnLt2PGImXuYlLFy9pzIphxjMYalxbSj0EzzpbqqSopB6CwtT7Og7i
auqhiHrEW1SeVVAPhfXH5FMZ7tepSxAnEk9eFpJZCYK6zjS+kdN8JgyxtyZf
uHTTemwYqn5x59p4curhcDo2XhHUTf8rG46vhWy/SekV6hHgVy0UDYwUUo+A
h57lu5LpTOp7/53/Nz5lFaQ=
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.08039233645206104, 0.3677936891458129}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700162601*^9, 3.6159875829463587`*^9, 
  3.616263653876512*^9, 3.624968783911557*^9, 3.625184978641672*^9, 
  3.65540340182775*^9, 3.657377223942342*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
The Quantum Fourier Transform is in fact the same as the Discrete Fourier \
Transform (DFT):; therefore, 
we can check our results by performing the DFT that is already provided in \
Mathematica.  For instance,  this last 
step of inverting is invoked by the command  InverseFourier[ ].
[NOTE: Although mathematically the same thing, it is essential to emphasize \
that the physical implementation of the QFT making good use of quantum \
interference is what makes it appealing.]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"itras", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"InverseFourier", "[", "QFTSin", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"Flatten", "[", "itras", "]"}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], PointBox[CompressedData["
1:eJxd0nss1WEcx/HfakmlCLmecplLSSKHcM7hw3E7HLeTTW2GaWqrlC6icvl1
c1vstB2OXE9yOZ3KTrIWK6SiaChU6zaVmFopLX+YVdr5atOz/fbb6/d+9jx7
nv2sEg9IkhYxDIO55+9bM775zFZcKvia/9yHPsBvvNHR1mje2kiKWSy23v6M
rIdV5REWwuhhsiE6Qnvq0vWGyCZ4GXvCdJ3sKZkDTla26sH4ANkSo06MwcyG
PrI1Tpl6LDMW95Bt4MXvjooJ7SLbQb6p/8cZQSd5PVr7DLN3pNwmO+BgtUJp
oWgmO+LWh27F4UQV2Qm6vns4l80vkJ3BLBguqE3VN02zL2nXeAsE6eUJR8/W
kl3R/PiVer/nVTIXyjX1TW5618lumOZ9rBwxv0F2R2InvmT6N5O3wsxiQi6X
zdsDk49UOrPm8/ZEnH7KoNlkE9kLI73uG4V28+vzsKR/KqSk5RqZj67h1TfF
CUqyAA/lK/aOZ9aQvdE0bdlgrygl+8AvX2AQJc4jY+H5WYCViD5vG8yh+/EF
w+XFZLYWa8z6Qt0SJrlbW0XdDxncnb+lP2uoz/nIpNV3dR11IfhSa15rWgN1
IaJ36dTzA5XU/cHaMq9lVmTWH/GbM9LNjGk+E4DCAIu1Uh6txwYgXqJ6x1HR
fkwgKr0DjFy1qqkHwoTX+OnYsJx6EAzjlnft1pVSD4J1TrHxPS2WevB/5w+G
1C15qKwti+5HhH1F9omz5wo1ZkUw4er3Nywtph6CgTZ1hQRl1ENQkHxH20FU
RT0Ukfe5M8rTCuqhsHmfdDzV/SJ1MeJ8RGNFfDIrRmD7yYYX5TSfCUPslbEn
Lh20HhuGol/2U83ccurhcD70pjSog/5XNhyjuWyPUfF56hFwrpjO6x3KpR4B
8UrOW9lEGvXIf+f/A4n2FMw=
       "]]}, {}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.08039233645206104, 0.3677936891458129}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700183454*^9, 3.615987582983344*^9, 
  3.616263653913817*^9, 3.6249687839844303`*^9, 3.625184978671236*^9, 
  3.655403401912734*^9, 3.6573772242795887`*^9},
 CellTags->"b:2.0.0"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["QFT of a constant function", "Subsubsection",
 FontFamily->"Helvetica",
 CellTags->{"c:6", "b:2.0.0"}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CteVector", "=", 
   RowBox[{"Table", "[", 
    RowBox[{"1", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       SuperscriptBox["2", "6"]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"CteVector", "=", 
   FractionBox["CteVector", 
    RowBox[{"Norm", "[", "CteVector", "]"}]]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"CteVector", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Cte\>\""}], "}"}]}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdyUtKAgAUheGDo4YOHDhoUCEiEVJZpqmplY9eGi0haNwWXFpLaglpiuh3
4XL4+I8/vz++CkkGy1/t+n776/3Z7H/e8QEu4hIu40N8hE9wBVdxDZ/iM1zH
5/gCX+IGvsLXuIlvcAu38S3u4C7u4Tvcx4N9L3CGdJx7Os4DHeeRjjOi44zp
OBM6zpSO80THeabjvNBxXuk4b3ScGR1nTsd53/oPbC5o7Q==
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Cte\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {0, 0.25}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6141757002047253`*^9, 3.615987583004891*^9, 
  3.616263653942174*^9, 3.6249687840213003`*^9, 3.6251849786976547`*^9, 
  3.6554034020776987`*^9, 3.6573772243049507`*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
we apply the quantum Fourier transform: (NOTE we compute the QFT of the base \
vectors)\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTCte", "=", 
   RowBox[{"N", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], 
      RowBox[{
       SuperscriptBox["2", "6"], "-", "1"}]], 
     RowBox[{
      RowBox[{"CteVector", "\[LeftDoubleBracket]", 
       RowBox[{"i", "+", "1"}], "\[RightDoubleBracket]"}], " ", 
      RowBox[{"QuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "6"}], "]"}], "]"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Chop", "[", "QFTCte", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"QFTCteplotRe", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Re", "[", 
     RowBox[{"N", "[", "QFTCte", "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"QFTCteplotIM", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Im", "[", "QFTCte", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "QFTCteplotRe", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Real part of QFT\>\""}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.05`"}], ",", "0.1`"}], "}"}]}]}], "]"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "QFTCteplotIM", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Imaginary part QFT\>\""}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.2`"}], ",", "0.2`"}], "}"}]}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdyU1KQgEUhuGPRg0dNHDQoCRCJEJLM7O6N7Psz5KWENTULbSN2ombCzVB
8PngcHh49z+m759bScrFLf/X//6K35/lZkVWK7OxbVzBO7iKd/EeruEDfIjr
uIGP8DFu4hY+wae4jTv4DHfxOe7hC9zHl/gKX+MCl5v+xrmh4wzoOLd0nCEd
546Oc0/HGdFxHug4j3ScJzrOMx3nhY4zpuO80nHe6DiTtec+5DAu
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Real part of QFT\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.05, 0.1}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.61417570031708*^9, 3.6159875831221123`*^9, 
  3.6162636540567913`*^9, 3.624968784151061*^9, 3.625184978805727*^9, 
  3.655403402269182*^9, 3.6573772244284153`*^9},
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdyctKggEUhdGNo4YOHDRokCEREZLXLCt/La95o0cQGvcKPXKPkBEE/9pw
OCy++uHz/aOSpDje7//b9yjlFWWe4Cqu4VN8hs/xBW7gS3yFr/ENbuJb3MJt
3MFd3MN9fIcH+B4/4CF+xE/4GY9wUfYXzpiOM6HjvNBxXuk4UzrOjI4zp+Ms
6DhLOs6KjvNGx1nTcTZ0nC0dZ0fH2f/7B3MMKS0=
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Imaginary part QFT\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.2, 0.2}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.61417570031708*^9, 3.6159875831221123`*^9, 
  3.6162636540567913`*^9, 3.624968784151061*^9, 3.625184978805727*^9, 
  3.655403402269182*^9, 3.657377224451992*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["And we invert it to recover the original function:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"IQFTCte", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], 
      RowBox[{
       SuperscriptBox["2", "6"], "-", "1"}]], 
     RowBox[{
      RowBox[{
       RowBox[{"QFTCte", "\[LeftDoubleBracket]", 
        RowBox[{"i", "+", "1"}], "\[RightDoubleBracket]"}], 
       "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], " ", 
      RowBox[{"IQuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "6"}], "]"}], "]"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"IQFTCte", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"N", "[", "IQFTCte", "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "IQFTCte", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}]}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdkUkKwkAQRVtXLl24cOFCRURExHnWtBlM4oxHEFx7hdxMj+QBHGOCWK8h
NI9f/1d1pXA8H05JpZT+fOEdnZsR3df4/sp/nAKn9fMVnkvMGf0SnEV9TicE
56EX4S8hvwx/Bf4quAaugxvo14TeQv826jvgLrgH7iNvoO+Ch6gf4b1j+CeY
dwqegQ34tdQDDX0u5wnAypT1gYl+FvIt+G3oNnRHP0S+g/kWmA+sXOiuzFMe
dA/z+3L/gQ99CX0JfYX8Ff7nGvtbY/4N8jfI32J/W+xnh/47+Pc/fgOLhC4K

       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {0, 1}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700439913*^9, 3.6159875832382393`*^9, 
  3.6162636541756372`*^9, 3.624968784267519*^9, 3.6251849789234467`*^9, 
  3.6554034025443993`*^9, 3.6573772245522947`*^9},
 CellTags->"b:2.0.0"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["QFT of the Mod[x,5] function", "Subsubsection",
 FontFamily->"Helvetica",
 CellTags->{"c:7", "b:2.0.0"}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ModVector", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"i", ",", "5"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       SuperscriptBox["2", "6"]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ModVector", "=", 
   FractionBox["ModVector", 
    RowBox[{"Norm", "[", "ModVector", "]"}]]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"ModVector", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Mod[Number,5]\>\""}], "}"}]}]}], 
  "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdzssKQUEYwPGTlaWFRCmXJElyv3PmOMf9cuQRlLU8gXkHT+EFlLWVpQdR
NsoSnUnN/6tp+mbq1z+x2a23PsMwxPf8bm+e5usQOYceJ1M9CG+/qN0v3rGj
/76/qj2g/m9qDwpDm7DQvSi8OLwkvBS8NLwMvCy8HLw8vAK8IrwSvDK8Crwq
vBq8OrwGvCa8Frw2vA68LrwePBOe0D0p4Fm6Jy14fd2TffTZuidt9Dm6Jx30
DdA3QN8QfUP0jdA3Qt8YfWP0TdA3Qd8UfVP0zdA3Q98cfXP0LdC3QN8SfUv0
uehz0bf6ex+bgwIs
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Mod[Number,5]\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {0, 0.20254787341673333`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700461405*^9, 3.615987583258816*^9, 
  3.616263654209136*^9, 3.6249687842890177`*^9, 3.6251849789432583`*^9, 
  3.655403402646549*^9, 3.657377224584777*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
we apply the quantum Fourier transform: (NOTE we compute the QFT of the base \
vectors)\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTMod", "=", 
   RowBox[{"N", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], 
      RowBox[{
       SuperscriptBox["2", "6"], "-", "1"}]], 
     RowBox[{
      RowBox[{"ModVector", "\[LeftDoubleBracket]", 
       RowBox[{"i", "+", "1"}], "\[RightDoubleBracket]"}], " ", 
      RowBox[{"QuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "6"}], "]"}], "]"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Chop", "[", "QFTMod", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"QFTModplotRe", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Re", "[", 
     RowBox[{"N", "[", "QFTMod", "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"QFTModplotIM", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Im", "[", "QFTMod", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "QFTModplotRe", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Real part of QFT\>\""}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.05`"}], ",", "0.1`"}], "}"}]}]}], "]"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "QFTModplotIM", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Number", ",", "\"\<Imaginary part QFT\>\""}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.2`"}], ",", "0.2`"}], "}"}]}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], 
      LineBox[{{1.8922649781481689`, 0.1}, {2., 0.012720613585153417`}, {3., 
       0.012909214006722097`}, {4., 0.013239360557653734`}, {5., 
       0.013738104441905485`}, {6., 0.014451360589879059`}, {7., 
       0.015456154786957}, {8., 0.01688606503394746}, {9., 
       0.01898886313281875}, {10., 0.022275469950666052`}, {11., 
       0.02798830072186262}, {12., 0.04013078774742769}, {13., 
       0.08271769513729671}, {13.346344499820013`, -0.05}}], 
      LineBox[{{14.980086051393409`, -0.05}, {15., -0.044910646362191}, {
       16., -0.021557870652016142`}, {17., -0.012659242088545835`}, {
       18., -0.007825119188185462}, {19., -0.004630248882827273}, {
       20., -0.002172581364224352}, {21., 0.00001721075580536635}, {22., 
       0.0023058960792915473`}, {23., 0.005171288414610398}, {24., 
       0.009619293717174536}, {25., 0.01898886313281878}, {26., 
       0.05860788669023376}, {26.687998698854265`, -0.05}}], 
      LineBox[{{27.728436228887148`, -0.05}, {28., -0.03163840326272813}, {
       29., -0.02051755780373256}, {30., -0.01614115129420609}, {
       31., -0.014005287115887118`}, {32., -0.012971557335481988`}, {
       33., -0.012659242088545831`}, {34., -0.012971557335482008`}, {
       35., -0.014005287115887138`}, {36., -0.016141151294206055`}, {
       37., -0.020517557803732538`}, {38., -0.031638403262728154`}, {
       38.27156377111285, -0.05}}], 
      LineBox[{{39.312001301145735`, -0.05}, {40., 0.05860788669023378}, {41.,
        0.018988863132818737`}, {42., 0.009619293717174515}, {43., 
       0.00517128841461037}, {44., 0.0023058960792915473`}, {45., 
       0.000017210755805352473`}, {46., -0.0021725813642244074`}, {
       47., -0.0046302488828273425`}, {48., -0.007825119188185528}, {
       49., -0.012659242088545835`}, {50., -0.02155787065201619}, {
       51., -0.04491064636219104}, {51.01991394860659, -0.05}}], 
      LineBox[{{52.65365550017999, -0.05}, {53., 0.08271769513729664}, {54., 
       0.04013078774742767}, {55., 0.02798830072186259}, {56., 
       0.022275469950665983`}, {57., 0.018988863132818723`}, {58., 
       0.01688606503394742}, {59., 0.015456154786956972`}, {60., 
       0.014451360589879038`}, {61., 0.013738104441905472`}, {62., 
       0.01323936055765369}, {63., 0.012909214006722125`}, {64., 
       0.012720613585153417`}}]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Real part of QFT\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.05, 0.1}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700602639*^9, 3.6159875834136333`*^9, 
  3.616263654367147*^9, 3.624968784447843*^9, 3.625184979087237*^9, 
  3.6554034029009647`*^9, 3.657377224789715*^9},
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxVk21Ik1EUx28ioVGmVGIQMUtSRKT2lmXOv3O+z1pZkNYSzDSGUERvg8Rl
SVOGW4Rl0DuBliBZUuttjUoCv7QgP9SnIr+YFSuIbBg0d/774IWHhx+/5z73
nHvOyWw6tONAglIK0WfuLStcrOYvqOD1QqNp1CSYBGXR9rSMTJJToYYmuuo0
Cdy3HC59yj7PcDo5Ayrs/XtzIpu8Cqpda+gYNJI1gDXxz9LdFvIauJyd5sj4
dnIWZtYem/qkaySvw2uH/fG4s42cg6MnGt9PpraTc3Hv35j33ZI+ch76g+7Q
af+TF8L5eHk1+YjdfpG8HgeTUx60tvSSN+B3x8qH6d+7yVrUre6zZzrdZB1C
Ba/OF3nirEfngtb7hkD8ewO+zaaNDC3zkI3oev7U+HXaS96IpNHmnIjmArkA
e83+hpMf+8mbkL+4w+j8cou8Ged+DCz6tf8RuRDFl7MXvskIMr8tcA+Hch29
N8hFmEkz+/QffGQT9kzrB9runCEXY/Z204r6K8fJwGD9lO/t52ZhF6DmrRI8
Ey/nu0ri+xmPGTb5P70ZETmfvhTdEh99aTx+egvckh/PtyBP8md8ZWiQ+6Ev
Q6LcH305euR+6csRlvunr8BZqQ99Rbx+9JXYJfWlr4zXn74KDukP+iqMSf/Q
V+OS9Bd9NUak/5hfDQ5LfzL/Gvilf+mtUFmx/qa3AtL/9LWwyXzQ1yJ4KjY/
9Fuhfsbmiz7Kutj80W+DuhubT/oo74zNb0C8LTrf1+bmO/AfrrcbnQ==
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["Number", TraditionalForm], 
     FormBox["\"Imaginary part QFT\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {-0.2, 0.2}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700602639*^9, 3.6159875834136333`*^9, 
  3.616263654367147*^9, 3.624968784447843*^9, 3.625184979087237*^9, 
  3.6554034029009647`*^9, 3.657377224825234*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["And we invert it to recover the original function:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"IQFTMod", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], 
      RowBox[{
       SuperscriptBox["2", "6"], "-", "1"}]], 
     RowBox[{
      RowBox[{
       RowBox[{"QFTMod", "\[LeftDoubleBracket]", 
        RowBox[{"i", "+", "1"}], "\[RightDoubleBracket]"}], 
       "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], " ", 
      RowBox[{"IQuantumFourierTransformV", "[", 
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", "6"}], "]"}], "]"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"IQFTMod", "=", 
   RowBox[{"Chop", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"N", "[", "IQFTMod", "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", "IQFTMod", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:2.0.0"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdk0lKA0EYhRtXWWYhYiDggIiIBMcYx36Z50RyBMG1eILUHXKKXEBw7cpl
7pGGqDHDLib5a/FeQdP8XdVfffWqau/5tfOyEQQB/p/le91G4bCbeN+K+qF9
wN+q/rA6hvlOLzZ4+7Q6jvGq/8vqTQTUtjEjXlJ4u5gQb9/6Pe9AeIcYEe8I
v8Q7xox4J8JLCe8UE+KdCe9c/C6Edym8K3wTLy3rvcaUeBmbz/NuhHeLOfHu
xO9eeA+yH4/CC+1/zwP7Och6s8xzWeHlOD+XE78874fL23jvV2CeK9h83q+I
H/Iril+Jea4kfmXmubLsR4XzcxXJryp+VTkvNcmvJuevLvnVxa+BiPwaNt77
NcWvKfm1xK8l960tfm3xe8LUeAsxcwIC
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 64.}, {0, 0.2025478734167334}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6141757008367233`*^9, 3.615987583635518*^9, 
  3.616263654617951*^9, 3.624968784567534*^9, 3.625184979332847*^9, 
  3.655403403300058*^9, 3.657377224948814*^9},
 CellTags->"b:2.0.0"]
}, Open  ]],

Cell["\<\
This kind of periodicity plays an important role in Shor's algorithm.\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.0"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["b) Quantum Fourier Transform using  density matrices", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:8", "b:2.0.1"}],

Cell["\<\
To compute the Quantum Fourier Transform in a density matrix framework, we \
define it in a multiqubit system by following the above figure where it is \
built by combining Controlled phase gates and Hadamard gates. First we define \
the gates we need, and provide compiled versions to speed the computation.\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[TextData[{
 "The controlled phase gate used ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "k"], " ", "="}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {"0", 
       SuperscriptBox["e", 
        RowBox[{"2", "\[Pi]", " ", 
         RowBox[{"\[ImaginaryI]", "/", 
          SuperscriptBox["2", "k"]}]}]]}
     }], ")"}], TraditionalForm]]],
 ", can be written in the following way as a two qubit operator:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ControlP", "[", 
    RowBox[{"L_", ",", "q1_", ",", "q2_", ",", "k_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[Phi]", "}"}], ",", 
     RowBox[{
      RowBox[{"\[Phi]", "=", 
       RowBox[{"Exp", "[", 
        RowBox[{"2", "*", " ", "\[Pi]", " ", "*", " ", 
         RowBox[{"\[ImaginaryI]", " ", "/", 
          SuperscriptBox["2", "k"]}]}], "]"}]}], ";", 
      RowBox[{
       RowBox[{"1", "/", "4"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"3", "+", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "0", ",", "0"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "3", ",", "0"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "0", ",", "3"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"\[Phi]", "-", "1"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "3", ",", "3"}], "]"}]}]}],
         ")"}]}]}]}], "]"}]}], ";"}]], "Input",
 CellTags->"b:2.0.1"],

Cell[" Forms Operator to Invert L qubits... the last step of QFT ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"InvertQubits", "[", "L_", "]"}], ":=", 
  RowBox[{
   RowBox[{"InvertQubits", "[", "L", "]"}], "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Reverse", "[", 
         RowBox[{"IntegerDigits", "[", 
          RowBox[{"i", ",", "2", ",", "L"}], "]"}], "]"}], "\[Equal]", " ", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"j", ",", "2", ",", "L"}], "]"}]}], ",", "1", ",", "0"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "L"}], "-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "L"}], "-", "1"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellTags->"b:2.0.1"],

Cell[TextData[StyleBox["Generate Compiled versions of the Functions to speed \
the computation: ",
 FontWeight->"Bold"]], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompControlP", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q1", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q2", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Lv", "=", "L"}], ",", 
         RowBox[{"qv1", "=", "q1"}], ",", 
         RowBox[{"qv2", "=", "q2"}], ",", 
         RowBox[{"kv", "=", "k"}]}], "  ", "}"}], ",", " ", 
       RowBox[{"ControlP", "[", 
        RowBox[{"Lv", ",", "qv1", ",", "qv2", ",", "kv"}], "]"}]}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"ControlP", "[", 
         RowBox[{"Lv", ",", "qv1", ",", "qv2", ",", "kv"}], "]"}], ",", 
        "_Complex", ",", "2"}], "}"}], "}"}]}], "]"}]}], " ", ";"}]], "Input",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompHad", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Lv", "=", "L"}], ",", 
         RowBox[{"iv", "=", "i"}]}], "}"}], ",", " ", 
       RowBox[{"had", "[", 
        RowBox[{"Lv", ",", "iv"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"had", "[", 
         RowBox[{"Lv", ",", "iv"}], "]"}], ",", "_Real", ",", "2"}], "}"}], 
      "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompInvertQubits", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"L", ",", "_Integer"}], "}"}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Lv", "=", "L"}], " ", "}"}], ",", 
       RowBox[{"InvertQubits", "[", "Lv", "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "  ", 
       RowBox[{
        RowBox[{"InvertQubits", "[", "Lv", "]"}], ",", "_Real", ",", "2"}], 
       "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:2.0.1"],

Cell["Then the Quantum Fourier Transform operator reads:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumFourierTransform", "=", 
    RowBox[{"Compile", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[CapitalOmega]s", ",", "_Complex", ",", "2"}], "}"}]}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Module", "[", 
       RowBox[{
        RowBox[{"{", "\[CapitalOmega]", "}"}], ",", 
        RowBox[{
         RowBox[{"\[CapitalOmega]", "=", "\[CapitalOmega]s"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"i", "=", "1"}], ",", 
           RowBox[{"i", "\[LessEqual]", "L"}], ",", 
           RowBox[{"i", "++"}], ",", 
           RowBox[{
            RowBox[{"\[CapitalOmega]", "=", 
             RowBox[{"N", "[", 
              RowBox[{
               RowBox[{"CompHad", "[", 
                RowBox[{"L", ",", "i"}], "]"}], ".", "\[CapitalOmega]"}], 
              "]"}]}], ";", 
            RowBox[{"For", "[", " ", 
             RowBox[{
              RowBox[{"j", "=", 
               RowBox[{"i", "+", "1"}]}], ",", 
              RowBox[{"j", "\[LessEqual]", "L"}], ",", 
              RowBox[{"j", "++"}], ",", 
              RowBox[{"\[CapitalOmega]", "=", 
               RowBox[{"N", "[", 
                RowBox[{
                 RowBox[{"CompControlP", "[", 
                  RowBox[{"L", ",", "j", ",", "i", ",", 
                   RowBox[{"j", "-", "i", "+", "1"}]}], "]"}], ".", 
                 "\[CapitalOmega]"}], "]"}]}]}], "]"}]}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"\[CapitalOmega]", "=", 
          RowBox[{
           RowBox[{"CompInvertQubits", "[", "L", "]"}], ".", 
           "\[CapitalOmega]"}]}], ";", "\[CapitalOmega]"}]}], "]"}], ",", " ",
       "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompHad", "[", 
           RowBox[{"L", ",", "i"}], "]"}], ",", "_Real", ",", "2"}], "}"}], 
        ",", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompControlP", "[", 
           RowBox[{"L", ",", "j", ",", "i", ",", 
            RowBox[{"j", "-", "i", "+", "1"}]}], "]"}], ",", "_Complex", ",", 
          "2"}], "}"}], ",", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompInvertQubits", "[", "L", "]"}], ",", "_Real", ",", 
          "2"}], "}"}]}], "}"}]}], " ", "]"}]}], ";"}], "  "}]], "Input",
 CellTags->"b:2.0.1"],

Cell["\<\
We can check both definitions, the density matrix one and the vector one. \
Making use of our previous RanVector, we can construct the pure state density \
matrix:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DensRanVec", "=", 
   RowBox[{"AF", "[", 
    RowBox[{
     RowBox[{"{", "RanVector", "}"}], "\[TensorProduct]", 
     RowBox[{"Adj", "[", 
      RowBox[{"{", "RanVector", "}"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.614175648893342*^9, 3.614175681108389*^9}},
 CellTags->"b:2.0.1"],

Cell["And apply the quantum fourier transform just defined:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTDens", "=", 
   RowBox[{"QuantumFourierTransform", "[", 
    RowBox[{"4", ",", 
     RowBox[{"IdentityMatrix", "[", 
      RowBox[{"4", "^", "2"}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"QFTDensVec", "=", 
   RowBox[{"QFTDens", ".", "DensRanVec", ".", 
    RowBox[{"Adj", "[", "QFTDens", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Length", "[", "QFTDens", "]"}]}], "Input",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 TagBox["16",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700886567*^9, 3.615987583724449*^9, 
  3.616263654694394*^9, 3.624968784682973*^9, 3.625184979413748*^9, 
  3.655403403674739*^9, 3.6573772253030777`*^9},
 CellTags->"b:2.0.1"]
}, Open  ]],

Cell["\<\
Which we can compare to the pure state density matrix constructed with the \
fourier transform of RanVector:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:2.0.1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DensQFTVec", "=", 
  RowBox[{"AF", "[", 
   RowBox[{"QFTVec", "\[TensorProduct]", 
    RowBox[{"Adj", "[", "QFTVec", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.614175576030506*^9, 3.614175583136767*^9}},
 CellTags->"b:2.0.1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.7774039881435516`", 
      RowBox[{
       RowBox[{"-", "0.07396945663355203`"}], "+", 
       RowBox[{"0.04825647716096191`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.10262306713994826`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0756117418402557`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.05012727888352382`"}], "-", 
       RowBox[{"0.009015802000027153`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.07678812146069307`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0510840042804185`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.026716293584490347`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.08840169640076863`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0507494807772027`"}], "+", 
       RowBox[{"0.17855502466546833`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.08483013249491309`"}], "+", 
       RowBox[{"0.0053821987882953755`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"-", "0.04824002395367303`"}], 
      RowBox[{
       RowBox[{"-", "0.08483013249491306`"}], "-", 
       RowBox[{"0.005382198788295363`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.05074948077720274`"}], "-", 
       RowBox[{"0.17855502466546833`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.026716293584490326`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.08840169640076864`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.07678812146069307`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0510840042804185`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.05012727888352388`"}], "+", 
       RowBox[{"0.009015802000027153`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.10262306713994822`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.07561174184025571`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.07396945663355206`"}], "-", 
       RowBox[{"0.04825647716096189`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.07396945663355203`"}], "-", 
       RowBox[{"0.04825647716096191`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010033609579590982`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0050710007683517355`"}], "-", 
       RowBox[{"0.01356461674431194`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0042099304713450695`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0039694417712092004`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010477319167937073`"}], "+", 
       RowBox[{"0.00009407182224796355`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.002945398735494988`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.010069744661496478`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01591238040443972`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.013839171341699858`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.008405622892813663`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.004753622425147356`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004590005215131242`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0029944451657930274`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.007737435034597598`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.005777847476564136`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.006254823780659192`"}], "+", 
       RowBox[{"0.02013960508633236`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.00802947278154243`"}], "+", 
       RowBox[{"0.006752976981247719`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004135355083766987`"}], "-", 
       RowBox[{"0.009627144165624883`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005329221984991714`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0022537470095956203`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.014458028231197951`"}], "+", 
       RowBox[{"0.0008241940782210645`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0040426766193745935`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.009183141453055266`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.10262306713994826`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0756117418402557`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0050710007683517355`"}], "+", 
       RowBox[{"0.01356461674431194`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.02090113976920763`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.007494064461423762`"}], "+", 
       RowBox[{"0.0036853189073133546`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005168074851259704`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.014212019729555025`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01212485963605162`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.009071200867418566`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010667283912413105`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.028506569636198187`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010674721359507963`"}], "+", 
       RowBox[{"0.008961224707282363`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.006368039388185333`"}], "+", 
       RowBox[{"0.004691913462218237`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.011721686467791865`"}], "+", 
       RowBox[{"0.00754024473795966`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.02406589635213908`"}], "-", 
       RowBox[{"0.018634593938681436`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005071363559194731`"}], "-", 
       RowBox[{"0.014268152579135149`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01510512329126406`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0007250881444323676`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005740277489126852`"}], "+", 
       RowBox[{"0.006065623788894231`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006192865586642519`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.01996261654888285`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.014458028231197958`"}], "+", 
       RowBox[{"0.000824194078221068`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.05012727888352382`"}], "+", 
       RowBox[{"0.009015802000027153`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0042099304713450695`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0039694417712092004`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.007494064461423762`"}], "-", 
       RowBox[{"0.0036853189073133546`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.003336783465910476`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004358887223531382`"}], "+", 
       RowBox[{"0.004184450655911252`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0027478985482966463`"}], "-", 
       RowBox[{"0.005390334678025579`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0012015845581776902`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.012101847856760172`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005407457299952105`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0013308481480115676`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.003110533481887818`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0005594549437307137`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005532295451904307`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0006367560574584454`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005343103698922225`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.010924732026973728`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0006974532113255393`"}], "+", 
       RowBox[{"0.00601000943350904`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005543762204840965`"}], "-", 
       RowBox[{"0.0024033779842958687`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0031276652032223707`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.001162684082167432`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005740277489126842`"}], "+", 
       RowBox[{"0.006065623788894225`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005329221984991711`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0022537470095956146`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.07678812146069307`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0510840042804185`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010477319167937073`"}], "-", 
       RowBox[{"0.00009407182224796355`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005168074851259704`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.014212019729555025`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004358887223531382`"}], "-", 
       RowBox[{"0.004184450655911252`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010941532614331399`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.003170061741608501`"}], "+", 
       RowBox[{"0.010487437137864373`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.01674581445018787`"}], "+", 
       RowBox[{"0.014301982492691198`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008732770717702846`"}], "-", 
       RowBox[{"0.005042647036785601`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004764911005238344`"}], "-", 
       RowBox[{"0.003169900885151981`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008025431238789404`"}], "-", 
       RowBox[{"0.006105900996334117`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006720248969637858`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.020971582167481972`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.00844786846252427`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.006976327659030548`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004227969182392983`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.010091650635213887`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005543762204840971`"}], "-", 
       RowBox[{"0.002403377984295873`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.015105123291264056`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0007250881444323719`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004135355083766992`"}], "-", 
       RowBox[{"0.009627144165624883`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.026716293584490347`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.08840169640076863`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.002945398735494988`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.010069744661496478`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01212485963605162`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.009071200867418566`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0027478985482966463`"}], "+", 
       RowBox[{"0.005390334678025579`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.003170061741608501`"}], "-", 
       RowBox[{"0.010487437137864373`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010970641261813923`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01856014282620319`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.011907153543444638`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.002303244192983489`"}], "+", 
       RowBox[{"0.009831336265281891`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0016578184086073829`"}], "+", 
       RowBox[{"0.005485564798943311`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0035273066122866775`"}], "+", 
       RowBox[{"0.009461406588688472`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.022048259811406602`"}], "-", 
       RowBox[{"0.0003653033327804438`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.009134375037872507`"}], "-", 
       RowBox[{"0.006076031794099327`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.008447868462524271`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.006976327659030545`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0006974532113255434`"}], "+", 
       RowBox[{"0.006010009433509046`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005071363559194729`"}], "-", 
       RowBox[{"0.014268152579135144`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.00802947278154243`"}], "+", 
       RowBox[{"0.00675297698124772`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.0507494807772027`"}], "-", 
       RowBox[{"0.17855502466546833`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01591238040443972`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.013839171341699858`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010667283912413105`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.028506569636198187`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0012015845581776902`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.012101847856760172`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.01674581445018787`"}], "-", 
       RowBox[{"0.014301982492691198`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01856014282620319`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.011907153543444638`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.044323681326521494`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006773960381078302`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.019132526759448487`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0031491427953372715`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.011079823101345275`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004301581406811765`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.019835234227152495`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.03769775751229973`"}], "+", 
       RowBox[{"0.02331239594891267`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.022048259811406602`"}], "-", 
       RowBox[{"0.0003653033327804412`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006720248969637862`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.02097158216748197`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005343103698922226`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.010924732026973742`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.024065896352139075`"}], "-", 
       RowBox[{"0.018634593938681433`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.006254823780659188`"}], "+", 
       RowBox[{"0.020139605086332363`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.08483013249491309`"}], "-", 
       RowBox[{"0.0053821987882953755`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.008405622892813663`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.004753622425147356`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010674721359507963`"}], "-", 
       RowBox[{"0.008961224707282363`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005407457299952105`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0013308481480115676`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008732770717702846`"}], "+", 
       RowBox[{"0.005042647036785601`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.002303244192983489`"}], "-", 
       RowBox[{"0.009831336265281891`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006773960381078302`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.019132526759448487`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.009293905810999111`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005263939580912232`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.00033398001866547606`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0092193806883125`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.001174608423132371`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004301581406811769`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.019835234227152505`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.003527306612286678`"}], "+", 
       RowBox[{"0.009461406588688479`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008025431238789408`"}], "-", 
       RowBox[{"0.00610590099633412`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0055322954519043165`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0006367560574584445`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.011721686467791866`"}], "+", 
       RowBox[{"0.0075402447379596645`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.007737435034597606`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.005777847476564137`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"-", "0.04824002395367303`"}], 
      RowBox[{"0.004590005215131242`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0029944451657930274`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.006368039388185333`"}], "-", 
       RowBox[{"0.004691913462218237`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.003110533481887818`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0005594549437307137`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004764911005238344`"}], "+", 
       RowBox[{"0.003169900885151981`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0016578184086073829`"}], "-", 
       RowBox[{"0.005485564798943311`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0031491427953372715`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.011079823101345275`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005263939580912232`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.00033398001866547606`", " ", "\[ImaginaryI]"}]}], 
      "0.00299342419969839`", 
      RowBox[{"0.005263939580912229`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0003339800186654753`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.003149142795337274`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.011079823101345275`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0016578184086073816`"}], "+", 
       RowBox[{"0.005485564798943312`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004764911005238344`"}], "-", 
       RowBox[{"0.003169900885151981`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0031105334818878217`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0005594549437307137`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.00636803938818533`"}], "+", 
       RowBox[{"0.0046919134622182375`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004590005215131245`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.002994445165793026`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.08483013249491306`"}], "+", 
       RowBox[{"0.005382198788295363`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.007737435034597598`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.005777847476564136`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.011721686467791865`"}], "-", 
       RowBox[{"0.00754024473795966`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005532295451904307`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0006367560574584454`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008025431238789404`"}], "+", 
       RowBox[{"0.006105900996334117`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0035273066122866775`"}], "-", 
       RowBox[{"0.009461406588688472`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004301581406811765`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.019835234227152495`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0092193806883125`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.001174608423132371`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005263939580912229`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0003339800186654753`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.009293905810999103`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006773960381078302`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.019132526759448477`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0023032441929834863`"}], "+", 
       RowBox[{"0.00983133626528189`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008732770717702842`"}], "-", 
       RowBox[{"0.0050426470367856`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005407457299952109`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0013308481480115667`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010674721359507956`"}], "+", 
       RowBox[{"0.00896122470728236`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.008405622892813665`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.004753622425147351`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.05074948077720274`"}], "+", 
       RowBox[{"0.17855502466546833`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.006254823780659192`"}], "-", 
       RowBox[{"0.02013960508633236`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.02406589635213908`"}], "+", 
       RowBox[{"0.018634593938681436`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005343103698922225`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.010924732026973728`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006720248969637858`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.020971582167481972`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.022048259811406602`"}], "+", 
       RowBox[{"0.0003653033327804438`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.03769775751229973`"}], "-", 
       RowBox[{"0.02331239594891267`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004301581406811769`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.019835234227152505`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.003149142795337274`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.011079823101345275`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006773960381078302`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.019132526759448477`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0443236813265215`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.018560142826203194`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.01190715354344464`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.016745814450187874`"}], "+", 
       RowBox[{"0.014301982492691196`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0012015845581776967`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.012101847856760186`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010667283912413101`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.028506569636198187`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01591238040443972`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.013839171341699865`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.026716293584490326`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.08840169640076864`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.00802947278154243`"}], "-", 
       RowBox[{"0.006752976981247719`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005071363559194731`"}], "+", 
       RowBox[{"0.014268152579135149`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0006974532113255393`"}], "-", 
       RowBox[{"0.00601000943350904`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.00844786846252427`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.006976327659030548`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.009134375037872507`"}], "+", 
       RowBox[{"0.006076031794099327`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.022048259811406602`"}], "+", 
       RowBox[{"0.0003653033327804412`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.003527306612286678`"}], "-", 
       RowBox[{"0.009461406588688479`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0016578184086073816`"}], "-", 
       RowBox[{"0.005485564798943312`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0023032441929834863`"}], "-", 
       RowBox[{"0.00983133626528189`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.018560142826203194`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.01190715354344464`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010970641261813926`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0031700617416085047`"}], "+", 
       RowBox[{"0.010487437137864373`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0027478985482966476`"}], "-", 
       RowBox[{"0.005390334678025587`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01212485963605162`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.009071200867418566`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.002945398735494987`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.010069744661496481`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.07678812146069307`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0510840042804185`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004135355083766987`"}], "+", 
       RowBox[{"0.009627144165624883`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01510512329126406`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0007250881444323676`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005543762204840965`"}], "+", 
       RowBox[{"0.0024033779842958687`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004227969182392983`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.010091650635213887`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.008447868462524271`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.006976327659030545`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006720248969637862`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.02097158216748197`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008025431238789408`"}], "+", 
       RowBox[{"0.00610590099633412`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004764911005238344`"}], "+", 
       RowBox[{"0.003169900885151981`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.008732770717702842`"}], "+", 
       RowBox[{"0.0050426470367856`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.016745814450187874`"}], "-", 
       RowBox[{"0.014301982492691196`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0031700617416085047`"}], "-", 
       RowBox[{"0.010487437137864373`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010941532614331399`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004358887223531388`"}], "+", 
       RowBox[{"0.004184450655911256`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005168074851259701`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.014212019729555023`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010477319167937076`"}], "+", 
       RowBox[{"0.00009407182224796876`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.05012727888352388`"}], "-", 
       RowBox[{"0.009015802000027153`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005329221984991714`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0022537470095956203`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005740277489126852`"}], "-", 
       RowBox[{"0.006065623788894231`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0031276652032223707`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.001162684082167432`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005543762204840971`"}], "+", 
       RowBox[{"0.002403377984295873`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0006974532113255434`"}], "-", 
       RowBox[{"0.006010009433509046`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005343103698922226`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.010924732026973742`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0055322954519043165`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0006367560574584445`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0031105334818878217`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0005594549437307137`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005407457299952109`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0013308481480115667`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0012015845581776967`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.012101847856760186`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.0027478985482966476`"}], "+", 
       RowBox[{"0.005390334678025587`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004358887223531388`"}], "-", 
       RowBox[{"0.004184450655911256`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.003336783465910484`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.007494064461423769`"}], "+", 
       RowBox[{"0.0036853189073133624`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004209930471345077`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.003969441771209203`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.10262306713994822`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.07561174184025571`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.014458028231197951`"}], "-", 
       RowBox[{"0.0008241940782210645`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006192865586642519`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.01996261654888285`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005740277489126842`"}], "-", 
       RowBox[{"0.006065623788894225`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.015105123291264056`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0007250881444323719`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005071363559194729`"}], "+", 
       RowBox[{"0.014268152579135144`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.024065896352139075`"}], "+", 
       RowBox[{"0.018634593938681433`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.011721686467791866`"}], "-", 
       RowBox[{"0.0075402447379596645`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.00636803938818533`"}], "-", 
       RowBox[{"0.0046919134622182375`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010674721359507956`"}], "-", 
       RowBox[{"0.00896122470728236`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010667283912413101`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.028506569636198187`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01212485963605162`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.009071200867418566`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005168074851259701`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.014212019729555023`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.007494064461423769`"}], "-", 
       RowBox[{"0.0036853189073133624`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.02090113976920762`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005071000768351739`"}], "-", 
       RowBox[{"0.013564616744311937`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.07396945663355206`"}], "+", 
       RowBox[{"0.04825647716096189`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.0040426766193745935`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.009183141453055266`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.014458028231197958`"}], "-", 
       RowBox[{"0.000824194078221068`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.005329221984991711`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0022537470095956146`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.004135355083766992`"}], "+", 
       RowBox[{"0.009627144165624883`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.00802947278154243`"}], "-", 
       RowBox[{"0.00675297698124772`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.006254823780659188`"}], "-", 
       RowBox[{"0.020139605086332363`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.007737435034597606`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.005777847476564137`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004590005215131245`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.002994445165793026`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.008405622892813665`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.004753622425147351`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.01591238040443972`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.013839171341699865`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.002945398735494987`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.010069744661496481`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.010477319167937076`"}], "-", 
       RowBox[{"0.00009407182224796876`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.004209930471345077`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.003969441771209203`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.005071000768351739`"}], "+", 
       RowBox[{"0.013564616744311937`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.010033609579590985`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700903187*^9, 3.615987583746613*^9, 
  3.616263654718687*^9, 3.624968784714676*^9, 3.625184979436646*^9, 
  3.655403403766698*^9, 3.657377225360183*^9},
 CellTags->"b:2.0.1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chop", "[", 
  RowBox[{"DensQFTVec", "-", "QFTDensVec"}], "]"}]], "Input",
 CellTags->"b:2.0.1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.614175700933292*^9, 3.615987583775478*^9, 
  3.6162636547436743`*^9, 3.624968784811788*^9, 3.6251849794635353`*^9, 
  3.655403403842657*^9, 3.657377225379859*^9},
 CellTags->"b:2.0.1"]
}, Open  ]],

Cell[BoxData["Quit"], "Input",
 CellTags->"b:2.0.1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[GridBox[{
     {
      RowBox[{"References", ":"}], 
      RowBox[{
       RowBox[{
        RowBox[{"[", "1", "]"}], " ", "Quantum", " ", "Computation", " ", 
        "and", " ", "Quantum", " ", "Information"}], ",", " ", 
       RowBox[{"Nielsen", " ", "and", " ", "Chuang"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
      "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], TraditionalForm]], 
  "DefinitionBox",
  GridBoxOptions->{
  GridBoxDividers->{
   "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
    "RowsIndexed" -> {}},
  GridBoxSpacings->{"Columns" -> {
      Offset[0.27999999999999997`], {
       Offset[0.5599999999999999]}, 
      Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
      Offset[0.2], {
       Offset[0.16]}, 
      Offset[0.2]}, "RowsIndexed" -> {}}},
  CellTags->"S5.1.1"],
 "\nLast Version: 25-Nov-2005"
}], "Abstract",
 CellTags->"b:2.0.1"]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{1014, 678},
WindowMargins->{{216, Automatic}, {Automatic, 138}},
PrintingStartingPageNumber->46,
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{612, 792},
"PostScriptOutputFile":>FrontEnd`FileName[{"export", "home", "brunojd"}, 
  "t.ps", CharacterEncoding -> "WindowsANSI"]},
ShowSelection->True,
TaggingRules:>{
 "PaginationCache" -> {{2005, 8, 9, 3, 36, 21.330055`8.081566958720561}, {46, 
    46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 47, 47, 47, 47, 47, 47, 
    47, 47, 47, 47, 47, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 
    49, 49, 49, 49, 49, 49, 49, 49, 49, 50, 50, 50, 50, 50, 50, 50, 50, 50, 
    50, 50, 50, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 52}, 
    Automatic}},
CellLabelAutoDelete->True,
Magnification->1,
FrontEndVersion->"10.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (October 9, \
2015)",
StyleDefinitions->"Classroom.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "c:1"->{
  Cell[545, 20, 60, 1, 51, "Title",
   CellTags->"c:1"]},
 "c:2"->{
  Cell[1208, 42, 303, 7, 51, "Section",
   CellTags->"c:2"]},
 "c:3"->{
  Cell[1536, 53, 135, 3, 58, "Subsection",
   CellTags->{"c:3", "b:2.0.0"}]},
 "b:2.0.0"->{
  Cell[1536, 53, 135, 3, 58, "Subsection",
   CellTags->{"c:3", "b:2.0.0"}],
  Cell[1674, 58, 1005, 29, 35, "Abstract",
   CellTags->"b:2.0.0"],
  Cell[2682, 89, 324, 7, 51, "Text",
   CellTags->"b:2.0.0"],
  Cell[3031, 100, 133, 3, 46, "Input",
   CellTags->"b:2.0.0"],
  Cell[3167, 105, 291, 5, 22, "Print",
   CellTags->"b:2.0.0"],
  Cell[3495, 115, 112, 3, 46, "Input",
   CellTags->"b:2.0.0"],
  Cell[3610, 120, 33520, 937, 495, "Print",
   CellTags->"b:2.0.0"],
  Cell[37167, 1062, 311, 9, 62, "Input",
   CellTags->"b:2.0.0"],
  Cell[37481, 1073, 3303, 59, 184, "Output",
   CellTags->"b:2.0.0"],
  Cell[40799, 1135, 219, 6, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[41021, 1143, 2996, 84, 61, "DefinitionBox",
   CellTags->{"S5.1.1", "b:2.0.0"}],
  Cell[44020, 1229, 461, 17, 54, "Text",
   CellTags->"b:2.0.0"],
  Cell[44484, 1248, 451, 14, 53, "Text",
   CellTags->"b:2.0.0"],
  Cell[44938, 1264, 1203, 46, 83, "Text",
   CellTags->"b:2.0.0"],
  Cell[46144, 1312, 458, 15, 46, "Input",
   CellTags->"b:2.0.0"],
  Cell[46605, 1329, 224, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[46832, 1336, 2187, 57, 125, "Input",
   CellTags->"b:2.0.0"],
  Cell[49022, 1395, 2150, 58, 110, "Input",
   CellTags->"b:2.0.0"],
  Cell[51175, 1455, 649, 13, 123, "Text",
   CellTags->"b:2.0.0"],
  Cell[51849, 1472, 186, 5, 45, "Input",
   CellTags->"b:2.0.0"],
  Cell[52038, 1479, 2141, 69, 294, "Output",
   CellTags->"b:2.0.0"],
  Cell[54194, 1551, 278, 9, 27, "Text",
   CellTags->"b:2.0.0"],
  Cell[54475, 1562, 102, 2, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[54602, 1568, 84, 2, 45, "Input",
   CellTags->"b:2.0.0"],
  Cell[54689, 1572, 295, 7, 45, "Output",
   CellTags->"b:2.0.0"],
  Cell[55021, 1584, 188, 7, 34, "Subsubsection",
   CellTags->{"c:4", "b:2.0.0"}],
  Cell[55234, 1595, 568, 17, 105, "Input",
   CellTags->"b:2.0.0"],
  Cell[55805, 1614, 1309, 37, 286, "Output",
   CellTags->"b:2.0.0"],
  Cell[57117, 1653, 1594, 38, 256, "Output",
   CellTags->"b:2.0.0"],
  Cell[58726, 1694, 159, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[58910, 1703, 496, 15, 46, "Input",
   CellTags->"b:2.0.0"],
  Cell[59409, 1720, 2790, 72, 286, "Output",
   CellTags->"b:2.0.0"],
  Cell[62214, 1795, 140, 4, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[62379, 1803, 823, 24, 78, "Input",
   CellTags->"b:2.0.0"],
  Cell[63205, 1829, 1244, 35, 286, "Output",
   CellTags->"b:2.0.0"],
  Cell[64452, 1866, 1012, 37, 286, "Output",
   CellTags->"b:2.0.0"],
  Cell[65479, 1906, 180, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[65696, 1916, 110, 2, 34, "Subsubsection",
   CellTags->{"c:5", "b:2.0.0"}],
  Cell[65809, 1920, 177, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[66011, 1929, 775, 24, 131, "Input",
   CellTags->"b:2.0.0"],
  Cell[66789, 1955, 2166, 49, 226, "Output",
   CellTags->"b:2.0.0"],
  Cell[68970, 2007, 159, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[69154, 2016, 1714, 49, 160, "Input",
   CellTags->"b:2.0.0"],
  Cell[70871, 2067, 1863, 43, 211, "Output",
   CellTags->"b:2.0.0"],
  Cell[72737, 2112, 2082, 47, 213, "Output",
   CellTags->"b:2.0.0"],
  Cell[74834, 2162, 114, 2, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[74973, 2168, 936, 27, 112, "Input",
   CellTags->"b:2.0.0"],
  Cell[75912, 2197, 2080, 47, 243, "Output",
   CellTags->"b:2.0.0"],
  Cell[78007, 2247, 551, 11, 123, "Text",
   CellTags->"b:2.0.0"],
  Cell[78583, 2262, 258, 7, 62, "Input",
   CellTags->"b:2.0.0"],
  Cell[78844, 2271, 2084, 47, 243, "Output",
   CellTags->"b:2.0.0"],
  Cell[80977, 2324, 108, 2, 34, "Subsubsection",
   CellTags->{"c:6", "b:2.0.0"}],
  Cell[81110, 2330, 607, 18, 105, "Input",
   CellTags->"b:2.0.0"],
  Cell[81720, 2350, 1421, 37, 239, "Output",
   CellTags->"b:2.0.0"],
  Cell[83156, 2390, 159, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[83340, 2399, 1714, 49, 160, "Input",
   CellTags->"b:2.0.0"],
  Cell[85057, 2450, 1400, 35, 211, "Output",
   CellTags->"b:2.0.0"],
  Cell[86460, 2487, 1387, 35, 213, "Output",
   CellTags->"b:2.0.0"],
  Cell[87862, 2525, 114, 2, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[88001, 2531, 1035, 30, 112, "Input",
   CellTags->"b:2.0.0"],
  Cell[89039, 2563, 1383, 35, 266, "Output",
   CellTags->"b:2.0.0"],
  Cell[90471, 2604, 110, 2, 34, "Subsubsection",
   CellTags->{"c:7", "b:2.0.0"}],
  Cell[90606, 2610, 680, 21, 105, "Input",
   CellTags->"b:2.0.0"],
  Cell[91289, 2633, 1531, 39, 230, "Output",
   CellTags->"b:2.0.0"],
  Cell[92835, 2675, 159, 5, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[93019, 2684, 1714, 49, 160, "Input",
   CellTags->"b:2.0.0"],
  Cell[94736, 2735, 3456, 65, 211, "Output",
   CellTags->"b:2.0.0"],
  Cell[98195, 2802, 1953, 44, 213, "Output",
   CellTags->"b:2.0.0"],
  Cell[100163, 2849, 114, 2, 23, "Text",
   CellTags->"b:2.0.0"],
  Cell[100302, 2855, 936, 27, 112, "Input",
   CellTags->"b:2.0.0"],
  Cell[101241, 2884, 1521, 38, 251, "Output",
   CellTags->"b:2.0.0"],
  Cell[102777, 2925, 141, 4, 23, "Text",
   CellTags->"b:2.0.0"]},
 "S5.1.1"->{
  Cell[41021, 1143, 2996, 84, 61, "DefinitionBox",
   CellTags->{"S5.1.1", "b:2.0.0"}]},
 "c:4"->{
  Cell[55021, 1584, 188, 7, 34, "Subsubsection",
   CellTags->{"c:4", "b:2.0.0"}]},
 "c:5"->{
  Cell[65696, 1916, 110, 2, 34, "Subsubsection",
   CellTags->{"c:5", "b:2.0.0"}]},
 "c:6"->{
  Cell[80977, 2324, 108, 2, 34, "Subsubsection",
   CellTags->{"c:6", "b:2.0.0"}]},
 "c:7"->{
  Cell[90471, 2604, 110, 2, 34, "Subsubsection",
   CellTags->{"c:7", "b:2.0.0"}]},
 "c:8"->{
  Cell[102967, 2935, 145, 3, 58, "Subsection",
   CellTags->{"c:8", "b:2.0.1"}]},
 "b:2.0.1"->{
  Cell[102967, 2935, 145, 3, 58, "Subsection",
   CellTags->{"c:8", "b:2.0.1"}],
  Cell[103115, 2940, 380, 7, 39, "Text",
   CellTags->"b:2.0.1"],
  Cell[103498, 2949, 539, 19, 51, "Text",
   CellTags->"b:2.0.1"],
  Cell[104040, 2970, 1476, 42, 85, "Input",
   CellTags->"b:2.0.1"],
  Cell[105519, 3014, 123, 2, 23, "Text",
   CellTags->"b:2.0.1"],
  Cell[105645, 3018, 813, 24, 62, "Input",
   CellTags->"b:2.0.1"],
  Cell[106461, 3044, 177, 4, 23, "Text",
   CellTags->"b:2.0.1"],
  Cell[106641, 3050, 1101, 32, 62, "Input",
   CellTags->"b:2.0.1"],
  Cell[107745, 3084, 786, 25, 46, "Input",
   CellTags->"b:2.0.1"],
  Cell[108534, 3111, 598, 18, 46, "Input",
   CellTags->"b:2.0.1"],
  Cell[109135, 3131, 114, 2, 23, "Text",
   CellTags->"b:2.0.1"],
  Cell[109252, 3135, 2561, 67, 126, "Input",
   CellTags->"b:2.0.1"],
  Cell[111816, 3204, 235, 6, 23, "Text",
   CellTags->"b:2.0.1"],
  Cell[112054, 3212, 336, 9, 46, "Input",
   CellTags->"b:2.0.1"],
  Cell[112393, 3223, 117, 2, 23, "Text",
   CellTags->"b:2.0.1"],
  Cell[112535, 3229, 474, 12, 78, "Input",
   CellTags->"b:2.0.1"],
  Cell[113012, 3243, 295, 7, 45, "Output",
   CellTags->"b:2.0.1"],
  Cell[113322, 3253, 180, 5, 23, "Text",
   CellTags->"b:2.0.1"],
  Cell[113527, 3262, 256, 6, 46, "Input",
   CellTags->"b:2.0.1"],
  Cell[113786, 3270, 34373, 666, 286, "Output",
   CellTags->"b:2.0.1"],
  Cell[148196, 3941, 120, 3, 46, "Input",
   CellTags->"b:2.0.1"],
  Cell[148319, 3946, 2261, 51, 286, "Output",
   CellTags->"b:2.0.1"],
  Cell[150595, 4000, 52, 1, 45, "Input",
   CellTags->"b:2.0.1"],
  Cell[150650, 4003, 1005, 29, 35, "Abstract",
   CellTags->"b:2.0.1"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"c:1", 152788, 4065},
 {"c:2", 152856, 4068},
 {"c:3", 152928, 4071},
 {"b:2.0.0", 153020, 4074},
 {"S5.1.1", 157987, 4219},
 {"c:4", 158086, 4222},
 {"c:5", 158180, 4225},
 {"c:6", 158274, 4228},
 {"c:7", 158368, 4231},
 {"c:8", 158462, 4234},
 {"b:2.0.1", 158558, 4237}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 60, 1, 51, "Title",
 CellTags->"c:1"],
Cell[608, 23, 45, 0, 35, "Subtitle"],
Cell[656, 25, 369, 9, 20, "Abstract"],
Cell[1028, 36, 155, 2, 22, "Text"],
Cell[CellGroupData[{
Cell[1208, 42, 303, 7, 51, "Section",
 CellTags->"c:2"],
Cell[CellGroupData[{
Cell[1536, 53, 135, 3, 58, "Subsection",
 CellTags->{"c:3", "b:2.0.0"}],
Cell[1674, 58, 1005, 29, 35, "Abstract",
 CellTags->"b:2.0.0"],
Cell[2682, 89, 324, 7, 51, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[3031, 100, 133, 3, 46, "Input",
 CellTags->"b:2.0.0"],
Cell[3167, 105, 291, 5, 22, "Print",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3495, 115, 112, 3, 46, "Input",
 CellTags->"b:2.0.0"],
Cell[3610, 120, 33520, 937, 495, "Print",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37167, 1062, 311, 9, 62, "Input",
 CellTags->"b:2.0.0"],
Cell[37481, 1073, 3303, 59, 184, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[40799, 1135, 219, 6, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[41021, 1143, 2996, 84, 61, "DefinitionBox",
 CellTags->{"S5.1.1", "b:2.0.0"}],
Cell[44020, 1229, 461, 17, 54, "Text",
 CellTags->"b:2.0.0"],
Cell[44484, 1248, 451, 14, 53, "Text",
 CellTags->"b:2.0.0"],
Cell[44938, 1264, 1203, 46, 83, "Text",
 CellTags->"b:2.0.0"],
Cell[46144, 1312, 458, 15, 46, "Input",
 CellTags->"b:2.0.0"],
Cell[46605, 1329, 224, 5, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[46832, 1336, 2187, 57, 125, "Input",
 CellTags->"b:2.0.0"],
Cell[49022, 1395, 2150, 58, 110, "Input",
 CellTags->"b:2.0.0"],
Cell[51175, 1455, 649, 13, 123, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[51849, 1472, 186, 5, 45, "Input",
 CellTags->"b:2.0.0"],
Cell[52038, 1479, 2141, 69, 294, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[54194, 1551, 278, 9, 27, "Text",
 CellTags->"b:2.0.0"],
Cell[54475, 1562, 102, 2, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[54602, 1568, 84, 2, 45, "Input",
 CellTags->"b:2.0.0"],
Cell[54689, 1572, 295, 7, 45, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55021, 1584, 188, 7, 34, "Subsubsection",
 CellTags->{"c:4", "b:2.0.0"}],
Cell[CellGroupData[{
Cell[55234, 1595, 568, 17, 105, "Input",
 CellTags->"b:2.0.0"],
Cell[55805, 1614, 1309, 37, 286, "Output",
 CellTags->"b:2.0.0"],
Cell[57117, 1653, 1594, 38, 256, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[58726, 1694, 159, 5, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[58910, 1703, 496, 15, 46, "Input",
 CellTags->"b:2.0.0"],
Cell[59409, 1720, 2790, 72, 286, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[62214, 1795, 140, 4, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[62379, 1803, 823, 24, 78, "Input",
 CellTags->"b:2.0.0"],
Cell[63205, 1829, 1244, 35, 286, "Output",
 CellTags->"b:2.0.0"],
Cell[64452, 1866, 1012, 37, 286, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[65479, 1906, 180, 5, 23, "Text",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[65696, 1916, 110, 2, 34, "Subsubsection",
 CellTags->{"c:5", "b:2.0.0"}],
Cell[65809, 1920, 177, 5, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[66011, 1929, 775, 24, 131, "Input",
 CellTags->"b:2.0.0"],
Cell[66789, 1955, 2166, 49, 226, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[68970, 2007, 159, 5, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[69154, 2016, 1714, 49, 160, "Input",
 CellTags->"b:2.0.0"],
Cell[70871, 2067, 1863, 43, 211, "Output",
 CellTags->"b:2.0.0"],
Cell[72737, 2112, 2082, 47, 213, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[74834, 2162, 114, 2, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[74973, 2168, 936, 27, 112, "Input",
 CellTags->"b:2.0.0"],
Cell[75912, 2197, 2080, 47, 243, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[78007, 2247, 551, 11, 123, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[78583, 2262, 258, 7, 62, "Input",
 CellTags->"b:2.0.0"],
Cell[78844, 2271, 2084, 47, 243, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[80977, 2324, 108, 2, 34, "Subsubsection",
 CellTags->{"c:6", "b:2.0.0"}],
Cell[CellGroupData[{
Cell[81110, 2330, 607, 18, 105, "Input",
 CellTags->"b:2.0.0"],
Cell[81720, 2350, 1421, 37, 239, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[83156, 2390, 159, 5, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[83340, 2399, 1714, 49, 160, "Input",
 CellTags->"b:2.0.0"],
Cell[85057, 2450, 1400, 35, 211, "Output",
 CellTags->"b:2.0.0"],
Cell[86460, 2487, 1387, 35, 213, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[87862, 2525, 114, 2, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[88001, 2531, 1035, 30, 112, "Input",
 CellTags->"b:2.0.0"],
Cell[89039, 2563, 1383, 35, 266, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[90471, 2604, 110, 2, 34, "Subsubsection",
 CellTags->{"c:7", "b:2.0.0"}],
Cell[CellGroupData[{
Cell[90606, 2610, 680, 21, 105, "Input",
 CellTags->"b:2.0.0"],
Cell[91289, 2633, 1531, 39, 230, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[92835, 2675, 159, 5, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[93019, 2684, 1714, 49, 160, "Input",
 CellTags->"b:2.0.0"],
Cell[94736, 2735, 3456, 65, 211, "Output",
 CellTags->"b:2.0.0"],
Cell[98195, 2802, 1953, 44, 213, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[100163, 2849, 114, 2, 23, "Text",
 CellTags->"b:2.0.0"],
Cell[CellGroupData[{
Cell[100302, 2855, 936, 27, 112, "Input",
 CellTags->"b:2.0.0"],
Cell[101241, 2884, 1521, 38, 251, "Output",
 CellTags->"b:2.0.0"]
}, Open  ]],
Cell[102777, 2925, 141, 4, 23, "Text",
 CellTags->"b:2.0.0"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[102967, 2935, 145, 3, 58, "Subsection",
 CellTags->{"c:8", "b:2.0.1"}],
Cell[103115, 2940, 380, 7, 39, "Text",
 CellTags->"b:2.0.1"],
Cell[103498, 2949, 539, 19, 51, "Text",
 CellTags->"b:2.0.1"],
Cell[104040, 2970, 1476, 42, 85, "Input",
 CellTags->"b:2.0.1"],
Cell[105519, 3014, 123, 2, 23, "Text",
 CellTags->"b:2.0.1"],
Cell[105645, 3018, 813, 24, 62, "Input",
 CellTags->"b:2.0.1"],
Cell[106461, 3044, 177, 4, 23, "Text",
 CellTags->"b:2.0.1"],
Cell[106641, 3050, 1101, 32, 62, "Input",
 CellTags->"b:2.0.1"],
Cell[107745, 3084, 786, 25, 46, "Input",
 CellTags->"b:2.0.1"],
Cell[108534, 3111, 598, 18, 46, "Input",
 CellTags->"b:2.0.1"],
Cell[109135, 3131, 114, 2, 23, "Text",
 CellTags->"b:2.0.1"],
Cell[109252, 3135, 2561, 67, 126, "Input",
 CellTags->"b:2.0.1"],
Cell[111816, 3204, 235, 6, 23, "Text",
 CellTags->"b:2.0.1"],
Cell[112054, 3212, 336, 9, 46, "Input",
 CellTags->"b:2.0.1"],
Cell[112393, 3223, 117, 2, 23, "Text",
 CellTags->"b:2.0.1"],
Cell[CellGroupData[{
Cell[112535, 3229, 474, 12, 78, "Input",
 CellTags->"b:2.0.1"],
Cell[113012, 3243, 295, 7, 45, "Output",
 CellTags->"b:2.0.1"]
}, Open  ]],
Cell[113322, 3253, 180, 5, 23, "Text",
 CellTags->"b:2.0.1"],
Cell[CellGroupData[{
Cell[113527, 3262, 256, 6, 46, "Input",
 CellTags->"b:2.0.1"],
Cell[113786, 3270, 34373, 666, 286, "Output",
 CellTags->"b:2.0.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148196, 3941, 120, 3, 46, "Input",
 CellTags->"b:2.0.1"],
Cell[148319, 3946, 2261, 51, 286, "Output",
 CellTags->"b:2.0.1"]
}, Open  ]],
Cell[150595, 4000, 52, 1, 45, "Input",
 CellTags->"b:2.0.1"],
Cell[150650, 4003, 1005, 29, 35, "Abstract",
 CellTags->"b:2.0.1"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
