(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    333649,       9481]
NotebookOptionsPosition[    282686,       7850]
NotebookOutlinePosition[    311428,       8725]
CellTagsIndexPosition[    309599,       8653]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Shor's factoring algorithms", "Title",
 CellTags->"c:1"],

Cell["Four examples of Shor's factoring algorithm", "Subtitle"],

Cell[TextData[{
 "Version: 4.0-July-2014--For ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 9.0,10."
}], "Abstract",
 CellChangeTimes->{{3.395090887981901*^9, 3.395090925377634*^9}, {
   3.512204933064251*^9, 3.5122049705968847`*^9}, 3.5122120029978447`*^9, {
   3.614175844992116*^9, 3.614175864474527*^9}}],

Cell[CellGroupData[{

Cell["\<\
Four examples of Shor's factoring algorithm                        To open \
double click at down arrow---------------------------->>\
\>", "Section",
 CellChangeTimes->{{3.512205126506694*^9, 3.51220512963229*^9}},
 FontFamily->"Times",
 CellTags->{"c:2", "b:3.0"}],

Cell[TextData[{
 "Last Version: 18-Aug-2005\n",
 Cell[BoxData[
  FormBox[GridBox[{
     {
      RowBox[{"References", ":"}], 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"[", "1", "]"}], " ", "Peter", " ", 
         RowBox[{"W", ".", " ", "Shor"}]}], ",", " ", 
        RowBox[{"SIAM", " ", 
         RowBox[{"J", ".", " ", "COMPUT"}], " ", "26", " ", 
         RowBox[{"(", "5", ")"}], " ", "1484", " ", 
         RowBox[{"(", "1997", ")"}]}]}], "\n", 
       RowBox[{
        RowBox[{"[", "2", "]"}], " ", "Edward", " ", "Gerjuoy", " ", 
        RowBox[{"Am", ".", " ", "J", ".", " ", "Phys", ".", " ", "73"}], " ", 
        RowBox[{"(", "6", ")"}], " ", "521", " ", 
        RowBox[{"(", "2005", ")"}]}], "\n", 
       RowBox[{
        RowBox[{
         RowBox[{"[", "3", "]"}], " ", "QDENSITY", " ", "Manual"}], ",", " ", 
        RowBox[{"Sect", ".", " ", "9"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
      "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], TraditionalForm]], 
  "DefinitionBox",
  GridBoxOptions->{
  GridBoxDividers->{
   "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
    "RowsIndexed" -> {}},
  GridBoxSpacings->{"Columns" -> {
      Offset[0.27999999999999997`], {
       Offset[0.5599999999999999]}, 
      Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
      Offset[0.2], {
       Offset[0.16]}, 
      Offset[0.2]}, "RowsIndexed" -> {}}},
  CellTags->"S5.1.1"],
 "\n\nWe present four different implementations of the factoring algorithm. \
The differences between them arise from the method used to perform the \
Quantum Fourier Transform. All of them follow essentially the same steps and \
mostly share the same comments and explanations.\n\n(1) The first case is \
closer to an experiment implementation, using a density matrix, where the QFT \
is explicitly constructed quantum mechanically using Hadamards and Controlled \
gates. \n\n(2) In the second case, a state vector formulation is used , \
instead of a density matrix, to compute the QFT.  This approach saves space, \
as explained in ",
 ButtonBox["QFT.nb",
  BaseStyle->"Hyperlink",
  ButtonData:>{"QFT.nb", None}],
 ". \n\n(3) The third case, is based on the second approach but also \
explictly constructs the state vectors for both registers, and uses the state \
vector formula for QFT.\n\n(4) Finally, if one is interested in analyzing the \
probabilities for factoring larger numbers, and not in a voracious, closer to \
reality implementation of the QFT, then use this version which replaces QFT \
by  Fourier and InverseFourier, the built-in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for Discreet Fourier Transforms (DFT), check ",
 ButtonBox["QFT.nb",
  BaseStyle->"Hyperlink",
  ButtonData:>{"QFT.nb", None}],
 "."
}], "Abstract",
 CellTags->"b:3.0"],

Cell[TextData[ButtonBox["Click here to get the 2014 palette: QDENSpalette.nb",
 BaseStyle->"Hyperlink",
 ButtonData:>{"QDENSpalette14.nb", None}]], "Text",
 CellTags->"b:3.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shor's Factoring algorithm using density matrix", "Section",
 FontFamily->"Times",
 CellTags->"c:2"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and NumberTheory\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:3", "b:3.1.0"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.1.0"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{{3.62533217597893*^9, 3.625332242304626*^9}, 
   3.655402751860746*^9, 3.65540334660738*^9, 3.657377240725655*^9},
 CellTags->"b:3.1.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["QDENSITY`Qdensity`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["QDENSITY`Qdensity`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["about",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "about", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["FidelityB",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "FidelityB", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nq",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "nq", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res1$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "res1$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ts$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ts$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Adj",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Adj", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["for",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "for", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Observables",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Observables", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "res2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["TwoOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "TwoOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["AF",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "AF", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["had",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "had", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["OneOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "OneOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res2$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "res2$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["U",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "U", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["at",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "at", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Had",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Had", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Oracle",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Oracle", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Rotqbit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Rotqbit", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["v",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "v", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Bra1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Bra1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Hadamard",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Hadamard", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTrace",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PartialTrace", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotX",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RotX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Vs",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Vs", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "BraV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["HALL",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "HALL", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTraceGen",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PartialTraceGen", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RotY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["v$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "v$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "BraX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["i8",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "i8", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTranspose",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PartialTranspose", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotZ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RotZ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ws",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ws", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "BraY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["intro",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "intro", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ProjG",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ProjG", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["s",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "s", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["xb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "xb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CNOT",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "CNOT", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["j8",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "j8", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PTr",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PTr", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Schmidt",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Schmidt", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Xb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Xb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Coeff",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Coeff", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Ket1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Ket1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PTrGen",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PTrGen", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma0",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Sigma0", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["xd",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "xd", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Comm",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Comm", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "KetV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Purity",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Purity", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Sigma1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Xd",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Xd", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CompMeasureKet",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "CompMeasureKet", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "KetX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Sigma2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ZeroMatrix",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ZeroMatrix", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CompPTr",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "CompPTr", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "KetY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "PY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma3",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Sigma3", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ZeroT",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ZeroT", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledX",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ControlledX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["look",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "look", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["qubit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "qubit", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Snos",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Snos", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[ScriptCapitalH]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ControlledY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["matrices",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "matrices", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBra1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomBra1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["SP",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "SP", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[ScriptCapitalP]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledZ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ControlledZ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mc",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "mc", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBra2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomBra2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Spin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Spin", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[DoubleStruckCapitalI]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CPHASE",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "CPHASE", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mcl",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "mcl", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBraN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomBraN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["SpinObs",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "SpinObs", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Rho]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Rho]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CPHASEV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "CPHASEV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mcl$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "mcl$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKet1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomKet1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["stA",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "stA", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Rho]$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Rho]$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["cRQN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "cRQN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mc$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "mc$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKet2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomKet2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["stB",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "stB", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Sigma]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Sigma]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Definef",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Definef", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MeasureKet",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "MeasureKet", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKetN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomKetN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Swap",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Swap", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]A1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Phi]A1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["density",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "density", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MultiProjCBasis",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "MultiProjCBasis", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubit1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomQubit1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["s$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "s$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]A2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Phi]A2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["DForm2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "DForm2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MultiQProj",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "MultiQProj", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubit2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomQubit2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["TensorProductQ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "TensorProductQ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]B1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Phi]B1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Discord",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Discord", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Mutual",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Mutual", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubitN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "RandomQubitN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ThreeOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ThreeOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]B2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[Phi]B2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["EnTropy",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "EnTropy", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "nb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["rcsDate",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "rcsDate", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["to",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "to", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[CapitalOmega]s",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "\[CapitalOmega]s", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["EnTropy2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "EnTropy2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["neg",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "neg", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["rcsRevision",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "rcsRevision", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Toffoli",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Toffoli", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["Fidelity",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "Fidelity", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nneg",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "nneg", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "res1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ts",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info-9e366c49-a106-4348-a81d-def94429f2b3", {
            "ts", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.19]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.65737724132514*^9},
 CellTags->"b:3.1.0"]
}, Open  ]],

Cell["\<\
To improve performance we precompile the quantum Fourier transform.  We \
prepare some utilities first, for alter application:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ControlP", "[", 
    RowBox[{"L_", ",", "q1_", ",", "q2_", ",", "k_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[Phi]", "}"}], ",", 
     RowBox[{
      RowBox[{"\[Phi]", "=", 
       RowBox[{"Exp", "[", 
        RowBox[{"2", "*", " ", "\[Pi]", " ", "*", " ", 
         RowBox[{"\[ImaginaryI]", " ", "/", 
          SuperscriptBox["2", "k"]}]}], "]"}]}], ";", 
      RowBox[{
       RowBox[{"1", "/", "4"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"3", "+", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "0", ",", "0"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "3", ",", "0"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "0", ",", "3"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"\[Phi]", "-", "1"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "3", ",", "3"}], "]"}]}]}],
         ")"}]}]}]}], "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"InvertQubits", "[", "L_", "]"}], ":=", 
  RowBox[{
   RowBox[{"InvertQubits", "[", "L", "]"}], "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Reverse", "[", 
         RowBox[{"IntegerDigits", "[", 
          RowBox[{"i", ",", "2", ",", "L"}], "]"}], "]"}], "\[Equal]", " ", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"j", ",", "2", ",", "L"}], "]"}]}], ",", "1", ",", "0"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "L"}], "-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "L"}], "-", "1"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompControlP", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q1", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q2", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Lv", "=", "L"}], ",", 
         RowBox[{"qv1", "=", "q1"}], ",", 
         RowBox[{"qv2", "=", "q2"}], ",", 
         RowBox[{"kv", "=", "k"}]}], "  ", "}"}], ",", " ", 
       RowBox[{"ControlP", "[", 
        RowBox[{"Lv", ",", "qv1", ",", "qv2", ",", "kv"}], "]"}]}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"ControlP", "[", 
         RowBox[{"Lv", ",", "qv1", ",", "qv2", ",", "kv"}], "]"}], ",", 
        "_Complex", ",", "2"}], "}"}], "}"}]}], "]"}]}], " ", ";"}]], "Input",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompHad", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Lv", "=", "L"}], ",", 
         RowBox[{"iv", "=", "i"}]}], "}"}], ",", " ", 
       RowBox[{"had", "[", 
        RowBox[{"Lv", ",", "iv"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"had", "[", 
         RowBox[{"Lv", ",", "iv"}], "]"}], ",", "_Real", ",", "2"}], "}"}], 
      "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompInvertQubits", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"L", ",", "_Integer"}], "}"}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Lv", "=", "L"}], " ", "}"}], ",", 
       RowBox[{"InvertQubits", "[", "Lv", "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "  ", 
       RowBox[{
        RowBox[{"InvertQubits", "[", "Lv", "]"}], ",", "_Real", ",", "2"}], 
       "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumFourierTransform", "=", 
    RowBox[{"Compile", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[CapitalOmega]s", ",", "_Complex", ",", "2"}], "}"}]}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Module", "[", 
       RowBox[{
        RowBox[{"{", "\[CapitalOmega]", "}"}], ",", 
        RowBox[{
         RowBox[{"\[CapitalOmega]", "=", "\[CapitalOmega]s"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"i", "=", "1"}], ",", 
           RowBox[{"i", "\[LessEqual]", "L"}], ",", 
           RowBox[{"i", "++"}], ",", 
           RowBox[{
            RowBox[{"\[CapitalOmega]", "=", 
             RowBox[{"N", "[", 
              RowBox[{
               RowBox[{"CompHad", "[", 
                RowBox[{"L", ",", "i"}], "]"}], ".", "\[CapitalOmega]"}], 
              "]"}]}], ";", 
            RowBox[{"For", "[", " ", 
             RowBox[{
              RowBox[{"j", "=", 
               RowBox[{"i", "+", "1"}]}], ",", 
              RowBox[{"j", "\[LessEqual]", "L"}], ",", 
              RowBox[{"j", "++"}], ",", 
              RowBox[{"\[CapitalOmega]", "=", 
               RowBox[{"N", "[", 
                RowBox[{
                 RowBox[{"CompControlP", "[", 
                  RowBox[{"L", ",", "j", ",", "i", ",", 
                   RowBox[{"j", "-", "i", "+", "1"}]}], "]"}], ".", 
                 "\[CapitalOmega]"}], "]"}]}]}], "]"}]}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"\[CapitalOmega]", "=", 
          RowBox[{
           RowBox[{"CompInvertQubits", "[", "L", "]"}], ".", 
           "\[CapitalOmega]"}]}], ";", "\[CapitalOmega]"}]}], "]"}], ",", " ",
       "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompHad", "[", 
           RowBox[{"L", ",", "i"}], "]"}], ",", "_Real", ",", "2"}], "}"}], 
        ",", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompControlP", "[", 
           RowBox[{"L", ",", "j", ",", "i", ",", 
            RowBox[{"j", "-", "i", "+", "1"}]}], "]"}], ",", "_Complex", ",", 
          "2"}], "}"}], ",", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompInvertQubits", "[", "L", "]"}], ",", "_Real", ",", 
          "2"}], "}"}]}], "}"}]}], " ", "]"}]}], ";"}], "  "}]], "Input",
 CellTags->"b:3.1.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.1.0"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.0"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.1.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers needed\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:4", "b:3.1.1"}],

Cell["M is the number we are trying to factor:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"M", "=", "21"}], ";"}]], "Input",
 CellTags->"b:3.1.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.1.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime, in which case we quit the process:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
     RowBox[{"Even", "=", "True"}], ";", 
     RowBox[{"MP", "=", 
      RowBox[{"MP", "/", "2"}]}]}]}], "]"}], ";"}]], "Input",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"MP", "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"M", ",", " ", "\"\< is a power of 2\>\""}], "]"}], ";", 
    RowBox[{"Quit", "[", "]"}]}], ","}], "]"}]], "Input",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.1.1"],

Cell[TextData[{
 " We need the ",
 StyleBox["minimun number",
  FontWeight->"Bold"],
 " of qubits for both registers. \nHere we should be careful, a high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " number will lengthen the computation.  \nBy default we give estimates for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 ",based on the fact that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " should be able to \nrepresent numbers up to M^2, while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 "should allow to write up to M/2. However, in \nmany cases it could be more \
illustrative to lower the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " \n[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 TagBox["512",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.62533217628399*^9, 3.6253322425902433`*^9}, 
   3.6554027526782513`*^9, 3.655403347415576*^9, 3.65737724145536*^9},
 CellTags->"b:3.1.1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["n", "1"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["n", "2"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["n", "1"], "=", "6"}], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellTags->"b:3.1.1"],

Cell[BoxData[
 TagBox["6",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332176299777*^9, 3.625332242610799*^9}, 
   3.655402752759913*^9, 3.655403347521124*^9, 3.6573772414676943`*^9},
 CellTags->"b:3.1.1"],

Cell[BoxData[
 TagBox["5",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332176299777*^9, 3.625332242610799*^9}, 
   3.655402752759913*^9, 3.655403347521124*^9, 3.657377241471406*^9},
 CellTags->"b:3.1.1"]
}, Open  ]],

Cell["\<\
Gerjuoy's test: a failure to pass this test  means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.1.1"],

Cell[BoxData["\<\"CHECK YOUR \\!\\(\\*SubscriptBox[\\(n\\), \\(1\\)]\\), YOU \
MAY HAVE PROBLEMS GETTING THE PERIOD\"\>"], "Print",
 CellChangeTimes->{{3.625332176334381*^9, 3.625332242628127*^9}, 
   3.655402752877487*^9, 3.655403347666133*^9, 3.65737724149271*^9},
 CellTags->"b:3.1.1"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]]],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:5", "b:3.1.2"}],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]]],
 " ; which is a \nstate where all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]]]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:6", "b:3.1.3"}],

Cell[" This is the random Coprime to MP: ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.1.3"],

Cell[BoxData[
 TagBox["5",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6253321764390783`*^9, 3.625332242658229*^9}, 
   3.655402752944232*^9, 3.655403347749587*^9, 3.6573772415066442`*^9},
 CellTags->"b:3.1.3"]
}, Open  ]],

Cell[TextData[{
 " The full wave function after we build the second register with  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]]],
 "reads:\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"|", 
         SubscriptBox["\[Psi]", "12"]}], "\[RightAngleBracket]"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{
           RowBox[{"-", 
            SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           SuperscriptBox["2", 
            SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
      "\[RightAngleBracket]"}], "|", 
     SubscriptBox["f", "j"]}], "  ", "\[RightAngleBracket]"}], 
   TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["xguess", "j"], 
     RowBox[{"(", 
      RowBox[{"mod", " ", "M"}], ")"}]}]}], TraditionalForm]]]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:7", "b:3.1.4"}],

Cell[TextData[{
 "We take an hypothetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]]],
 "]"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.1.4"],

Cell[BoxData[
 TagBox["16",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332176467925*^9, 3.625332242670425*^9}, 
   3.6554027530101223`*^9, 3.655403348002787*^9, 3.65737724152669*^9},
 CellTags->"b:3.1.4"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SuperscriptBox["2", 
  SubscriptBox["n", "1"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  SubscriptBox["n", "2"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], ")"}]]}], "Input",
 CellTags->"b:3.1.4"],

Cell[BoxData[
 TagBox["64",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6253321764797497`*^9, 3.625332242690426*^9}, 
   3.655402753249936*^9, 3.65540334811617*^9, 3.657377241539637*^9},
 CellTags->"b:3.1.4"],

Cell[BoxData[
 TagBox["32",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6253321764797497`*^9, 3.625332242690426*^9}, 
   3.655402753249936*^9, 3.65540334811617*^9, 3.657377241543166*^9},
 CellTags->"b:3.1.4"],

Cell[BoxData[
 TagBox["2048",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6253321764797497`*^9, 3.625332242690426*^9}, 
   3.655402753249936*^9, 3.65540334811617*^9, 3.6573772415456*^9},
 CellTags->"b:3.1.4"]
}, Open  ]],

Cell[TextData[{
 "The next function gives the number in the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 "register in decimal basis from a given number i in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["n", "1"],
     TraditionalForm], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 " bases."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xv", "[", "i_", "]"}], ":=", 
   RowBox[{"FromDigits", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}]}], "]"}], ",", 
       SubscriptBox["n", "1"]}], "]"}], ",", "2"}], "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.4"],

Cell[TextData[{
 "We select the positions in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 "register which remain when reg2 has been measured:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dens1", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Mod", "[", 
         RowBox[{
          RowBox[{"xguess", "^", 
           RowBox[{"xv", "[", "i", "]"}]}], ",", "MP"}], "]"}], "\[Equal]", 
        "reg2Measurement"}], ",", " ", 
       RowBox[{"dens1", "=", 
        RowBox[{"Join", "[", " ", 
         RowBox[{"dens1", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"xv", "[", "i", "]"}], "+", "1"}], " ", "}"}]}], "]"}]}], 
       ",", "\"\<\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        SuperscriptBox["2", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "1"], "+", 
           SubscriptBox["n", "2"]}], ")"}]], "-", "1"}], ",", 
       SuperscriptBox["2", 
        SubscriptBox["n", "2"]]}], " ", "}"}]}], "]"}], ";"}], 
  " "}]}], "Input",
 CellTags->"b:3.1.4"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"reg1", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"reg1", ",", "1", ",", 
      RowBox[{"dens1", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "dens1", "]"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"reg1", "/", 
    RowBox[{"Norm", "[", "reg1", "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.1.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"densx", "=", 
   RowBox[{"Distribute", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"dens1", ",", "dens1"}], "}"}], ",", "List"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.1.4"],

Cell["Share some memory.:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", " ", "]"}], ";"}]], "Input",
 CellTags->"b:3.1.4"],

Cell["Prepere desnity matrix for reg1", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rho1", " ", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"m", ",", "_Integer", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "_Integer", ",", "2"}], "}"}]}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "t", "}"}], ",", 
       RowBox[{
        RowBox[{"t", "=", "n"}], ";", " ", 
        RowBox[{"ReplacePart", "[", 
         RowBox[{"m", ",", "1", ",", "densx"}], "]"}]}]}], "]"}]}], "   ", 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{3.395094265479423*^9},
 CellTags->"b:3.1.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]1", " ", "=", 
   RowBox[{
    FractionBox["1", 
     SqrtBox[
      RowBox[{"Length", "[", " ", "densx", "]"}]]], 
    RowBox[{"rho1", "[", 
     RowBox[{
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]], ",", 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]}], "}"}]}], "]"}], ",", "densx"}], 
     "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.395094076233127*^9, 3.3950940974912663`*^9}},
 CellTags->"b:3.1.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", " ", "]"}], ";"}]], "Input",
 CellTags->"b:3.1.4"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:8", "b:3.1.5"}],

Cell["We apply the inverse of the quantum fourier transform to \[Rho]1:", \
"Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rho2", " ", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Complex", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Complex", ",", "2"}], "}"}]}], "}"}], ",", " ", 
     RowBox[{"N", "[", 
      RowBox[{
       RowBox[{"Conjugate", "[", "x", "]"}], ".", "y", ".", "x"}], "]"}]}], 
    "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]2", "=", " ", 
   RowBox[{"rho2", " ", "[", 
    RowBox[{
     RowBox[{"QuantumFourierTransform", "[", 
      RowBox[{
       SubscriptBox["n", "1"], ",", " ", 
       RowBox[{"N", "[", 
        RowBox[{"IdentityMatrix", "[", 
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "]"}], "]"}]}], "]"}], ",", " ", 
     "\[Rho]1"}], "  ", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.5"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< seconds\>\""}], 
  "]"}]}], "Input",
 CellTags->"b:3.1.5"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The QFT took \"\>", "\[InvisibleSpace]", 
   "0.430067`5.784051117280433", "\[InvisibleSpace]", "\<\" seconds\"\>"}],
  SequenceForm["The QFT took ", 0.430067`5.784051117280433, " seconds"],
  Editable->False]], "Print",
 CellChangeTimes->{{3.625332177001086*^9, 3.625332243252617*^9}, 
   3.655402754317539*^9, 3.655403349153571*^9, 3.657377242527336*^9},
 CellTags->"b:3.1.5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chop", "[", 
  RowBox[{"Tr", "[", "\[Rho]2", "]"}], "]"}]], "Input",
 CellTags->"b:3.1.5"],

Cell[BoxData[
 TagBox["0.9999999999999991`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.62533217701257*^9, 3.625332243264086*^9}, 
   3.655402754429933*^9, 3.655403349249757*^9, 3.657377242539837*^9},
 CellTags->"b:3.1.5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:9", "b:3.1.6"}],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.1.6"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{"\[Rho]2", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"num", "+", "1"}], ",", 
         RowBox[{"num", "+", "1"}]}], "]"}], "]"}], "]"}]}], ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.1.6"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.6"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.3`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.1.6"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], 
      LineBox[{{0.9026302117342296, 0.015851049491079044`}, {1., 
       0.0007057375947216824}, {2., 0.0007413467876771411}, {3., 
       0.0008070997807950853}, {4., 0.0009152913087920383}, {5., 
       0.001090700180602437}, {6., 0.0013864456170611754`}, {7., 
       0.0019324500467675118`}, {8., 0.0031249999999999976`}, {9., 
       0.006763862353264627}, {9.321359088567922, 0.015851049491079044`}}], 
      LineBox[{{11.901810187485902`, 0.015851049491079044`}, {12., 
       0.005334708691207957}, {13., 0.0011983420790088613`}, {14., 
       0.0003309836826591701}, {15., 0.00006494141625510448}, {16., 0.}, {17.,
        0.00006494141625510471}, {18., 0.00033098368265917005`}, {19., 
       0.0011983420790088611`}, {20., 0.005334708691207955}, {
       20.098189812514097`, 0.015851049491079044`}}], 
      LineBox[{{22.678640911432076`, 0.015851049491079044`}, {23., 
       0.006763862353264625}, {24., 0.0031249999999999976`}, {25., 
       0.001932450046767514}, {26., 0.001386445617061176}, {27., 
       0.0010907001806024356`}, {28., 0.0009152913087920379}, {29., 
       0.0008070997807950856}, {30., 0.0007413467876771406}, {31., 
       0.0007057375947216821}, {31.09736978826577, 0.015851049491079044`}}], 
      LineBox[{{32.90263021173423, 0.015851049491079044`}, {33., 
       0.0007057375947216824}, {34., 0.0007413467876771411}, {35., 
       0.0008070997807950853}, {36., 0.0009152913087920383}, {37., 
       0.001090700180602437}, {38., 0.0013864456170611754`}, {39., 
       0.0019324500467675118`}, {40., 0.0031249999999999976`}, {41., 
       0.006763862353264627}, {41.321359088567924`, 0.015851049491079044`}}], 
      LineBox[{{43.9018101874859, 0.015851049491079044`}, {44., 
       0.005334708691207957}, {45., 0.0011983420790088613`}, {46., 
       0.0003309836826591701}, {47., 0.00006494141625510448}, {48., 0.}, {49.,
        0.00006494141625510471}, {50., 0.00033098368265917005`}, {51., 
       0.0011983420790088611`}, {52., 0.005334708691207955}, {
       52.0981898125141, 0.015851049491079044`}}], 
      LineBox[{{54.678640911432076`, 0.015851049491079044`}, {55., 
       0.006763862353264625}, {56., 0.0031249999999999976`}, {57., 
       0.001932450046767514}, {58., 0.001386445617061176}, {59., 
       0.0010907001806024356`}, {60., 0.0009152913087920379}, {61., 
       0.0008070997807950856}, {62., 0.0007413467876771406}, {63., 
       0.0007057375947216821}}]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{},
   PlotRange->{{0, 63.}, {0, 0.015851049491079044`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332177188726*^9, 3.625332243364909*^9}, 
   3.6554027545979233`*^9, 3.655403349416091*^9, 3.657377242689536*^9},
 CellTags->"b:3.1.6"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd0dsrg3Ecx/GnFXHjUMosaZNTkuM2w3g+O9jRthxCSkoRJczhQiv9/ohd
cDklLdGSIikXcrhw486UlZIrF3OhpOWw/Kjv91dPT++enuf5/b4vw+TCwJRG
URT8XNn738p8Zde5+ltpdazw6rVW75at4OA9pkv5PLLzELY+RsNLPtlFsNzP
VOR8BmSXwDV6Nn2S6petxa4ltN1wNCy7HAVlhkVPfEK2HtHN7JqVXYnn5dja
3GFEdhWGbufHX9Ix2TX4eEhsxK9PZdchsH7zdpdclV2PS11EY94alN2ApDZR
mkmYZDdiJ9mUX51bLLsZClktOCbPW3FH3m/DBfm+EX7yfxPbnxmDZP/teCLn
s7Dzd0BL5tOJfTK/LqhkvlYYyfy7sUJ8erBH/FSMEF+A+AuA+tuov7Axfzv1
F3bm76D+wsH8nfT8wsn8e6m/6GXzdVF/4WL+buov3MzfA8IvPMzfS/2Fl/n7
qL/wsf35qb/wM/8+dv4+5h+g/iLA/IPUXwSZf4j6i9C//zeEARWM
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{},
   PlotRange->{{0, 63.}, {0, 0.3}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332177188726*^9, 3.625332243364909*^9}, 
   3.6554027545979233`*^9, 3.655403349416091*^9, 3.657377242728373*^9},
 CellTags->"b:3.1.6"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:10", "b:3.1.7"}],

Cell[TextData[{
 "Finally we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["xguess", 
    RowBox[{"r", "/", "2"}]], TraditionalForm]]],
 "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.1.7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.1.7"],

Cell[CellGroupData[{

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<xguess=\>\"", ",", "xguess"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{"TotalProbSucc", "\[Equal]", "0"}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<Your total probability was zero, you xguess was not very lucky!\>\"",
     "]"}]}], "]"}]}], "Input",
 CellTags->"b:3.1.7"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"Period", "Number", "\<\"f1\"\>", "\<\"f2\"\>", "Prob"},
     {"\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332177260748*^9, 3.6253322434286127`*^9}, 
   3.6554027548032713`*^9, 3.6554033496264763`*^9, 3.657377242810791*^9},
 CellTags->"b:3.1.7"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"xguess=\"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["xguess=", 5],
  Editable->False]], "Print",
 CellChangeTimes->{{3.625332177264585*^9, 3.625332243434225*^9}, 
   3.655402754863532*^9, 3.655403349688252*^9, 3.657377242814753*^9},
 CellTags->"b:3.1.7"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The TOTAL PROBABILITY OF SUCCESS is thus:\"\>", 
   "\[InvisibleSpace]", "0", "\[InvisibleSpace]", "\<\"  with xguess=\"\>", 
   "\[InvisibleSpace]", "5", 
   "\[InvisibleSpace]", "\<\" and measured register=\"\>", 
   "\[InvisibleSpace]", "16"}],
  SequenceForm[
  "The TOTAL PROBABILITY OF SUCCESS is thus:", 0, "  with xguess=", 5, 
   " and measured register=", 16],
  Editable->False]], "Print",
 CellChangeTimes->{{3.625332177264585*^9, 3.625332243434225*^9}, 
   3.655402754863532*^9, 3.655403349688252*^9, 3.657377242817204*^9},
 CellTags->"b:3.1.7"],

Cell[BoxData["\<\"Your total probability was zero, you xguess was not very \
lucky!\"\>"], "Print",
 CellChangeTimes->{{3.625332177264585*^9, 3.625332243434225*^9}, 
   3.655402754863532*^9, 3.655403349688252*^9, 3.65737724281966*^9},
 CellTags->"b:3.1.7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.1.7"],

Cell[BoxData[
 TagBox["8.377274`7.374647713873549",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332177277379*^9, 3.625332243458146*^9}, 
   3.655402755012768*^9, 3.655403349835878*^9, 3.6573772428445253`*^9},
 CellTags->"b:3.1.7"]
}, Open  ]],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.7"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       SuperscriptBox["xguess", "i"], ",", "M"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "60"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"a", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:3.1.7"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd0DkKwkAAheHBytLCwsJCRUREJHFfohn33cIjCNZeYY7sEVQyT5g/EB7/
kHzFVB+v+zNnjEm+72+z551mW7T+wG/Bb95vH+eRDf+L/ZZs6JbxXQVODX4d
fgN+E34Lfht+B36EjbFd+D34vIcBzofwR/DH8Cfwp/Bn8BP4c/gL+Cl8G/pO
LX8Z+k4tfxX6Ti1/DV8tfwNfLX8LXy1/B18tfw9fLf8AXy3/CF8t/wRfLf8M
Xy3/Al8t/wpfLf/29z9ib0V9
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 60.}, {0, 20.}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.625332177354452*^9, 3.625332243480536*^9}, 
   3.655402755097628*^9, 3.655403349924296*^9, 3.657377242876713*^9},
 CellTags->"b:3.1.7"]
}, Open  ]],

Cell["Quit", "Input",
 CellTags->"b:3.1.7"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Shor's Factoring algorithm using state vector formula for QFT\
\>", "Section",
 FontFamily->"Times",
 CellTags->"c:11"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and NumberTheory\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:12", "b:3.2.0"}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.2.0"],

Cell["\<\
To improve performance we precompile the quantum Fourier transform, now using \
state vectors:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.0"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Zeroff", "[", 
   RowBox[{"vac_", ",", "R_"}], "]"}], ":=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vac", "[", 
      RowBox[{"[", 
       RowBox[{"R", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "/", 
     RowBox[{"2", "^", "i"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "R", "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CompZeroff", " ", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"vac", ",", "_Integer", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"R", ",", "_Integer", ",", "1"}], "}"}]}], "}"}], ",", 
     RowBox[{"Zeroff", "[", 
      RowBox[{"vac", ",", "R"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Zeroff", "[", 
         RowBox[{"vac", ",", "R"}], "]"}], ",", "_Real"}], "}"}], "}"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"QuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"CompZeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[TensorProduct]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.614176064807555*^9, 3.614176074581315*^9}, {
  3.625187402849154*^9, 3.625187407890999*^9}, {3.6251874843020487`*^9, 
  3.625187490287272*^9}},
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex", 
        ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"IQuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"CompZeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{
            RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[TensorProduct]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.614176083784274*^9, 3.614176094504168*^9}},
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompIQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex", 
        ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.2.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.2.0"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.0"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.2.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:13", "b:3.2.1"}],

Cell["M is the number we are trying to factor:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{"13", "*", "17"}]}]], "Input",
 CellTags->"b:3.2.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.2.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime and quit if it is:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
     RowBox[{"Even", "=", "True"}], ";", 
     RowBox[{"MP", "=", 
      RowBox[{"MP", "/", "2"}]}]}]}], "]"}], ";"}]], "Input",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"MP", "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"M", ",", " ", "\"\< is a power of 2\>\""}], "]"}], ";", 
    RowBox[{"Quit", "[", "]"}]}], ","}], "]"}]], "Input",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.2.1"],

Cell[TextData[{
 " We need the ",
 StyleBox["minimun number",
  FontWeight->"Bold"],
 " of qubits for both registers. \nHere we should be careful, a high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " number will lengthen the computation.  \nBy default we give estimates for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 ",based on the fact that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " should be able to \nrepresent numbers up to M^2, while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 "should allow to write up to M/2. However,  in \nmany cases it could be more \
illustrative to lower the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " \n[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.2.1"],

Cell[TextData[{
 "Note the user should experiment with the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", 
    RowBox[{"1", "   "}]], TraditionalForm]]],
 "to reduce the computation time, especially for large M."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["n", "1"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["n", "2"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   SubscriptBox["n", "1"], "=", "9"}], " "}], "\[IndentingNewLine]", 
 SubscriptBox["n", "1"], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellChangeTimes->{{3.625187579649707*^9, 3.625187583890175*^9}, {
  3.625187644452126*^9, 3.625187649929905*^9}},
 CellTags->"b:3.2.1"],

Cell["\<\
Gerjuoy's test: a failure to pass this test  means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1"],

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.2.1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]]],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:14", "b:3.2.2"}],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]]],
 " ; which is a \nstate where all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]]]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:15", "b:3.2.3"}],

Cell[" This is the random Coprime to MP: ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.3"],

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.2.3"],

Cell[BoxData[
 RowBox[{"xguess", "=", "16"}]], "Input",
 CellTags->"b:3.2.3"],

Cell[TextData[{
 " The full wave function after we build the second register with  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]]],
 "reads:\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"|", 
         SubscriptBox["\[Psi]", "12"]}], "\[RightAngleBracket]"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{
           RowBox[{"-", 
            SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           SuperscriptBox["2", 
            SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
      "\[RightAngleBracket]"}], "|", 
     SubscriptBox["f", "j"]}], "  ", "\[RightAngleBracket]"}], 
   TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["xguess", "j"], 
     RowBox[{"(", 
      RowBox[{"mod", " ", "M"}], ")"}]}]}], TraditionalForm]]]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:16", "b:3.2.4"}],

Cell[TextData[{
 "We take an hypotetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]]],
 "]"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4"],

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.2.4"],

Cell[BoxData[{
 SuperscriptBox["2", 
  SubscriptBox["n", "1"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  SubscriptBox["n", "2"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], ")"}]]}], "Input",
 CellTags->"b:3.2.4"],

Cell[TextData[{
 "The next function gives the number in the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 "register in decimal basis from a given number i in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["n", "1"],
     TraditionalForm], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 " bases."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xv", "[", "i_", "]"}], ":=", 
   RowBox[{"FromDigits", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}]}], "]"}], ",", 
       SubscriptBox["n", "1"]}], "]"}], ",", "2"}], "]"}]}], ";"}]], "Input",
 CellTags->"b:3.2.4"],

Cell[TextData[{
 "We select the positions in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 "register which remain when reg2 has been measured:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dens1", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Mod", "[", 
         RowBox[{
          RowBox[{"xguess", "^", 
           RowBox[{"xv", "[", "i", "]"}]}], ",", "MP"}], "]"}], "\[Equal]", 
        "reg2Measurement"}], ",", " ", 
       RowBox[{"dens1", "=", 
        RowBox[{"Join", "[", " ", 
         RowBox[{"dens1", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"xv", "[", "i", "]"}], "+", "1"}], " ", "}"}]}], "]"}]}], 
       ",", "\"\<\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        SuperscriptBox["2", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "1"], "+", 
           SubscriptBox["n", "2"]}], ")"}]], "-", "1"}], ",", 
       SuperscriptBox["2", 
        SubscriptBox["n", "2"]]}], " ", "}"}]}], "]"}], ";"}], 
  " "}]}], "Input",
 CellTags->"b:3.2.4"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"reg1", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"reg1", ",", "1", ",", 
      RowBox[{"dens1", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "dens1", "]"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"reg1", "/", 
    RowBox[{"Norm", "[", "reg1", "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.2.4"],

Cell["Share some memory", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", " ", "]"}], ";"}]], "Input",
 CellTags->"b:3.2.4"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:17", "b:3.2.5"}],

Cell["\<\
We apply the inverse of the quantum Fourier transform to reg1 and check \
normalization:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.2.5"],

Cell[BoxData[{
 SubscriptBox["n", "1"], "\[IndentingNewLine]", 
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"0", ",", "2", ",", 
   SubscriptBox["n", "1"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"CompIQuantumFourierTransformV", "[", 
  RowBox[{"IntegerDigits", "[", 
   RowBox[{"0", ",", "2", ",", 
    SubscriptBox["n", "1"]}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.625186436403755*^9, 3.625186443029406*^9}, 
   3.6251864875596313`*^9, {3.62518687628403*^9, 3.625186876678273*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"reg1", "[", 
        RowBox[{"[", 
         RowBox[{"i", "+", "1"}], "]"}], "]"}], "*", 
       RowBox[{"CompIQuantumFourierTransformV", "[", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"i", ",", "2", ",", 
          SubscriptBox["n", "1"]}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         RowBox[{"2", "^", 
          SubscriptBox["n", "1"]}], "-", "1"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"QFTVec", "=", "QFTVec"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", " ", "[", 
  RowBox[{
  "\"\<The norm of the inverse fourier transformed vector is: \>\"", ",", 
   RowBox[{"Norm", "[", "QFTVec", "]"}]}], "]"}]}], "Input",
 CellTags->"b:3.2.5"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< seconds\>\""}], 
  "]"}]}], "Input",
 CellTags->"b:3.2.5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:18", "b:3.2.6"}],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.2.6"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"QFTVec", "[", 
          RowBox[{"[", 
           RowBox[{"num", "+", "1"}], "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "^", "2"}], "]"}]}], ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.2.6"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.6"],

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.3`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.2.6"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:19", "b:3.2.7"}],

Cell[TextData[{
 "Finally we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["xguess", 
    RowBox[{"r", "/", "2"}]], TraditionalForm]]],
 "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.2.7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.2.7"],

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<xguess=\>\"", ",", "xguess"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}]}], "Input",
 CellTags->"b:3.2.7"],

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.2.7"],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       SuperscriptBox["xguess", "i"], ",", "M"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "60"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"a", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:3.2.7"],

Cell["Quit", "Input",
 CellTags->"b:3.2.7"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Shor's Factoring algorithm with explict construction of both registers (using \
state vectors)\
\>", "Section",
 FontFamily->"Times",
 CellTags->"c:20"],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 0: Define functions for QFT and Number Theory[check ",
 ButtonBox["QFT2014.nb ",
  BaseStyle->"Hyperlink",
  ButtonData:>{"QFT2014.nb", None}],
 "for details]"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:21", "b:3.3.0"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.3.0"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{3.6253320349636087`*^9, 3.625332120186171*^9},
 CellTags->"b:3.3.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.3.0"],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["QDENSITY`Qdensity`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["QDENSITY`Qdensity`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["about",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"about", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["for",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"for", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Oracle",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Oracle", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res3",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"res3", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["TwoOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"TwoOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Adj",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Adj", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["had",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"had", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTrace",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"PartialTrace", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res3$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"res3$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["U",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"U", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["AF",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"AF", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Had",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Had", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTraceGen",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "PartialTraceGen", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Rotqbit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Rotqbit", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["v",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"v", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["at",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"at", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Hadamard",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3625314120-3485556", {"Hadamard", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PartialTranspose",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "PartialTranspose", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RotX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Vs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"Vs", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Bra1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Bra1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["HALL",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"HALL", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ProjG",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ProjG", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RotY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["v$",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"v$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"BraV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["i8",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"i8", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PTr",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"PTr", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RotZ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RotZ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ws",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"ws", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"BraX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["intro",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"intro", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PTrGen",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"PTrGen", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["s",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"s", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["xb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"xb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["BraY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"BraY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["j8",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"j8", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Purity",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Purity", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Schmidt",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Schmidt", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Xb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"Xb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CNOT",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"CNOT", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Ket1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Ket1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PX",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"PX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma0",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Sigma0", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["xd",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"xd", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Coeff",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Coeff", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"KetV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["PY",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"PY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Sigma1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Xd",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"Xd", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Comm",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Comm", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"KetX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["qubit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"qubit", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Sigma2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ZeroMatrix",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ZeroMatrix", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CompMeasureKet",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"CompMeasureKet", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["KetY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"KetY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBra1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomBra1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Sigma3",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Sigma3", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ZeroT",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ZeroT", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CompPTr",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"CompPTr", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["look",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"look", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBra2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomBra2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Snos",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Snos", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[ScriptCapitalH]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledX",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ControlledX", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["matrices",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"matrices", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomBraN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomBraN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["SP",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"SP", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[ScriptCapitalP]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledY",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ControlledY", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"mc", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKet1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomKet1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Spin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Spin", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[DoubleStruckCapitalI]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["ControlledZ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ControlledZ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mcl",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"mcl", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKet2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomKet2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["SpinObs",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"SpinObs", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Rho]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"\[Rho]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CPHASE",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"CPHASE", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mcl$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"mcl$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomKetN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomKetN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["stA",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"stA", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Sigma]",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"\[Sigma]", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["CPHASEV",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"CPHASEV", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["mc$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"mc$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubit1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomQubit1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["stB",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"stB", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]A1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"\[Phi]A1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["cRQN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"cRQN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MeasureKet",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"MeasureKet", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubit2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomQubit2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Swap",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Swap", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]A2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"\[Phi]A2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["Definef",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Definef", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MultiProjCBasis",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "MultiProjCBasis", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["RandomQubitN",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"RandomQubitN", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["s$",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"s$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]B1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"\[Phi]B1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["density",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"density", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["MultiQProj",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"MultiQProj", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["rcsDate",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"rcsDate", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["TensorProductQ",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"TensorProductQ", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[Phi]B2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"\[Phi]B2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["DForm2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"DForm2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"nb", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["rcsRevision",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"rcsRevision", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ThreeOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ThreeOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["\[CapitalOmega]s",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {
            "\[CapitalOmega]s", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"]},
        {
         ButtonBox["EnTropy",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"EnTropy", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nneg",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"nneg", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"res1", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["to",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"to", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["EnTropy2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"EnTropy2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["nq",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"nq", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res1$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"res1$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Toffoli",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Toffoli", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["Fidelity",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Fidelity", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["Observables",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"Observables", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"res2", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ts",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3625314120-3485556", {"ts", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""},
        {
         ButtonBox["FidelityB",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"FidelityB", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["OneOp",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"OneOp", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["res2$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"res2$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], 
         ButtonBox["ts$",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3625314120-3485556", {"ts$", "QDENSITY`Qdensity`"}},
          ButtonNote->"QDENSITY`Qdensity`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.19]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.625332120372224*^9},
 CellTags->"b:3.3.0"]
}, Open  ]],

Cell["\<\
To improve performance we precompile the quantum Fourier transform:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.0"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Zeroff", "[", 
   RowBox[{"vac_", ",", "R_"}], "]"}], ":=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vac", "[", 
      RowBox[{"[", 
       RowBox[{"R", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "/", 
     RowBox[{"2", "^", "i"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "R", "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CompZeroff", " ", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"vac", ",", "_Integer", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"R", ",", "_Integer", ",", "1"}], "}"}]}], "}"}], ",", 
     RowBox[{"Zeroff", "[", 
      RowBox[{"vac", ",", "R"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Zeroff", "[", 
         RowBox[{"vac", ",", "R"}], "]"}], ",", "_Real"}], "}"}], "}"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellTags->"b:3.3.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"QuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"Zeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[TensorProduct]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  ", "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.625186598713747*^9, 3.625186623970701*^9}, {
  3.6251867728252163`*^9, 3.6251867832093287`*^9}, {3.625332015413991*^9, 
  3.625332016413701*^9}},
 CellTags->"b:3.3.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex", 
        ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.3.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"IQuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"Zeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{
            RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[TensorProduct]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.625332022556472*^9, 3.625332024086441*^9}, {
  3.625332110055339*^9, 3.62533211335984*^9}},
 CellTags->"b:3.3.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompIQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"IQuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"IQuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex",
         ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellChangeTimes->{{3.6253315317114964`*^9, 3.6253315362777853`*^9}},
 CellTags->"b:3.3.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the continued fraction expansion. 
i is the ith cycle of continued fraction and j is the number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.3.0"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.0"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.3.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39,55,77 and \
determine the computer size required \
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:22", "b:3.3.1"}],

Cell["M is the number we are trying to factorize:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"M", "=", "35"}], ";"}]], "Input",
 CellTags->"b:3.3.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.3.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime and quit if it is:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
    "Print", "[", "\"\<Your number is a Even, please revise it\>\"", "]"}]}], 
   "]"}], ";"}]], "Input",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.3.1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 TagBox["2048",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6253320352299232`*^9, 3.6253321204566813`*^9},
 CellTags->"b:3.3.1"]
}, Open  ]],

Cell[TextData[{
 " We need the ",
 StyleBox["minimun number",
  FontWeight->"Bold"],
 " of qubits for both registers. Here we should be careful, \na high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " number will lengthen the computation, while providing high probability of \
success, \nhowever in many cases it could be more illustrative to lower the \
value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " and check the \nresulting probabilities."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["n", "1"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["n", "2"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["n", "1"], "=", "9"}], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 TagBox["9",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035241765*^9, 3.625332120489902*^9},
 CellTags->"b:3.3.1"],

Cell[BoxData[
 TagBox["6",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035241765*^9, 3.6253321204930363`*^9},
 CellTags->"b:3.3.1"]
}, Open  ]],

Cell["\<\
Gerjuoy's test: a failure to pass this test means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1"],

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.3.1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]]],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:23", "b:3.3.2"}],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]]],
 " ; which is a \nstate where all qubits have been Hadamarded"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.2"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "1"], "=", 
   RowBox[{
    RowBox[{"2", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SubscriptBox["n", "1"]}], "/", "2"}], ")"}]}], "*", 
    RowBox[{"Table", "[", 
     RowBox[{"1", ",", 
      RowBox[{"{", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}], "}"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Norm", "[", 
  SubscriptBox["\[Psi]", "1"], "]"}]}], "Input",
 CellTags->"b:3.3.2"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.62533203526842*^9, 3.6253321205224*^9},
 CellTags->"b:3.3.2"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]]]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:24", "b:3.3.3"}],

Cell[" This is the random Coprime to MP used in Shor's Algo ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.3.3"],

Cell[BoxData[
 TagBox["8",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035279619*^9, 3.625332120533016*^9},
 CellTags->"b:3.3.3"]
}, Open  ]],

Cell["\<\
The full state composed of register 1 and register 2 is build in the 
following way:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.3"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "12"], "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}], ")"}]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Psi]", "12"], "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{
      SubscriptBox["\[Psi]", "12"], ",", "1", ",", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{"xguess", "^", "i"}], ",", "MP"}], "]"}], " ", "+", 
       RowBox[{"i", "*", 
        SuperscriptBox["2", 
         SubscriptBox["n", "2"]]}], "+", "1"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", 
     RowBox[{
      SuperscriptBox["2", 
       SubscriptBox["n", "1"]], "-", "1"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "12"], "=", 
   RowBox[{
    SubscriptBox["\[Psi]", "12"], "/", 
    RowBox[{"Norm", "[", 
     SubscriptBox["\[Psi]", "12"], "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.3.3"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"ListPlot", "[", 
  SubscriptBox["\[Psi]", "12"], "]"}], "\n", 
 RowBox[{
  RowBox[{"Norm", "[", 
   SubscriptBox["\[Psi]", "12"], "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.614177547228889*^9, 3.614177582910481*^9}},
 CellTags->"b:3.3.3"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.002777777777777778],
       AbsoluteThickness[1.6], PointBox[CompressedData["
1:eJxc2XF01uMf+P9b3zBJJslkMkkmk8lkMrllMplMJpPJZDKZTCaT5JbJZDKZ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       "]]}, {}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 32768.}, {0, 0.04419417382415922}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035685164*^9, 3.625332120924634*^9},
 CellTags->"b:3.3.3"]
}, Open  ]],

Cell["Therefore the state of the full system is,", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Measure the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:25", "b:3.3.4"}],

Cell["\<\
We perform a measurement on register 2, that can be done in the following \
way. First a random 
possible outcome for register two is generated:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.4"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.3.4"],

Cell[BoxData[
 TagBox["22",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035704886*^9, 3.625332120959897*^9},
 CellTags->"b:3.3.4"]
}, Open  ]],

Cell[TextData[{
 "Then register 1 will be in the following state, making use of the function ",
 ButtonBox["MeasureKet",
  BaseStyle->"Hyperlink",
  ButtonData:>{"Tutorial.nb", "MeasureKet"}],
 ": "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.4"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"reg2list", "=", 
  RowBox[{"Table", "[", 
   RowBox[{"i", ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{
       SubscriptBox["n", "1"], "+", "1"}], ",", 
      RowBox[{
       SubscriptBox["n", "1"], "+", 
       SubscriptBox["n", "2"]}]}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"reg2mes", "=", 
  RowBox[{"IntegerDigits", "[", 
   RowBox[{"reg2Measurement", ",", "2", ",", 
    SubscriptBox["n", "2"]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", 
    RowBox[{"1", "P"}]], "=", 
   RowBox[{"MeasureKet", "[", 
    RowBox[{
     SubscriptBox["\[Psi]", "12"], ",", "reg2list", ",", "reg2mes"}], "]"}]}],
   ";"}]}], "Input",
 CellTags->"b:3.3.4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"10"},
      {"11"},
      {"12"},
      {"13"},
      {"14"},
      {"15"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035729211*^9, 3.625332120970586*^9},
 CellTags->"b:3.3.4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"1"},
      {"0"},
      {"1"},
      {"1"},
      {"0"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035729211*^9, 3.625332120973894*^9},
 CellTags->"b:3.3.4"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Norm", "[", 
  SubscriptBox["\[Psi]", 
   RowBox[{"1", " ", "P"}]], "]"}], "\n", 
 RowBox[{"ListPlot", "[", 
  SubscriptBox["\[Psi]", 
   RowBox[{"1", " ", "P"}]], "]"}]}], "Input",
 CellTags->"b:3.3.4"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035743627*^9, 3.625332121010888*^9},
 CellTags->"b:3.3.4"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011111111111111112`], AbsoluteThickness[1.6], PointBox[CompressedData["
1:eJxdz0mu3UQYhuFSRgwzYMCAgUEIIYSQ6ZsAMSGEEEJw+j6p3PTtdfo+t5bg
FaBagkeMawNIXkItwUsAoQjJj3Umj77z6le9s7F56PK28O+37b/f62/ZGdZf
t+YbeHv3946tK3/mv153b7K/hd/GDf277O/h9/EH9B+yf4Q/xi39J+yf4s/w
5/RfsH+Jv8Jf03/D/i3egb+j/579B7wTd+s+dew/rp1w2EW/i/0nehx20+9m
/5kehz30e9h/ocdhL/1e9l/pcdhHv4/9N3oc9tPvZ/+dHocD9AfY/6DHoV/3
Xc//ccHhIP1BelxwOER/iB4XHA7TH6bHBYcj9EfoccHhKP1RelxwOEZ/jB4X
HI7TH6fHBYcT9CfoccHhJP1JelxwOEV/ih4XHE7Tn6bHBYcz9GfoccHhLP1Z
elxwOEd/jh4XHM7Tn6fHBYe47pu43jsccaLP7AVXHC5w/wL3ccSJPrMXXHHY
4P4G93HEiT6zF1xxuMj9i9zHESf6zF5wxeES9y9xH0ec6DN7wRWHy9y/zH0c
caLP7AVXHK5w/wr3ccSJPrMXXHG4yv2r3McRJ/rMXnDF4Rr3r3EfR5zoM3vB
FYfr3L/OfRxxos/sBVccbnD/BvdxxIk+sxdccbjJ/ZvcxxEn+sxecMXhFvdv
cR9HnOgze8EVh9vcv819HHGiz+wFVxzucP8O93HEiT6zF1xx2OT+JvdxxIk+
sxdccRjW/fZhvTe4xR19zx7xgBP9yJ7xhAv9zF7xgsNd3n93vTe4xR19zx7x
gBP9yJ7xhAv9zF7xgsM93n9vvTe4xR19zx7xgBP9yJ7xhAv9zF7xgsN93n9/
vTe4xR19zx7xgBP9yJ7xhAv9zF7xgsMD3v9gvTe4xR19zx7xgBP9yJ7xhAv9
zF7xgsND3v9wvTe4xR19zx7xgBP9yJ7xhAv9zF7xgsMj3v9ovTe4xR19zx7x
gBP9yJ7xhAv9zF7xgsNj3v94vTe4xR19zx7xgBP9yJ7xhAv9zF7xgsMT3v9k
vTe4xR19zx7xgBP9yJ7xhAv9zF7xgsNT3v90vTe4xR19zx7xgBP9yJ7xhAv9
zF7xgsMz3v9svTe4xR19zx7xgBP9yJ7xhAv9zF7xgsNz3v98vTe4xR19zx7x
gBP9yJ7xhAv9zF7xgsML3v9ivTe4xR19zx7xgBP9yJ7xhAv9zF7xgsNL3v9y
vTe4xR19zx7xgBP9yJ7xhAv9zF7xgsMr3v9qvTe4xR19zx7xgBP9yJ7xhAv9
zF7xgsMW799a7w1ucUffs0c84EQ/smc84UI/s1e84JD+7/8BFHkyew==
       "]]}, {}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{},
   PlotRange->{{0., 512.}, {0, 0.08838834764831845}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332035743627*^9, 3.6253321210424*^9},
 CellTags->"b:3.3.4"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Quantum Fourier Transform register 1", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:26", "b:3.3.5"}],

Cell[TextData[{
 "We apply the inverse of the quantum Fourier transform to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", 
    RowBox[{"1", "P"}]], TraditionalForm]]],
 ":"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.3.5"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["\[Psi]", 
         RowBox[{"1", "P"}]], "[", 
        RowBox[{"[", 
         RowBox[{"i", "+", "1"}], "]"}], "]"}], "*", 
       RowBox[{"CompIQuantumFourierTransformV", "[", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"i", ",", "2", ",", 
          SubscriptBox["n", "1"]}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"QFTVec", "/", 
    RowBox[{"Norm", "[", "QFTVec", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Norm", "[", "QFTVec", "]"}]}], "Input",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 TagBox["1.`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332036496694*^9, 3.625332126153097*^9},
 CellTags->"b:3.3.5"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.3.5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< Seconds\>\""}], "]"}]], "Input",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The QFT took \"\>", "\[InvisibleSpace]", 
   "5.264616`6.871881697159205", "\[InvisibleSpace]", "\<\" Seconds\"\>"}],
  SequenceForm["The QFT took ", 5.264616`6.871881697159205, " Seconds"],
  Editable->False]], "Print",
 CellChangeTimes->{3.6253320366413603`*^9, 3.62533212637894*^9},
 CellTags->"b:3.3.5"]
}, Open  ]],

Cell["\<\
We store the possible measurements and their probabilities in MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"QFTVec", "[", 
          RowBox[{"[", 
           RowBox[{"num", "+", "1"}], "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "^", "2"}], "]"}]}], ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.3.5"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Normali", "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{"Normali", "=", 
     RowBox[{"Normali", "+", 
      RowBox[{"MeasProb", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "MeasProb", "]"}]}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", "Normali"}], "Input",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 TagBox["0.9999999999999998`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.62533203698112*^9, 3.625332126470343*^9},
 CellTags->"b:3.3.5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Meassured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Meassured", " ", "number"}], ",", "Probability"}], "}"}]}], 
   ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.12`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.3.5"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011111111111111112`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd1UcO5UQUBdCnHjH8AwYMGBiEEEIImZzpImcwOUPR0GRok5tcS/ASagle
Qu0AL6GW4BWQJZDP058cXd1/Pasrzl147vyZ+OvO/PP7737/4+/77ey/2s/G
8dKRl+ATvhRfhi/HA74SX4Wvxtfga/F1+Ho84hvwjfgmfDO+Bd+Kb8O34zvw
nfgufDe+B9+Lz+J0dMFxHzmO+8lxPECO40FyHA+R43iYHMcj5DgeJcfxGDmO
x8lxPEGO40lyHE+R43iaHMcz5DimoxMuuOF4lj4uuOF4jj4uuOF4nj4uuOF4
gT4uuOF4kT4uuOF4iT4uuOF4mT4uuOF4hT4uuOF4lT4uuOF4jT4uuOF4nT4u
uOF4gz4uuOF4kz4uuOF4iz4uuOF4mz4uuOHI6fD+D5n/wxkXXHHDHcc7Rw84
4YwLrrjhjuMc+zjhjAuuuOGO4132ccIZF1xxwx3He+zjhDMuuOKGO47z7OOE
My644oY7jvfZxwlnXHDFDXccH7CPE8644Iob7jg+ZB8nnHHBFTfccXzEPk44
44Irbrjj+Jh9nHDGBVfccMfxCfs44YwLrrjhjuNT9nHCGRdcccMdx2fs44Qz
LrjihjuOz9nHCWdccMUNdxwX2McJZ1xwxQ13HPPx/T/NfA8eccITznjGBS+4
4hU3vOGOdxxfHH3CAx5xwhPOeMYFL7jiFTe84Y53HF8efcIDHnHCE854xgUv
uOIVN7zhjnccXx19wgMeccITznjGBS+44hU3vOGOdxxfH33CAx5xwhPOeMYF
L7jiFTe84Y53HN8cfcIDHnHCE854xgUvuOIVN7zhjncc3x59wgMeccITznjG
BS+44hU3vOGOdxzfHX3CAx5xwhPOeMYFL7jiFTe84Y53HBd5/y8e8wGPOOEJ
Zzzjghdc8Yob3nDHO47vjz7hAY844QlnPOOCF1zxihvecMc7jh+OPuEBjzjh
CWc844IXXPGKG95wxzuOH48+4QGPOOEJZzzjghdc8Yob3nDHO46fjj7hAY84
4QlnPOOCF1zxihvecMc7jp+PPuEBjzjhCWc844IXXPGKG95wxzuOX44+4QGP
OOEJZzzjghdc8Yob3nDHO45fjz7hAY844QlnPOOCF1zxihvecMf7//4TvGQ9
uQ==
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Meassured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{},
   PlotRange->{{0, 511.}, {0, 0.24999999999999994`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332042679492*^9, 3.625332126635289*^9},
 CellTags->"b:3.3.5"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011111111111111112`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd1ctK1VEYxuGXRuGogYMGoikhISKlZQfd7X8eSu2kdrTTBqGxXcLXoCvx
JhzJugtn0rxJl1CGFftZsFg8vPzGa3L/YPfzhSRff9+z9/BLb+Tk8Hv/aPzb
j52j437+nJ/n77/TDfMivoRH8WU8hq/gKXwVT+NreAbP4jl8Hd/A83gB38S3
8CK+je/gu/geXsLLuIfv4z7uhl04D9hxVthxVtlx1thx1tlxHrLjPGLH2WDH
2WTH2WLHecyO84Qd5yk7zjN2nOfsONvD7nDhhrNDjws3nF16XLjhvKDHhRvO
S3pcuOG8oseFG85rely44byhx4Ubzlt6XLjh7NHjwg3nHT0u3HDe0+PCDecD
PS7ccD7S48IN5xM9LtxwBvS4cPvvibNv/3TQ/f3/fwEzoXde
       "]], LineBox[CompressedData["
1:eJxd1StOBUAMQNEGiWIFBBybIHQLoBHl8f++sINBoNnCJGwCRSZsAjsegcID
hs9p0jRHXdnNxXLveCUi7r72+95ur74+rlU+rd+/7T4978T3bFTG30lcuOGO
B544Dv57Aycu3HDHA08cC/o4ceGGOx544jikjxMXbrjjgSeOI/o4ceGGOx54
4jimjxMXbrjjgSeOE/o4ceGGOx544jiljxMXbrjjgSeOM/o4ceGGOx544jin
jxMXbrjjgSeOC/o4ceGGOx544rikjxMXbrjjgSeOK/o4ceGGOx544rimjxMX
brjjgSeOG/o4ceGGOx544ljSx4kLN9zxwPPXDy8fW/vvy5///wkmJJ0x
       "]], LineBox[CompressedData["
1:eJxd1StOxkAABOANClmBJuUGvQErkYUTbEjQLO8EtQh8j9ATkMpfkR5hj1A0
pg7LQwD9JplMvhPM0fnl2cVeCOHpq9+7O3x+P93P8Wd3r8fhO02O4X9a3OGI
e5xwxgUPeMQTnnHFC15xuNq6wS3ucMQ9Tjjjggc84gnPuOIFrzhcb93gFnc4
4h4nnHHBAx7xhGdc8YJXHG62bnCLOxxxjxPOuOABj3jCM654wSsOt1s3uMUd
jrjHCWdc8IBHPOEZV7zgFYe7rRvc4g5H3OOEMy54wCOe8IwrXvCKw/3WDW5x
hyPuccIZFzzgEU94xhUveMXhYesGt7jDEfc44YwLHvCIJzzjihe8/vnk7fHg
5ePh9/8/ARjbpzk=
       "]], LineBox[CompressedData["
1:eJxdx7tNwDAARVGLijIToLCBN8AjBCawkKhJyR839BkhI6SkQh7BI7inyQh8
Gsi50tPTOb++vbo5CSGU7/38+9nb5+XpXfr994+L8NNwl8L/RhxxwhPOeMYF
L3jFG6644Y53HO6PHvCII054whnPuOAFr3jDFTfc8Y7Dw9EDHnHECU844xkX
vOAVb7jihjvecXg8esAjjjjhCWc844IXvOINV9xwxzsOT0cPeMQRJzzhjGdc
8IJXvOGKG+54x+H56AGPOOKEJ5zxjAte8Io3XHHDHe84vBw94BFHnPCEM55x
wQte8YYrbrjjHYfXowc84ogTnnDGMy54wSvecMUNd7z/+QvlBqLZ
       "]]}}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Meassured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{},
   PlotRange->{{0, 511.}, {0, 0.12}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.625332042679492*^9, 3.625332127583067*^9},
 CellTags->"b:3.3.5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead, we study all posible measurements and \
their probabilities\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:27", "b:3.3.6"}],

Cell["\<\
Finally,  we go through all the measurements, compute the periods and check \
the 
factors we would find and their probabilities:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.6"],

Cell[CellGroupData[{

Cell[BoxData["xguess"], "Input",
 CellTags->"b:3.3.6"],

Cell[BoxData[
 TagBox["8",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.62533204681219*^9, 3.625332127796438*^9},
 CellTags->"b:3.3.6"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.3.6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.3.6"],

Cell[CellGroupData[{

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}]}], "Input",
 CellTags->"b:3.3.6"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"Period", "Number", "\<\"f1\"\>", "\<\"f2\"\>", "Prob"},
     {"\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>"},
     {"4", "128", "5", "7", "0.24999999999999994`"},
     {"4", "384", "5", "7", "0.24999999999999994`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6253320469062243`*^9, 3.6253321279638643`*^9},
 CellTags->"b:3.3.6"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The TOTAL PROBABILITY OF SUCCESS is thus:\"\>", 
   "\[InvisibleSpace]", "0.4999999999999999`", 
   "\[InvisibleSpace]", "\<\"  with xguess=\"\>", "\[InvisibleSpace]", "8", 
   "\[InvisibleSpace]", "\<\" and measured register=\"\>", 
   "\[InvisibleSpace]", "22"}],
  SequenceForm[
  "The TOTAL PROBABILITY OF SUCCESS is thus:", 0.4999999999999999, 
   "  with xguess=", 8, " and measured register=", 22],
  Editable->False]], "Print",
 CellChangeTimes->{3.625332046909507*^9, 3.625332127967189*^9},
 CellTags->"b:3.3.6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.3.6"],

Cell[BoxData[
 TagBox["13.377753`7.577928166595074",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6253320469191523`*^9, 3.625332128015974*^9},
 CellTags->"b:3.3.6"]
}, Open  ]],

Cell[BoxData["Quit"], "Input",
 CellTags->"b:3.3.6"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Shor's Factoring algorithm using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " DFT formula"
}], "Section",
 FontFamily->"Times",
 CellTags->"c:28"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and NumberTheory\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:29", "b:3.4.0"}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.4.0"],

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.4.0"],

Cell["\<\
In this notebook we make use of the Discrete Fourier Transform already \
present in Mathematica.\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.4.0"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.0"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.4.0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:30", "b:3.4.1"}],

Cell["M is the number we are trying to factor:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{"11", "*", "7"}]}]], "Input",
 CellTags->"b:3.4.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.4.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
     RowBox[{"Even", "=", "True"}], ";", 
     RowBox[{"MP", "=", 
      RowBox[{"MP", "/", "2"}]}]}]}], "]"}], ";"}]], "Input",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"MP", "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"M", ",", " ", "\"\< is a power of 2\>\""}], "]"}], ";", 
    RowBox[{"Quit", "[", "]"}]}], ","}], "]"}]], "Input",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.4.1"],

Cell[TextData[{
 " We need the ",
 StyleBox["minimun number",
  FontWeight->"Bold"],
 " of qubits for both registers. \nHere we should be careful, a high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " number will lengthen the computation.  \nBy default we give estimates for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 ",based on the fact that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " should be able to \nrepresent numbers up to M^2, while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 "should allow to write up to M/2. However in \nmany cases it could be more \
illustrative to lower the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " \n[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.4.1"],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["n", "1"], "=", 
  RowBox[{"Ceiling", "[", 
   RowBox[{"Log", "[", 
    RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["n", "2"], "=", 
  RowBox[{"Ceiling", "[", 
   RowBox[{"Log", "[", 
    RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 SubscriptBox["n", 
  RowBox[{"1", " "}]], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellTags->"b:3.4.1"],

Cell["\<\
Gerjuoy's test: a failure to pass this test means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1"],

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.4.1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]]],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:31", "b:3.4.2"}],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]]],
 " ; which is a \nstate where all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]]]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:32", "b:3.4.3"}],

Cell[" This is the random Coprime to MP: ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.3"],

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.4.3"],

Cell[TextData[{
 " The full wave function after we build the second register with  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]]],
 "reads:\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"|", 
         SubscriptBox["\[Psi]", "12"]}], "\[RightAngleBracket]"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{
           RowBox[{"-", 
            SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           SuperscriptBox["2", 
            SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
      "\[RightAngleBracket]"}], "|", 
     SubscriptBox["f", "j"]}], "  ", "\[RightAngleBracket]"}], 
   TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["xguess", "j"], 
     RowBox[{"(", 
      RowBox[{"mod", " ", "M"}], ")"}]}]}], TraditionalForm]]]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:33", "b:3.4.4"}],

Cell[TextData[{
 "We take an hypotetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]]],
 "]"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4"],

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.4.4"],

Cell[BoxData[{
 SuperscriptBox["2", 
  SubscriptBox["n", "1"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  SubscriptBox["n", "2"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], ")"}]]}], "Input",
 CellTags->"b:3.4.4"],

Cell[TextData[{
 "The next function gives the number in the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 "register in decimal basis from a given number i in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["n", "1"],
     TraditionalForm], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 " bases."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"xv", "[", "i_", "]"}], ":=", 
   RowBox[{"FromDigits", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}]}], "]"}], ",", 
       SubscriptBox["n", "1"]}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compxv", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"i", ",", "_Integer", ",", "0"}], "}"}], "}"}], ",", 
     RowBox[{"xv", "[", "i", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"xv", "[", "i", "]"}], ",", "_Integer", ",", "0"}], "}"}], 
      "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{3.395094492673421*^9},
 CellTags->"b:3.4.4"],

Cell[TextData[{
 "We select the positions in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 "register which remain when reg2 has been measured:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dens1", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Do", "[", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"xguess", "^", 
            RowBox[{"xv", "[", "i", "]"}]}], ",", "MP"}], "]"}], "\[Equal]", 
         "reg2Measurement"}], ",", " ", 
        RowBox[{"dens1", "=", 
         RowBox[{"Join", "[", " ", 
          RowBox[{"dens1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"xv", "[", "i", "]"}], "+", "1"}], " ", "}"}]}], "]"}]}],
         ",", "\"\<\>\""}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "1"], "+", 
            SubscriptBox["n", "2"]}], ")"}]], "-", "1"}], ",", 
        SuperscriptBox["2", 
         SubscriptBox["n", "2"]]}], " ", "}"}]}], "]"}], ";"}], " ", "*)"}], 
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"xx", "=", 
       RowBox[{"Compxv", "[", 
        RowBox[{"i", "*", 
         SuperscriptBox["2", 
          SubscriptBox["n", "2"]]}], "]"}]}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"xguess", "^", "xx"}], ",", "MP"}], "]"}], "\[Equal]", 
         "reg2Measurement"}], ",", " ", 
        RowBox[{"dens1", "=", 
         RowBox[{"Join", "[", " ", 
          RowBox[{"dens1", ",", 
           RowBox[{"{", 
            RowBox[{"xx", "+", "1"}], " ", "}"}]}], "]"}]}], ",", 
        "\"\<\>\""}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]], "-", "1"}]}], " ", "}"}]}], "]"}], 
   ";"}]}]}], "Input",
 CellTags->"b:3.4.4"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"reg1", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"reg1", ",", "1", ",", 
      RowBox[{"dens1", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "dens1", "]"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"reg1", "/", 
    RowBox[{"Norm", "[", "reg1", "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.4.4"],

Cell[BoxData[
 RowBox[{"Share", "[", " ", "]"}]], "Input",
 CellTags->"b:3.4.4"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:34", "b:3.4.5"}],

Cell["\<\
We apply the inverse of the quantum Fourier transform to reg1 and check \
normalization:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.4.5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"Fourier", "[", "reg1", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.4.5"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< seconds\>\""}], 
  "]"}]}], "Input",
 CellTags->"b:3.4.5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:35", "b:3.4.6"}],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.4.6"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{"QFTVec", "[", 
         RowBox[{"[", 
          RowBox[{"num", "+", "1"}], "]"}], "]"}], "]"}], "^", "2"}], "]"}]}],
     ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.4.6"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.6"],

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.3`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.4.6"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{"c:36", "b:3.4.7"}],

Cell[TextData[{
 "Finally,  we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["xguess", 
    RowBox[{"r", "/", "2"}]], TraditionalForm]]],
 "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.4.7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.4.7"],

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<xguess=\>\"", ",", "xguess"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}]}], "Input",
 CellTags->"b:3.4.7"],

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.4.7"],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       RowBox[{"xguess", "^", "i"}], ",", "M"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "60"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"a", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.625323644663308*^9, 3.625323645505684*^9}},
 CellTags->"b:3.4.7"],

Cell["Quit", "Input",
 CellTags->"b:3.4.7"]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{890, 606},
WindowMargins->{{231, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingStartingPageNumber->53,
PrintingPageRange->{1, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{612, 792},
"PostScriptOutputFile":>FrontEnd`FileName[{
  "export", "home", "brunojd", "PITT", "QC", "QdensityPK"}, "35_22.ps", 
  CharacterEncoding -> "WindowsANSI"],
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintingMargins"->{{54, 54}, {72, 72}}},
ShowSelection->True,
TaggingRules:>{
 "PaginationCache" -> {{2005, 8, 9, 3, 36, 22.593709`8.106562514035977}, {53, 
    53, 53, 53, 53, 53, 53, 53, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 
    55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 56, 56, 56, 56, 56, 
    56, 56, 56, 56, 56, 56, 56, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 
    57, 57, 57, 57, 57, 57, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 
    59, 59, 59, 59, 59, 59, 59, 59, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 
    60, 60, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 62, 62, 
    62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 63, 63, 63, 63, 63, 63, 63, 63, 
    63, 63, 63, 63, 63, 63, 64, 64, 64, 64, 64, 64, 64, 64, 64, 65, 65, 65, 
    65, 65, 65, 65, 65, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 67, 
    67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 68, 68, 68, 68, 68, 68, 
    68, 68, 68, 68, 68, 68, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 
    69, 69, 70, 70, 70, 70, 70, 70, 70, 70, 71, 71, 71, 71, 71, 71, 71, 71, 
    71, 71, 71, 71, 71, 71, 71, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 
    72, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 74, 74, 74, 74, 
    74, 74, 74, 74, 74, 74, 74, 74, 74, 75, 75, 75, 75, 75, 75, 75, 76, 76}, 
    Automatic}},
CellLabelAutoDelete->True,
Magnification->1,
FrontEndVersion->"10.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (October 9, \
2015)",
StyleDefinitions->"Classroom.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "c:1"->{
  Cell[545, 20, 62, 1, 51, "Title",
   CellTags->"c:1"]},
 "c:2"->{
  Cell[1018, 37, 276, 6, 51, "Section",
   CellTags->{"c:2", "b:3.0"}],
  Cell[4406, 122, 106, 2, 51, "Section",
   CellTags->"c:2"]},
 "b:3.0"->{
  Cell[1018, 37, 276, 6, 51, "Section",
   CellTags->{"c:2", "b:3.0"}],
  Cell[1297, 45, 2894, 67, 234, "Abstract",
   CellTags->"b:3.0"],
  Cell[4194, 114, 175, 3, 22, "Text",
   CellTags->"b:3.0"]},
 "c:3"->{
  Cell[4537, 128, 179, 5, 58, "Subsection",
   CellTags->{"c:3", "b:3.1.0"}]},
 "b:3.1.0"->{
  Cell[4537, 128, 179, 5, 58, "Subsection",
   CellTags->{"c:3", "b:3.1.0"}],
  Cell[4741, 137, 133, 3, 46, "Input",
   CellTags->"b:3.1.0"],
  Cell[4877, 142, 265, 4, 22, "Print",
   CellTags->"b:3.1.0"],
  Cell[5179, 151, 112, 3, 46, "Input",
   CellTags->"b:3.1.0"],
  Cell[5294, 156, 33535, 941, 579, "Print",
   CellTags->"b:3.1.0"],
  Cell[38844, 1100, 198, 5, 23, "Text",
   CellTags->"b:3.1.0"],
  Cell[39045, 1107, 1476, 42, 85, "Input",
   CellTags->"b:3.1.0"],
  Cell[40524, 1151, 813, 24, 78, "Input",
   CellTags->"b:3.1.0"],
  Cell[41340, 1177, 1101, 32, 78, "Input",
   CellTags->"b:3.1.0"],
  Cell[42444, 1211, 786, 25, 62, "Input",
   CellTags->"b:3.1.0"],
  Cell[43233, 1238, 598, 18, 62, "Input",
   CellTags->"b:3.1.0"],
  Cell[43834, 1258, 2561, 67, 142, "Input",
   CellTags->"b:3.1.0"],
  Cell[46398, 1327, 252, 7, 51, "Text",
   CellTags->"b:3.1.0"],
  Cell[46653, 1336, 407, 13, 63, "Input",
   CellTags->"b:3.1.0"],
  Cell[47063, 1351, 112, 2, 23, "Text",
   CellTags->"b:3.1.0"],
  Cell[47178, 1355, 115, 4, 45, "Input",
   CellTags->"b:3.1.0"]},
 "c:4"->{
  Cell[47330, 1364, 212, 6, 58, "Subsection",
   CellTags->{"c:4", "b:3.1.1"}]},
 "b:3.1.1"->{
  Cell[47330, 1364, 212, 6, 58, "Subsection",
   CellTags->{"c:4", "b:3.1.1"}],
  Cell[47545, 1372, 104, 2, 23, "Text",
   CellTags->"b:3.1.1"],
  Cell[47652, 1376, 90, 3, 45, "Input",
   CellTags->"b:3.1.1"],
  Cell[47745, 1381, 119, 2, 23, "Text",
   CellTags->"b:3.1.1"],
  Cell[47867, 1385, 90, 3, 45, "Input",
   CellTags->"b:3.1.1"],
  Cell[47960, 1390, 192, 5, 23, "Text",
   CellTags->"b:3.1.1"],
  Cell[48155, 1397, 327, 11, 45, "Input",
   CellTags->"b:3.1.1"],
  Cell[48485, 1410, 277, 8, 46, "Input",
   CellTags->"b:3.1.1"],
  Cell[48765, 1420, 680, 19, 78, "Input",
   CellTags->"b:3.1.1"],
  Cell[49448, 1441, 1122, 37, 173, "Text",
   CellTags->"b:3.1.1"],
  Cell[50595, 1482, 215, 7, 46, "Input",
   CellTags->"b:3.1.1"],
  Cell[50813, 1491, 252, 6, 45, "Output",
   CellTags->"b:3.1.1"],
  Cell[51102, 1502, 513, 16, 94, "Input",
   CellTags->"b:3.1.1"],
  Cell[51618, 1520, 250, 6, 45, "Output",
   CellTags->"b:3.1.1"],
  Cell[51871, 1528, 248, 6, 45, "Output",
   CellTags->"b:3.1.1"],
  Cell[52134, 1537, 249, 7, 51, "Text",
   CellTags->"b:3.1.1"],
  Cell[52408, 1548, 379, 12, 46, "Input",
   CellTags->"b:3.1.1"],
  Cell[52790, 1562, 288, 4, 22, "Print",
   CellTags->"b:3.1.1"]},
 "c:5"->{
  Cell[53127, 1572, 352, 11, 60, "Subsection",
   CellTags->{"c:5", "b:3.1.2"}]},
 "b:3.1.2"->{
  Cell[53127, 1572, 352, 11, 60, "Subsection",
   CellTags->{"c:5", "b:3.1.2"}],
  Cell[53482, 1585, 896, 29, 97, "Text",
   CellTags->"b:3.1.2"]},
 "c:6"->{
  Cell[54415, 1619, 468, 15, 63, "Subsection",
   CellTags->{"c:6", "b:3.1.3"}]},
 "b:3.1.3"->{
  Cell[54415, 1619, 468, 15, 63, "Subsection",
   CellTags->{"c:6", "b:3.1.3"}],
  Cell[54886, 1636, 99, 2, 23, "Text",
   CellTags->"b:3.1.3"],
  Cell[55010, 1642, 499, 16, 46, "Input",
   CellTags->"b:3.1.3"],
  Cell[55512, 1660, 252, 6, 45, "Output",
   CellTags->"b:3.1.3"],
  Cell[55779, 1669, 1154, 40, 63, "Text",
   CellTags->"b:3.1.3"]},
 "c:7"->{
  Cell[56970, 1714, 148, 3, 58, "Subsection",
   CellTags->{"c:7", "b:3.1.4"}]},
 "b:3.1.4"->{
  Cell[56970, 1714, 148, 3, 58, "Subsection",
   CellTags->{"c:7", "b:3.1.4"}],
  Cell[57121, 1719, 287, 9, 55, "Text",
   CellTags->"b:3.1.4"],
  Cell[57433, 1732, 363, 12, 59, "Input",
   CellTags->"b:3.1.4"],
  Cell[57799, 1746, 250, 6, 45, "Output",
   CellTags->"b:3.1.4"],
  Cell[58086, 1757, 315, 10, 82, "Input",
   CellTags->"b:3.1.4"],
  Cell[58404, 1769, 250, 6, 45, "Output",
   CellTags->"b:3.1.4"],
  Cell[58657, 1777, 250, 6, 45, "Output",
   CellTags->"b:3.1.4"],
  Cell[58910, 1785, 250, 6, 45, "Output",
   CellTags->"b:3.1.4"],
  Cell[59175, 1794, 425, 16, 25, "Text",
   CellTags->"b:3.1.4"],
  Cell[59603, 1812, 446, 14, 46, "Input",
   CellTags->"b:3.1.4"],
  Cell[60052, 1828, 287, 10, 25, "Text",
   CellTags->"b:3.1.4"],
  Cell[60342, 1840, 1054, 34, 85, "Input",
   CellTags->"b:3.1.4"],
  Cell[61399, 1876, 134, 4, 23, "Text",
   CellTags->"b:3.1.4"],
  Cell[61536, 1882, 764, 25, 78, "Input",
   CellTags->"b:3.1.4"],
  Cell[62303, 1909, 223, 8, 45, "Input",
   CellTags->"b:3.1.4"],
  Cell[62529, 1919, 83, 2, 23, "Text",
   CellTags->"b:3.1.4"],
  Cell[62615, 1923, 98, 3, 45, "Input",
   CellTags->"b:3.1.4"],
  Cell[62716, 1928, 95, 2, 23, "Text",
   CellTags->"b:3.1.4"],
  Cell[62814, 1932, 653, 20, 46, "Input",
   CellTags->"b:3.1.4"],
  Cell[63470, 1954, 597, 19, 69, "Input",
   CellTags->"b:3.1.4"],
  Cell[64070, 1975, 98, 3, 45, "Input",
   CellTags->"b:3.1.4"]},
 "c:8"->{
  Cell[64205, 1983, 142, 3, 58, "Subsection",
   CellTags->{"c:8", "b:3.1.5"}]},
 "b:3.1.5"->{
  Cell[64205, 1983, 142, 3, 58, "Subsection",
   CellTags->{"c:8", "b:3.1.5"}],
  Cell[64350, 1988, 131, 3, 23, "Text",
   CellTags->"b:3.1.5"],
  Cell[64484, 1993, 477, 15, 46, "Input",
   CellTags->"b:3.1.5"],
  Cell[64964, 2010, 127, 4, 45, "Input",
   CellTags->"b:3.1.5"],
  Cell[65094, 2016, 438, 13, 50, "Input",
   CellTags->"b:3.1.5"],
  Cell[65557, 2033, 291, 8, 61, "Input",
   CellTags->"b:3.1.5"],
  Cell[65851, 2043, 427, 8, 21, "Print",
   CellTags->"b:3.1.5"],
  Cell[66315, 2056, 114, 3, 46, "Input",
   CellTags->"b:3.1.5"],
  Cell[66432, 2061, 265, 6, 45, "Output",
   CellTags->"b:3.1.5"]},
 "c:9"->{
  Cell[66746, 2073, 189, 6, 58, "Subsection",
   CellTags->{"c:9", "b:3.1.6"}]},
 "b:3.1.6"->{
  Cell[66746, 2073, 189, 6, 58, "Subsection",
   CellTags->{"c:9", "b:3.1.6"}],
  Cell[66938, 2081, 154, 5, 23, "Text",
   CellTags->"b:3.1.6"],
  Cell[67095, 2088, 120, 4, 45, "Input",
   CellTags->"b:3.1.6"],
  Cell[67218, 2094, 678, 21, 85, "Input",
   CellTags->"b:3.1.6"],
  Cell[67899, 2117, 156, 5, 23, "Text",
   CellTags->"b:3.1.6"],
  Cell[68080, 2126, 803, 22, 78, "Input",
   CellTags->"b:3.1.6"],
  Cell[68886, 2150, 3593, 69, 222, "Output",
   CellTags->"b:3.1.6"],
  Cell[72482, 2221, 1697, 41, 225, "Output",
   CellTags->"b:3.1.6"]},
 "c:10"->{
  Cell[74228, 2268, 169, 5, 58, "Subsection",
   CellTags->{"c:10", "b:3.1.7"}]},
 "b:3.1.7"->{
  Cell[74228, 2268, 169, 5, 58, "Subsection",
   CellTags->{"c:10", "b:3.1.7"}],
  Cell[74400, 2275, 512, 12, 136, "Text",
   CellTags->"b:3.1.7"],
  Cell[74915, 2289, 89, 3, 45, "Input",
   CellTags->"b:3.1.7"],
  Cell[75007, 2294, 4219, 115, 267, "Input",
   CellTags->"b:3.1.7"],
  Cell[79251, 2413, 1016, 27, 126, "Input",
   CellTags->"b:3.1.7"],
  Cell[80270, 2442, 845, 20, 62, "Output",
   CellTags->"b:3.1.7"],
  Cell[81118, 2464, 308, 7, 22, "Print",
   CellTags->"b:3.1.7"],
  Cell[81429, 2473, 607, 13, 22, "Print",
   CellTags->"b:3.1.7"],
  Cell[82039, 2488, 256, 4, 22, "Print",
   CellTags->"b:3.1.7"],
  Cell[82332, 2497, 81, 2, 45, "Input",
   CellTags->"b:3.1.7"],
  Cell[82416, 2501, 275, 6, 45, "Output",
   CellTags->"b:3.1.7"],
  Cell[82706, 2510, 264, 7, 79, "Text",
   CellTags->"b:3.1.7"],
  Cell[82995, 2521, 413, 13, 66, "Input",
   CellTags->"b:3.1.7"],
  Cell[83411, 2536, 1330, 35, 261, "Output",
   CellTags->"b:3.1.7"],
  Cell[84756, 2574, 43, 1, 45, "Input",
   CellTags->"b:3.1.7"]},
 "c:11"->{
  Cell[84848, 2581, 129, 4, 51, "Section",
   CellTags->"c:11"]},
 "c:12"->{
  Cell[85002, 2589, 180, 5, 58, "Subsection",
   CellTags->{"c:12", "b:3.2.0"}]},
 "b:3.2.0"->{
  Cell[85002, 2589, 180, 5, 58, "Subsection",
   CellTags->{"c:12", "b:3.2.0"}],
  Cell[85185, 2596, 133, 3, 46, "Input",
   CellTags->"b:3.2.0"],
  Cell[85321, 2601, 112, 3, 46, "Input",
   CellTags->"b:3.2.0"],
  Cell[85436, 2606, 166, 5, 23, "Text",
   CellTags->"b:3.2.0"],
  Cell[85605, 2613, 1050, 34, 62, "Input",
   CellTags->"b:3.2.0"],
  Cell[86658, 2649, 2290, 59, 141, "Input",
   CellTags->"b:3.2.0"],
  Cell[88951, 2710, 520, 14, 62, "Input",
   CellTags->"b:3.2.0"],
  Cell[89474, 2726, 2248, 59, 141, "Input",
   CellTags->"b:3.2.0"],
  Cell[91725, 2787, 521, 14, 62, "Input",
   CellTags->"b:3.2.0"],
  Cell[92249, 2803, 252, 7, 51, "Text",
   CellTags->"b:3.2.0"],
  Cell[92504, 2812, 407, 13, 63, "Input",
   CellTags->"b:3.2.0"],
  Cell[92914, 2827, 112, 2, 23, "Text",
   CellTags->"b:3.2.0"],
  Cell[93029, 2831, 115, 4, 45, "Input",
   CellTags->"b:3.2.0"]},
 "c:13"->{
  Cell[93181, 2840, 206, 6, 58, "Subsection",
   CellTags->{"c:13", "b:3.2.1"}]},
 "b:3.2.1"->{
  Cell[93181, 2840, 206, 6, 58, "Subsection",
   CellTags->{"c:13", "b:3.2.1"}],
  Cell[93390, 2848, 104, 2, 23, "Text",
   CellTags->"b:3.2.1"],
  Cell[93497, 2852, 96, 3, 45, "Input",
   CellTags->"b:3.2.1"],
  Cell[93596, 2857, 119, 2, 23, "Text",
   CellTags->"b:3.2.1"],
  Cell[93718, 2861, 90, 3, 45, "Input",
   CellTags->"b:3.2.1"],
  Cell[93811, 2866, 175, 5, 23, "Text",
   CellTags->"b:3.2.1"],
  Cell[93989, 2873, 327, 11, 45, "Input",
   CellTags->"b:3.2.1"],
  Cell[94319, 2886, 277, 8, 46, "Input",
   CellTags->"b:3.2.1"],
  Cell[94599, 2896, 680, 19, 78, "Input",
   CellTags->"b:3.2.1"],
  Cell[95282, 2917, 1123, 37, 173, "Text",
   CellTags->"b:3.2.1"],
  Cell[96408, 2956, 215, 7, 46, "Input",
   CellTags->"b:3.2.1"],
  Cell[96626, 2965, 284, 9, 25, "Text",
   CellTags->"b:3.2.1"],
  Cell[96913, 2976, 701, 20, 110, "Input",
   CellTags->"b:3.2.1"],
  Cell[97617, 2998, 249, 7, 51, "Text",
   CellTags->"b:3.2.1"],
  Cell[97869, 3007, 379, 12, 46, "Input",
   CellTags->"b:3.2.1"]},
 "c:14"->{
  Cell[98285, 3024, 353, 11, 60, "Subsection",
   CellTags->{"c:14", "b:3.2.2"}]},
 "b:3.2.2"->{
  Cell[98285, 3024, 353, 11, 60, "Subsection",
   CellTags->{"c:14", "b:3.2.2"}],
  Cell[98641, 3037, 896, 29, 97, "Text",
   CellTags->"b:3.2.2"]},
 "c:15"->{
  Cell[99574, 3071, 469, 15, 63, "Subsection",
   CellTags->{"c:15", "b:3.2.3"}]},
 "b:3.2.3"->{
  Cell[99574, 3071, 469, 15, 63, "Subsection",
   CellTags->{"c:15", "b:3.2.3"}],
  Cell[100046, 3088, 99, 2, 23, "Text",
   CellTags->"b:3.2.3"],
  Cell[100148, 3092, 499, 16, 46, "Input",
   CellTags->"b:3.2.3"],
  Cell[100650, 3110, 77, 2, 46, "Input",
   CellTags->"b:3.2.3"],
  Cell[100730, 3114, 1154, 40, 63, "Text",
   CellTags->"b:3.2.3"]},
 "c:16"->{
  Cell[101921, 3159, 149, 3, 58, "Subsection",
   CellTags->{"c:16", "b:3.2.4"}]},
 "b:3.2.4"->{
  Cell[101921, 3159, 149, 3, 58, "Subsection",
   CellTags->{"c:16", "b:3.2.4"}],
  Cell[102073, 3164, 286, 9, 55, "Text",
   CellTags->"b:3.2.4"],
  Cell[102362, 3175, 363, 12, 59, "Input",
   CellTags->"b:3.2.4"],
  Cell[102728, 3189, 315, 10, 82, "Input",
   CellTags->"b:3.2.4"],
  Cell[103046, 3201, 425, 16, 25, "Text",
   CellTags->"b:3.2.4"],
  Cell[103474, 3219, 446, 14, 46, "Input",
   CellTags->"b:3.2.4"],
  Cell[103923, 3235, 287, 10, 25, "Text",
   CellTags->"b:3.2.4"],
  Cell[104213, 3247, 1054, 34, 85, "Input",
   CellTags->"b:3.2.4"],
  Cell[105270, 3283, 134, 4, 23, "Text",
   CellTags->"b:3.2.4"],
  Cell[105407, 3289, 764, 25, 78, "Input",
   CellTags->"b:3.2.4"],
  Cell[106174, 3316, 81, 2, 23, "Text",
   CellTags->"b:3.2.4"],
  Cell[106258, 3320, 98, 3, 45, "Input",
   CellTags->"b:3.2.4"]},
 "c:17"->{
  Cell[106393, 3328, 143, 3, 58, "Subsection",
   CellTags->{"c:17", "b:3.2.5"}]},
 "b:3.2.5"->{
  Cell[106393, 3328, 143, 3, 58, "Subsection",
   CellTags->{"c:17", "b:3.2.5"}],
  Cell[106539, 3333, 160, 5, 23, "Text",
   CellTags->"b:3.2.5"],
  Cell[106702, 3340, 127, 4, 45, "Input",
   CellTags->"b:3.2.5"],
  Cell[107328, 3358, 913, 26, 77, "Input",
   CellTags->"b:3.2.5"],
  Cell[108244, 3386, 291, 8, 61, "Input",
   CellTags->"b:3.2.5"]},
 "c:18"->{
  Cell[108572, 3399, 190, 6, 58, "Subsection",
   CellTags->{"c:18", "b:3.2.6"}]},
 "b:3.2.6"->{
  Cell[108572, 3399, 190, 6, 58, "Subsection",
   CellTags->{"c:18", "b:3.2.6"}],
  Cell[108765, 3407, 154, 5, 23, "Text",
   CellTags->"b:3.2.6"],
  Cell[108922, 3414, 120, 4, 45, "Input",
   CellTags->"b:3.2.6"],
  Cell[109045, 3420, 750, 23, 85, "Input",
   CellTags->"b:3.2.6"],
  Cell[109798, 3445, 156, 5, 23, "Text",
   CellTags->"b:3.2.6"],
  Cell[109957, 3452, 803, 22, 78, "Input",
   CellTags->"b:3.2.6"]},
 "c:19"->{
  Cell[110797, 3479, 169, 5, 58, "Subsection",
   CellTags->{"c:19", "b:3.2.7"}]},
 "b:3.2.7"->{
  Cell[110797, 3479, 169, 5, 58, "Subsection",
   CellTags->{"c:19", "b:3.2.7"}],
  Cell[110969, 3486, 512, 12, 136, "Text",
   CellTags->"b:3.2.7"],
  Cell[111484, 3500, 89, 3, 45, "Input",
   CellTags->"b:3.2.7"],
  Cell[111576, 3505, 4219, 115, 267, "Input",
   CellTags->"b:3.2.7"],
  Cell[115798, 3622, 767, 20, 110, "Input",
   CellTags->"b:3.2.7"],
  Cell[116568, 3644, 81, 2, 45, "Input",
   CellTags->"b:3.2.7"],
  Cell[116652, 3648, 264, 7, 79, "Text",
   CellTags->"b:3.2.7"],
  Cell[116919, 3657, 413, 13, 66, "Input",
   CellTags->"b:3.2.7"],
  Cell[117335, 3672, 43, 1, 45, "Input",
   CellTags->"b:3.2.7"]},
 "c:20"->{
  Cell[117427, 3679, 162, 5, 51, "Section",
   CellTags->"c:20"]},
 "c:21"->{
  Cell[117614, 3688, 272, 9, 58, "Subsection",
   CellTags->{"c:21", "b:3.3.0"}]},
 "b:3.3.0"->{
  Cell[117614, 3688, 272, 9, 58, "Subsection",
   CellTags->{"c:21", "b:3.3.0"}],
  Cell[117911, 3701, 133, 3, 46, "Input",
   CellTags->"b:3.3.0"],
  Cell[118047, 3706, 197, 3, 22, "Print",
   CellTags->"b:3.3.0"],
  Cell[118281, 3714, 112, 3, 46, "Input",
   CellTags->"b:3.3.0"],
  Cell[118396, 3719, 28584, 764, 579, "Print",
   CellTags->"b:3.3.0"],
  Cell[146995, 4486, 139, 4, 23, "Text",
   CellTags->"b:3.3.0"],
  Cell[147137, 4492, 1050, 34, 62, "Input",
   CellTags->"b:3.3.0"],
  Cell[148190, 4528, 2311, 59, 141, "Input",
   CellTags->"b:3.3.0"],
  Cell[150504, 4589, 520, 14, 62, "Input",
   CellTags->"b:3.3.0"],
  Cell[151027, 4605, 2292, 60, 141, "Input",
   CellTags->"b:3.3.0"],
  Cell[153322, 4667, 593, 15, 62, "Input",
   CellTags->"b:3.3.0"],
  Cell[153918, 4684, 260, 6, 51, "Text",
   CellTags->"b:3.3.0"],
  Cell[154181, 4692, 407, 13, 63, "Input",
   CellTags->"b:3.3.0"],
  Cell[154591, 4707, 112, 2, 23, "Text",
   CellTags->"b:3.3.0"],
  Cell[154706, 4711, 115, 4, 45, "Input",
   CellTags->"b:3.3.0"]},
 "c:22"->{
  Cell[154858, 4720, 214, 6, 58, "Subsection",
   CellTags->{"c:22", "b:3.3.1"}]},
 "b:3.3.1"->{
  Cell[154858, 4720, 214, 6, 58, "Subsection",
   CellTags->{"c:22", "b:3.3.1"}],
  Cell[155075, 4728, 107, 2, 23, "Text",
   CellTags->"b:3.3.1"],
  Cell[155185, 4732, 90, 3, 45, "Input",
   CellTags->"b:3.3.1"],
  Cell[155278, 4737, 119, 2, 23, "Text",
   CellTags->"b:3.3.1"],
  Cell[155400, 4741, 90, 3, 45, "Input",
   CellTags->"b:3.3.1"],
  Cell[155493, 4746, 175, 5, 23, "Text",
   CellTags->"b:3.3.1"],
  Cell[155671, 4753, 301, 10, 46, "Input",
   CellTags->"b:3.3.1"],
  Cell[155975, 4765, 680, 19, 78, "Input",
   CellTags->"b:3.3.1"],
  Cell[156680, 4788, 215, 7, 46, "Input",
   CellTags->"b:3.3.1"],
  Cell[156898, 4797, 183, 5, 45, "Output",
   CellTags->"b:3.3.1"],
  Cell[157096, 4805, 576, 17, 111, "Text",
   CellTags->"b:3.3.1"],
  Cell[157697, 4826, 513, 16, 94, "Input",
   CellTags->"b:3.3.1"],
  Cell[158213, 4844, 176, 5, 45, "Output",
   CellTags->"b:3.3.1"],
  Cell[158392, 4851, 178, 5, 45, "Output",
   CellTags->"b:3.3.1"],
  Cell[158585, 4859, 248, 7, 51, "Text",
   CellTags->"b:3.3.1"],
  Cell[158836, 4868, 379, 12, 46, "Input",
   CellTags->"b:3.3.1"]},
 "c:23"->{
  Cell[159252, 4885, 353, 11, 60, "Subsection",
   CellTags->{"c:23", "b:3.3.2"}]},
 "b:3.3.2"->{
  Cell[159252, 4885, 353, 11, 60, "Subsection",
   CellTags->{"c:23", "b:3.3.2"}],
  Cell[159608, 4898, 748, 24, 67, "Text",
   CellTags->"b:3.3.2"],
  Cell[160381, 4926, 503, 18, 62, "Input",
   CellTags->"b:3.3.2"],
  Cell[160887, 4946, 173, 5, 45, "Output",
   CellTags->"b:3.3.2"]},
 "c:24"->{
  Cell[161109, 4957, 469, 15, 63, "Subsection",
   CellTags->{"c:24", "b:3.3.3"}]},
 "b:3.3.3"->{
  Cell[161109, 4957, 469, 15, 63, "Subsection",
   CellTags->{"c:24", "b:3.3.3"}],
  Cell[161581, 4974, 118, 2, 23, "Text",
   CellTags->"b:3.3.3"],
  Cell[161724, 4980, 499, 16, 46, "Input",
   CellTags->"b:3.3.3"],
  Cell[162226, 4998, 176, 5, 45, "Output",
   CellTags->"b:3.3.3"],
  Cell[162417, 5006, 156, 5, 51, "Text",
   CellTags->"b:3.3.3"],
  Cell[162576, 5013, 1224, 41, 81, "Input",
   CellTags->"b:3.3.3"],
  Cell[163825, 5058, 267, 7, 62, "Input",
   CellTags->"b:3.3.3"],
  Cell[164095, 5067, 69174, 1147, 251, "Output",
   CellTags->"b:3.3.3"],
  Cell[233284, 6217, 106, 2, 23, "Text",
   CellTags->"b:3.3.3"]},
 "c:25"->{
  Cell[233427, 6224, 129, 3, 58, "Subsection",
   CellTags->{"c:25", "b:3.3.4"}]},
 "b:3.3.4"->{
  Cell[233427, 6224, 129, 3, 58, "Subsection",
   CellTags->{"c:25", "b:3.3.4"}],
  Cell[233559, 6229, 216, 6, 51, "Text",
   CellTags->"b:3.3.4"],
  Cell[233800, 6239, 363, 12, 59, "Input",
   CellTags->"b:3.3.4"],
  Cell[234166, 6253, 177, 5, 45, "Output",
   CellTags->"b:3.3.4"],
  Cell[234358, 6261, 258, 8, 23, "Text",
   CellTags->"b:3.3.4"],
  Cell[234641, 6273, 736, 23, 78, "Input",
   CellTags->"b:3.3.4"],
  Cell[235380, 6298, 771, 25, 126, "Output",
   CellTags->"b:3.3.4"],
  Cell[236154, 6325, 765, 25, 126, "Output",
   CellTags->"b:3.3.4"],
  Cell[236956, 6355, 228, 7, 62, "Input",
   CellTags->"b:3.3.4"],
  Cell[237187, 6364, 176, 5, 45, "Output",
   CellTags->"b:3.3.4"],
  Cell[237366, 6371, 2343, 51, 251, "Output",
   CellTags->"b:3.3.4"]},
 "c:26"->{
  Cell[239758, 6428, 138, 3, 58, "Subsection",
   CellTags->{"c:26", "b:3.3.5"}]},
 "b:3.3.5"->{
  Cell[239758, 6428, 138, 3, 58, "Subsection",
   CellTags->{"c:26", "b:3.3.5"}],
  Cell[239899, 6433, 240, 9, 25, "Text",
   CellTags->"b:3.3.5"],
  Cell[240142, 6444, 127, 4, 45, "Input",
   CellTags->"b:3.3.5"],
  Cell[240294, 6452, 905, 27, 81, "Input",
   CellTags->"b:3.3.5"],
  Cell[241202, 6481, 178, 5, 45, "Output",
   CellTags->"b:3.3.5"],
  Cell[241395, 6489, 127, 4, 45, "Input",
   CellTags->"b:3.3.5"],
  Cell[241547, 6497, 180, 4, 45, "Input",
   CellTags->"b:3.3.5"],
  Cell[241730, 6503, 356, 7, 21, "Print",
   CellTags->"b:3.3.5"],
  Cell[242101, 6513, 143, 4, 23, "Text",
   CellTags->"b:3.3.5"],
  Cell[242247, 6519, 120, 4, 45, "Input",
   CellTags->"b:3.3.5"],
  Cell[242370, 6525, 750, 23, 85, "Input",
   CellTags->"b:3.3.5"],
  Cell[243145, 6552, 496, 15, 78, "Input",
   CellTags->"b:3.3.5"],
  Cell[243644, 6569, 193, 5, 45, "Output",
   CellTags->"b:3.3.5"],
  Cell[243874, 6579, 810, 23, 78, "Input",
   CellTags->"b:3.3.5"],
  Cell[244687, 6604, 2301, 53, 218, "Output",
   CellTags->"b:3.3.5"],
  Cell[246991, 6659, 2720, 60, 221, "Output",
   CellTags->"b:3.3.5"]},
 "c:27"->{
  Cell[249760, 6725, 198, 6, 58, "Subsection",
   CellTags->{"c:27", "b:3.3.6"}]},
 "b:3.3.6"->{
  Cell[249760, 6725, 198, 6, 58, "Subsection",
   CellTags->{"c:27", "b:3.3.6"}],
  Cell[249961, 6733, 201, 6, 51, "Text",
   CellTags->"b:3.3.6"],
  Cell[250187, 6743, 54, 1, 46, "Input",
   CellTags->"b:3.3.6"],
  Cell[250244, 6746, 175, 5, 45, "Output",
   CellTags->"b:3.3.6"],
  Cell[250434, 6754, 89, 3, 45, "Input",
   CellTags->"b:3.3.6"],
  Cell[250526, 6759, 4219, 115, 267, "Input",
   CellTags->"b:3.3.6"],
  Cell[254770, 6878, 666, 18, 94, "Input",
   CellTags->"b:3.3.6"],
  Cell[255439, 6898, 877, 21, 94, "Output",
   CellTags->"b:3.3.6"],
  Cell[256319, 6921, 570, 12, 22, "Print",
   CellTags->"b:3.3.6"],
  Cell[256926, 6938, 81, 2, 45, "Input",
   CellTags->"b:3.3.6"],
  Cell[257010, 6942, 204, 5, 45, "Output",
   CellTags->"b:3.3.6"],
  Cell[257229, 6950, 52, 1, 45, "Input",
   CellTags->"b:3.3.6"]},
 "c:28"->{
  Cell[257330, 6957, 174, 7, 51, "Section",
   CellTags->"c:28"]},
 "c:29"->{
  Cell[257529, 6968, 180, 5, 58, "Subsection",
   CellTags->{"c:29", "b:3.4.0"}]},
 "b:3.4.0"->{
  Cell[257529, 6968, 180, 5, 58, "Subsection",
   CellTags->{"c:29", "b:3.4.0"}],
  Cell[257712, 6975, 133, 3, 46, "Input",
   CellTags->"b:3.4.0"],
  Cell[257848, 6980, 112, 3, 46, "Input",
   CellTags->"b:3.4.0"],
  Cell[257963, 6985, 168, 5, 23, "Text",
   CellTags->"b:3.4.0"],
  Cell[258134, 6992, 252, 7, 51, "Text",
   CellTags->"b:3.4.0"],
  Cell[258389, 7001, 407, 13, 63, "Input",
   CellTags->"b:3.4.0"],
  Cell[258799, 7016, 112, 2, 23, "Text",
   CellTags->"b:3.4.0"],
  Cell[258914, 7020, 115, 4, 45, "Input",
   CellTags->"b:3.4.0"]},
 "c:30"->{
  Cell[259066, 7029, 206, 6, 58, "Subsection",
   CellTags->{"c:30", "b:3.4.1"}]},
 "b:3.4.1"->{
  Cell[259066, 7029, 206, 6, 58, "Subsection",
   CellTags->{"c:30", "b:3.4.1"}],
  Cell[259275, 7037, 104, 2, 23, "Text",
   CellTags->"b:3.4.1"],
  Cell[259382, 7041, 95, 3, 45, "Input",
   CellTags->"b:3.4.1"],
  Cell[259480, 7046, 119, 2, 23, "Text",
   CellTags->"b:3.4.1"],
  Cell[259602, 7050, 90, 3, 45, "Input",
   CellTags->"b:3.4.1"],
  Cell[259695, 7055, 157, 5, 23, "Text",
   CellTags->"b:3.4.1"],
  Cell[259855, 7062, 327, 11, 45, "Input",
   CellTags->"b:3.4.1"],
  Cell[260185, 7075, 277, 8, 46, "Input",
   CellTags->"b:3.4.1"],
  Cell[260465, 7085, 680, 19, 78, "Input",
   CellTags->"b:3.4.1"],
  Cell[261148, 7106, 1121, 37, 173, "Text",
   CellTags->"b:3.4.1"],
  Cell[262272, 7145, 215, 7, 46, "Input",
   CellTags->"b:3.4.1"],
  Cell[262490, 7154, 464, 14, 94, "Input",
   CellTags->"b:3.4.1"],
  Cell[262957, 7170, 248, 7, 51, "Text",
   CellTags->"b:3.4.1"],
  Cell[263208, 7179, 379, 12, 46, "Input",
   CellTags->"b:3.4.1"]},
 "c:31"->{
  Cell[263624, 7196, 353, 11, 60, "Subsection",
   CellTags->{"c:31", "b:3.4.2"}]},
 "b:3.4.2"->{
  Cell[263624, 7196, 353, 11, 60, "Subsection",
   CellTags->{"c:31", "b:3.4.2"}],
  Cell[263980, 7209, 896, 29, 97, "Text",
   CellTags->"b:3.4.2"]},
 "c:32"->{
  Cell[264913, 7243, 469, 15, 63, "Subsection",
   CellTags->{"c:32", "b:3.4.3"}]},
 "b:3.4.3"->{
  Cell[264913, 7243, 469, 15, 63, "Subsection",
   CellTags->{"c:32", "b:3.4.3"}],
  Cell[265385, 7260, 99, 2, 23, "Text",
   CellTags->"b:3.4.3"],
  Cell[265487, 7264, 499, 16, 46, "Input",
   CellTags->"b:3.4.3"],
  Cell[265989, 7282, 1154, 40, 63, "Text",
   CellTags->"b:3.4.3"]},
 "c:33"->{
  Cell[267180, 7327, 149, 3, 58, "Subsection",
   CellTags->{"c:33", "b:3.4.4"}]},
 "b:3.4.4"->{
  Cell[267180, 7327, 149, 3, 58, "Subsection",
   CellTags->{"c:33", "b:3.4.4"}],
  Cell[267332, 7332, 286, 9, 55, "Text",
   CellTags->"b:3.4.4"],
  Cell[267621, 7343, 363, 12, 59, "Input",
   CellTags->"b:3.4.4"],
  Cell[267987, 7357, 315, 10, 82, "Input",
   CellTags->"b:3.4.4"],
  Cell[268305, 7369, 425, 16, 25, "Text",
   CellTags->"b:3.4.4"],
  Cell[268733, 7387, 906, 29, 62, "Input",
   CellTags->"b:3.4.4"],
  Cell[269642, 7418, 287, 10, 25, "Text",
   CellTags->"b:3.4.4"],
  Cell[269932, 7430, 1964, 62, 105, "Input",
   CellTags->"b:3.4.4"],
  Cell[271899, 7494, 134, 4, 23, "Text",
   CellTags->"b:3.4.4"],
  Cell[272036, 7500, 764, 25, 78, "Input",
   CellTags->"b:3.4.4"],
  Cell[272803, 7527, 80, 2, 45, "Input",
   CellTags->"b:3.4.4"]},
 "c:34"->{
  Cell[272920, 7534, 143, 3, 58, "Subsection",
   CellTags->{"c:34", "b:3.4.5"}]},
 "b:3.4.5"->{
  Cell[272920, 7534, 143, 3, 58, "Subsection",
   CellTags->{"c:34", "b:3.4.5"}],
  Cell[273066, 7539, 160, 5, 23, "Text",
   CellTags->"b:3.4.5"],
  Cell[273229, 7546, 127, 4, 45, "Input",
   CellTags->"b:3.4.5"],
  Cell[273359, 7552, 132, 4, 46, "Input",
   CellTags->"b:3.4.5"],
  Cell[273494, 7558, 291, 8, 61, "Input",
   CellTags->"b:3.4.5"]},
 "c:35"->{
  Cell[273822, 7571, 190, 6, 58, "Subsection",
   CellTags->{"c:35", "b:3.4.6"}]},
 "b:3.4.6"->{
  Cell[273822, 7571, 190, 6, 58, "Subsection",
   CellTags->{"c:35", "b:3.4.6"}],
  Cell[274015, 7579, 154, 5, 23, "Text",
   CellTags->"b:3.4.6"],
  Cell[274172, 7586, 120, 4, 45, "Input",
   CellTags->"b:3.4.6"],
  Cell[274295, 7592, 688, 22, 85, "Input",
   CellTags->"b:3.4.6"],
  Cell[274986, 7616, 156, 5, 23, "Text",
   CellTags->"b:3.4.6"],
  Cell[275145, 7623, 803, 22, 78, "Input",
   CellTags->"b:3.4.6"]},
 "c:36"->{
  Cell[275985, 7650, 169, 5, 58, "Subsection",
   CellTags->{"c:36", "b:3.4.7"}]},
 "b:3.4.7"->{
  Cell[275985, 7650, 169, 5, 58, "Subsection",
   CellTags->{"c:36", "b:3.4.7"}],
  Cell[276157, 7657, 514, 12, 136, "Text",
   CellTags->"b:3.4.7"],
  Cell[276674, 7671, 89, 3, 45, "Input",
   CellTags->"b:3.4.7"],
  Cell[276766, 7676, 4219, 115, 267, "Input",
   CellTags->"b:3.4.7"],
  Cell[280988, 7793, 767, 20, 110, "Input",
   CellTags->"b:3.4.7"],
  Cell[281758, 7815, 81, 2, 45, "Input",
   CellTags->"b:3.4.7"],
  Cell[281842, 7819, 264, 7, 79, "Text",
   CellTags->"b:3.4.7"],
  Cell[282109, 7828, 503, 15, 62, "Input",
   CellTags->"b:3.4.7"],
  Cell[282615, 7845, 43, 1, 45, "Input",
   CellTags->"b:3.4.7"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"c:1", 284821, 7896},
 {"c:2", 284889, 7899},
 {"b:3.0", 285036, 7904},
 {"c:3", 285247, 7911},
 {"b:3.1.0", 285340, 7914},
 {"c:4", 286424, 7947},
 {"b:3.1.1", 286519, 7950},
 {"c:5", 287735, 7987},
 {"b:3.1.2", 287831, 7990},
 {"c:6", 287989, 7995},
 {"b:3.1.3", 288085, 7998},
 {"c:7", 288442, 8009},
 {"b:3.1.4", 288537, 8012},
 {"c:8", 289952, 8055},
 {"b:3.1.5", 290047, 8058},
 {"c:9", 290668, 8077},
 {"b:3.1.6", 290763, 8080},
 {"c:10", 291325, 8097},
 {"b:3.1.7", 291421, 8100},
 {"c:11", 292451, 8131},
 {"c:12", 292528, 8134},
 {"b:3.2.0", 292624, 8137},
 {"c:13", 293517, 8164},
 {"b:3.2.1", 293613, 8167},
 {"c:14", 294630, 8198},
 {"b:3.2.2", 294727, 8201},
 {"c:15", 294887, 8206},
 {"b:3.2.3", 294984, 8209},
 {"c:16", 295345, 8220},
 {"b:3.2.4", 295442, 8223},
 {"c:17", 296274, 8248},
 {"b:3.2.5", 296371, 8251},
 {"c:18", 296733, 8262},
 {"b:3.2.6", 296830, 8265},
 {"c:19", 297259, 8278},
 {"b:3.2.7", 297356, 8281},
 {"c:20", 297990, 8300},
 {"c:21", 298068, 8303},
 {"b:3.3.0", 298165, 8306},
 {"c:22", 299210, 8337},
 {"b:3.3.1", 299307, 8340},
 {"c:23", 300408, 8373},
 {"b:3.3.2", 300506, 8376},
 {"c:24", 300804, 8385},
 {"b:3.3.3", 300902, 8388},
 {"c:25", 301541, 8407},
 {"b:3.3.4", 301638, 8410},
 {"c:26", 302414, 8433},
 {"b:3.3.5", 302511, 8436},
 {"c:27", 303622, 8469},
 {"b:3.3.6", 303719, 8472},
 {"c:28", 304555, 8497},
 {"c:29", 304633, 8500},
 {"b:3.4.0", 304730, 8503},
 {"c:30", 305291, 8520},
 {"b:3.4.1", 305388, 8523},
 {"c:31", 306353, 8552},
 {"b:3.4.2", 306451, 8555},
 {"c:32", 306613, 8560},
 {"b:3.4.3", 306711, 8563},
 {"c:33", 307007, 8572},
 {"b:3.4.4", 307104, 8575},
 {"c:34", 307872, 8598},
 {"b:3.4.5", 307969, 8601},
 {"c:35", 308330, 8612},
 {"b:3.4.6", 308427, 8615},
 {"c:36", 308856, 8628},
 {"b:3.4.7", 308953, 8631}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 62, 1, 51, "Title",
 CellTags->"c:1"],
Cell[610, 23, 63, 0, 35, "Subtitle"],
Cell[676, 25, 317, 8, 20, "Abstract"],
Cell[CellGroupData[{
Cell[1018, 37, 276, 6, 51, "Section",
 CellTags->{"c:2", "b:3.0"}],
Cell[1297, 45, 2894, 67, 234, "Abstract",
 CellTags->"b:3.0"],
Cell[4194, 114, 175, 3, 22, "Text",
 CellTags->"b:3.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4406, 122, 106, 2, 51, "Section",
 CellTags->"c:2"],
Cell[CellGroupData[{
Cell[4537, 128, 179, 5, 58, "Subsection",
 CellTags->{"c:3", "b:3.1.0"}],
Cell[CellGroupData[{
Cell[4741, 137, 133, 3, 46, "Input",
 CellTags->"b:3.1.0"],
Cell[4877, 142, 265, 4, 22, "Print",
 CellTags->"b:3.1.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5179, 151, 112, 3, 46, "Input",
 CellTags->"b:3.1.0"],
Cell[5294, 156, 33535, 941, 579, "Print",
 CellTags->"b:3.1.0"]
}, Open  ]],
Cell[38844, 1100, 198, 5, 23, "Text",
 CellTags->"b:3.1.0"],
Cell[39045, 1107, 1476, 42, 85, "Input",
 CellTags->"b:3.1.0"],
Cell[40524, 1151, 813, 24, 78, "Input",
 CellTags->"b:3.1.0"],
Cell[41340, 1177, 1101, 32, 78, "Input",
 CellTags->"b:3.1.0"],
Cell[42444, 1211, 786, 25, 62, "Input",
 CellTags->"b:3.1.0"],
Cell[43233, 1238, 598, 18, 62, "Input",
 CellTags->"b:3.1.0"],
Cell[43834, 1258, 2561, 67, 142, "Input",
 CellTags->"b:3.1.0"],
Cell[46398, 1327, 252, 7, 51, "Text",
 CellTags->"b:3.1.0"],
Cell[46653, 1336, 407, 13, 63, "Input",
 CellTags->"b:3.1.0"],
Cell[47063, 1351, 112, 2, 23, "Text",
 CellTags->"b:3.1.0"],
Cell[47178, 1355, 115, 4, 45, "Input",
 CellTags->"b:3.1.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47330, 1364, 212, 6, 58, "Subsection",
 CellTags->{"c:4", "b:3.1.1"}],
Cell[47545, 1372, 104, 2, 23, "Text",
 CellTags->"b:3.1.1"],
Cell[47652, 1376, 90, 3, 45, "Input",
 CellTags->"b:3.1.1"],
Cell[47745, 1381, 119, 2, 23, "Text",
 CellTags->"b:3.1.1"],
Cell[47867, 1385, 90, 3, 45, "Input",
 CellTags->"b:3.1.1"],
Cell[47960, 1390, 192, 5, 23, "Text",
 CellTags->"b:3.1.1"],
Cell[48155, 1397, 327, 11, 45, "Input",
 CellTags->"b:3.1.1"],
Cell[48485, 1410, 277, 8, 46, "Input",
 CellTags->"b:3.1.1"],
Cell[48765, 1420, 680, 19, 78, "Input",
 CellTags->"b:3.1.1"],
Cell[49448, 1441, 1122, 37, 173, "Text",
 CellTags->"b:3.1.1"],
Cell[CellGroupData[{
Cell[50595, 1482, 215, 7, 46, "Input",
 CellTags->"b:3.1.1"],
Cell[50813, 1491, 252, 6, 45, "Output",
 CellTags->"b:3.1.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51102, 1502, 513, 16, 94, "Input",
 CellTags->"b:3.1.1"],
Cell[51618, 1520, 250, 6, 45, "Output",
 CellTags->"b:3.1.1"],
Cell[51871, 1528, 248, 6, 45, "Output",
 CellTags->"b:3.1.1"]
}, Open  ]],
Cell[52134, 1537, 249, 7, 51, "Text",
 CellTags->"b:3.1.1"],
Cell[CellGroupData[{
Cell[52408, 1548, 379, 12, 46, "Input",
 CellTags->"b:3.1.1"],
Cell[52790, 1562, 288, 4, 22, "Print",
 CellTags->"b:3.1.1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[53127, 1572, 352, 11, 60, "Subsection",
 CellTags->{"c:5", "b:3.1.2"}],
Cell[53482, 1585, 896, 29, 97, "Text",
 CellTags->"b:3.1.2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54415, 1619, 468, 15, 63, "Subsection",
 CellTags->{"c:6", "b:3.1.3"}],
Cell[54886, 1636, 99, 2, 23, "Text",
 CellTags->"b:3.1.3"],
Cell[CellGroupData[{
Cell[55010, 1642, 499, 16, 46, "Input",
 CellTags->"b:3.1.3"],
Cell[55512, 1660, 252, 6, 45, "Output",
 CellTags->"b:3.1.3"]
}, Open  ]],
Cell[55779, 1669, 1154, 40, 63, "Text",
 CellTags->"b:3.1.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56970, 1714, 148, 3, 58, "Subsection",
 CellTags->{"c:7", "b:3.1.4"}],
Cell[57121, 1719, 287, 9, 55, "Text",
 CellTags->"b:3.1.4"],
Cell[CellGroupData[{
Cell[57433, 1732, 363, 12, 59, "Input",
 CellTags->"b:3.1.4"],
Cell[57799, 1746, 250, 6, 45, "Output",
 CellTags->"b:3.1.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58086, 1757, 315, 10, 82, "Input",
 CellTags->"b:3.1.4"],
Cell[58404, 1769, 250, 6, 45, "Output",
 CellTags->"b:3.1.4"],
Cell[58657, 1777, 250, 6, 45, "Output",
 CellTags->"b:3.1.4"],
Cell[58910, 1785, 250, 6, 45, "Output",
 CellTags->"b:3.1.4"]
}, Open  ]],
Cell[59175, 1794, 425, 16, 25, "Text",
 CellTags->"b:3.1.4"],
Cell[59603, 1812, 446, 14, 46, "Input",
 CellTags->"b:3.1.4"],
Cell[60052, 1828, 287, 10, 25, "Text",
 CellTags->"b:3.1.4"],
Cell[60342, 1840, 1054, 34, 85, "Input",
 CellTags->"b:3.1.4"],
Cell[61399, 1876, 134, 4, 23, "Text",
 CellTags->"b:3.1.4"],
Cell[61536, 1882, 764, 25, 78, "Input",
 CellTags->"b:3.1.4"],
Cell[62303, 1909, 223, 8, 45, "Input",
 CellTags->"b:3.1.4"],
Cell[62529, 1919, 83, 2, 23, "Text",
 CellTags->"b:3.1.4"],
Cell[62615, 1923, 98, 3, 45, "Input",
 CellTags->"b:3.1.4"],
Cell[62716, 1928, 95, 2, 23, "Text",
 CellTags->"b:3.1.4"],
Cell[62814, 1932, 653, 20, 46, "Input",
 CellTags->"b:3.1.4"],
Cell[63470, 1954, 597, 19, 69, "Input",
 CellTags->"b:3.1.4"],
Cell[64070, 1975, 98, 3, 45, "Input",
 CellTags->"b:3.1.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64205, 1983, 142, 3, 58, "Subsection",
 CellTags->{"c:8", "b:3.1.5"}],
Cell[64350, 1988, 131, 3, 23, "Text",
 CellTags->"b:3.1.5"],
Cell[64484, 1993, 477, 15, 46, "Input",
 CellTags->"b:3.1.5"],
Cell[64964, 2010, 127, 4, 45, "Input",
 CellTags->"b:3.1.5"],
Cell[65094, 2016, 438, 13, 50, "Input",
 CellTags->"b:3.1.5"],
Cell[CellGroupData[{
Cell[65557, 2033, 291, 8, 61, "Input",
 CellTags->"b:3.1.5"],
Cell[65851, 2043, 427, 8, 21, "Print",
 CellTags->"b:3.1.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[66315, 2056, 114, 3, 46, "Input",
 CellTags->"b:3.1.5"],
Cell[66432, 2061, 265, 6, 45, "Output",
 CellTags->"b:3.1.5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[66746, 2073, 189, 6, 58, "Subsection",
 CellTags->{"c:9", "b:3.1.6"}],
Cell[66938, 2081, 154, 5, 23, "Text",
 CellTags->"b:3.1.6"],
Cell[67095, 2088, 120, 4, 45, "Input",
 CellTags->"b:3.1.6"],
Cell[67218, 2094, 678, 21, 85, "Input",
 CellTags->"b:3.1.6"],
Cell[67899, 2117, 156, 5, 23, "Text",
 CellTags->"b:3.1.6"],
Cell[CellGroupData[{
Cell[68080, 2126, 803, 22, 78, "Input",
 CellTags->"b:3.1.6"],
Cell[68886, 2150, 3593, 69, 222, "Output",
 CellTags->"b:3.1.6"],
Cell[72482, 2221, 1697, 41, 225, "Output",
 CellTags->"b:3.1.6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[74228, 2268, 169, 5, 58, "Subsection",
 CellTags->{"c:10", "b:3.1.7"}],
Cell[74400, 2275, 512, 12, 136, "Text",
 CellTags->"b:3.1.7"],
Cell[74915, 2289, 89, 3, 45, "Input",
 CellTags->"b:3.1.7"],
Cell[75007, 2294, 4219, 115, 267, "Input",
 CellTags->"b:3.1.7"],
Cell[CellGroupData[{
Cell[79251, 2413, 1016, 27, 126, "Input",
 CellTags->"b:3.1.7"],
Cell[80270, 2442, 845, 20, 62, "Output",
 CellTags->"b:3.1.7"],
Cell[81118, 2464, 308, 7, 22, "Print",
 CellTags->"b:3.1.7"],
Cell[81429, 2473, 607, 13, 22, "Print",
 CellTags->"b:3.1.7"],
Cell[82039, 2488, 256, 4, 22, "Print",
 CellTags->"b:3.1.7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82332, 2497, 81, 2, 45, "Input",
 CellTags->"b:3.1.7"],
Cell[82416, 2501, 275, 6, 45, "Output",
 CellTags->"b:3.1.7"]
}, Open  ]],
Cell[82706, 2510, 264, 7, 79, "Text",
 CellTags->"b:3.1.7"],
Cell[CellGroupData[{
Cell[82995, 2521, 413, 13, 66, "Input",
 CellTags->"b:3.1.7"],
Cell[83411, 2536, 1330, 35, 261, "Output",
 CellTags->"b:3.1.7"]
}, Open  ]],
Cell[84756, 2574, 43, 1, 45, "Input",
 CellTags->"b:3.1.7"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[84848, 2581, 129, 4, 51, "Section",
 CellTags->"c:11"],
Cell[CellGroupData[{
Cell[85002, 2589, 180, 5, 58, "Subsection",
 CellTags->{"c:12", "b:3.2.0"}],
Cell[85185, 2596, 133, 3, 46, "Input",
 CellTags->"b:3.2.0"],
Cell[85321, 2601, 112, 3, 46, "Input",
 CellTags->"b:3.2.0"],
Cell[85436, 2606, 166, 5, 23, "Text",
 CellTags->"b:3.2.0"],
Cell[85605, 2613, 1050, 34, 62, "Input",
 CellTags->"b:3.2.0"],
Cell[86658, 2649, 2290, 59, 141, "Input",
 CellTags->"b:3.2.0"],
Cell[88951, 2710, 520, 14, 62, "Input",
 CellTags->"b:3.2.0"],
Cell[89474, 2726, 2248, 59, 141, "Input",
 CellTags->"b:3.2.0"],
Cell[91725, 2787, 521, 14, 62, "Input",
 CellTags->"b:3.2.0"],
Cell[92249, 2803, 252, 7, 51, "Text",
 CellTags->"b:3.2.0"],
Cell[92504, 2812, 407, 13, 63, "Input",
 CellTags->"b:3.2.0"],
Cell[92914, 2827, 112, 2, 23, "Text",
 CellTags->"b:3.2.0"],
Cell[93029, 2831, 115, 4, 45, "Input",
 CellTags->"b:3.2.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93181, 2840, 206, 6, 58, "Subsection",
 CellTags->{"c:13", "b:3.2.1"}],
Cell[93390, 2848, 104, 2, 23, "Text",
 CellTags->"b:3.2.1"],
Cell[93497, 2852, 96, 3, 45, "Input",
 CellTags->"b:3.2.1"],
Cell[93596, 2857, 119, 2, 23, "Text",
 CellTags->"b:3.2.1"],
Cell[93718, 2861, 90, 3, 45, "Input",
 CellTags->"b:3.2.1"],
Cell[93811, 2866, 175, 5, 23, "Text",
 CellTags->"b:3.2.1"],
Cell[93989, 2873, 327, 11, 45, "Input",
 CellTags->"b:3.2.1"],
Cell[94319, 2886, 277, 8, 46, "Input",
 CellTags->"b:3.2.1"],
Cell[94599, 2896, 680, 19, 78, "Input",
 CellTags->"b:3.2.1"],
Cell[95282, 2917, 1123, 37, 173, "Text",
 CellTags->"b:3.2.1"],
Cell[96408, 2956, 215, 7, 46, "Input",
 CellTags->"b:3.2.1"],
Cell[96626, 2965, 284, 9, 25, "Text",
 CellTags->"b:3.2.1"],
Cell[96913, 2976, 701, 20, 110, "Input",
 CellTags->"b:3.2.1"],
Cell[97617, 2998, 249, 7, 51, "Text",
 CellTags->"b:3.2.1"],
Cell[97869, 3007, 379, 12, 46, "Input",
 CellTags->"b:3.2.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[98285, 3024, 353, 11, 60, "Subsection",
 CellTags->{"c:14", "b:3.2.2"}],
Cell[98641, 3037, 896, 29, 97, "Text",
 CellTags->"b:3.2.2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[99574, 3071, 469, 15, 63, "Subsection",
 CellTags->{"c:15", "b:3.2.3"}],
Cell[100046, 3088, 99, 2, 23, "Text",
 CellTags->"b:3.2.3"],
Cell[100148, 3092, 499, 16, 46, "Input",
 CellTags->"b:3.2.3"],
Cell[100650, 3110, 77, 2, 46, "Input",
 CellTags->"b:3.2.3"],
Cell[100730, 3114, 1154, 40, 63, "Text",
 CellTags->"b:3.2.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[101921, 3159, 149, 3, 58, "Subsection",
 CellTags->{"c:16", "b:3.2.4"}],
Cell[102073, 3164, 286, 9, 55, "Text",
 CellTags->"b:3.2.4"],
Cell[102362, 3175, 363, 12, 59, "Input",
 CellTags->"b:3.2.4"],
Cell[102728, 3189, 315, 10, 82, "Input",
 CellTags->"b:3.2.4"],
Cell[103046, 3201, 425, 16, 25, "Text",
 CellTags->"b:3.2.4"],
Cell[103474, 3219, 446, 14, 46, "Input",
 CellTags->"b:3.2.4"],
Cell[103923, 3235, 287, 10, 25, "Text",
 CellTags->"b:3.2.4"],
Cell[104213, 3247, 1054, 34, 85, "Input",
 CellTags->"b:3.2.4"],
Cell[105270, 3283, 134, 4, 23, "Text",
 CellTags->"b:3.2.4"],
Cell[105407, 3289, 764, 25, 78, "Input",
 CellTags->"b:3.2.4"],
Cell[106174, 3316, 81, 2, 23, "Text",
 CellTags->"b:3.2.4"],
Cell[106258, 3320, 98, 3, 45, "Input",
 CellTags->"b:3.2.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[106393, 3328, 143, 3, 58, "Subsection",
 CellTags->{"c:17", "b:3.2.5"}],
Cell[106539, 3333, 160, 5, 23, "Text",
 CellTags->"b:3.2.5"],
Cell[106702, 3340, 127, 4, 45, "Input",
 CellTags->"b:3.2.5"],
Cell[106832, 3346, 493, 10, 78, "Input"],
Cell[107328, 3358, 913, 26, 77, "Input",
 CellTags->"b:3.2.5"],
Cell[108244, 3386, 291, 8, 61, "Input",
 CellTags->"b:3.2.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108572, 3399, 190, 6, 58, "Subsection",
 CellTags->{"c:18", "b:3.2.6"}],
Cell[108765, 3407, 154, 5, 23, "Text",
 CellTags->"b:3.2.6"],
Cell[108922, 3414, 120, 4, 45, "Input",
 CellTags->"b:3.2.6"],
Cell[109045, 3420, 750, 23, 85, "Input",
 CellTags->"b:3.2.6"],
Cell[109798, 3445, 156, 5, 23, "Text",
 CellTags->"b:3.2.6"],
Cell[109957, 3452, 803, 22, 78, "Input",
 CellTags->"b:3.2.6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110797, 3479, 169, 5, 58, "Subsection",
 CellTags->{"c:19", "b:3.2.7"}],
Cell[110969, 3486, 512, 12, 136, "Text",
 CellTags->"b:3.2.7"],
Cell[111484, 3500, 89, 3, 45, "Input",
 CellTags->"b:3.2.7"],
Cell[111576, 3505, 4219, 115, 267, "Input",
 CellTags->"b:3.2.7"],
Cell[115798, 3622, 767, 20, 110, "Input",
 CellTags->"b:3.2.7"],
Cell[116568, 3644, 81, 2, 45, "Input",
 CellTags->"b:3.2.7"],
Cell[116652, 3648, 264, 7, 79, "Text",
 CellTags->"b:3.2.7"],
Cell[116919, 3657, 413, 13, 66, "Input",
 CellTags->"b:3.2.7"],
Cell[117335, 3672, 43, 1, 45, "Input",
 CellTags->"b:3.2.7"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[117427, 3679, 162, 5, 51, "Section",
 CellTags->"c:20"],
Cell[CellGroupData[{
Cell[117614, 3688, 272, 9, 58, "Subsection",
 CellTags->{"c:21", "b:3.3.0"}],
Cell[CellGroupData[{
Cell[117911, 3701, 133, 3, 46, "Input",
 CellTags->"b:3.3.0"],
Cell[118047, 3706, 197, 3, 22, "Print",
 CellTags->"b:3.3.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118281, 3714, 112, 3, 46, "Input",
 CellTags->"b:3.3.0"],
Cell[118396, 3719, 28584, 764, 579, "Print",
 CellTags->"b:3.3.0"]
}, Open  ]],
Cell[146995, 4486, 139, 4, 23, "Text",
 CellTags->"b:3.3.0"],
Cell[147137, 4492, 1050, 34, 62, "Input",
 CellTags->"b:3.3.0"],
Cell[148190, 4528, 2311, 59, 141, "Input",
 CellTags->"b:3.3.0"],
Cell[150504, 4589, 520, 14, 62, "Input",
 CellTags->"b:3.3.0"],
Cell[151027, 4605, 2292, 60, 141, "Input",
 CellTags->"b:3.3.0"],
Cell[153322, 4667, 593, 15, 62, "Input",
 CellTags->"b:3.3.0"],
Cell[153918, 4684, 260, 6, 51, "Text",
 CellTags->"b:3.3.0"],
Cell[154181, 4692, 407, 13, 63, "Input",
 CellTags->"b:3.3.0"],
Cell[154591, 4707, 112, 2, 23, "Text",
 CellTags->"b:3.3.0"],
Cell[154706, 4711, 115, 4, 45, "Input",
 CellTags->"b:3.3.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[154858, 4720, 214, 6, 58, "Subsection",
 CellTags->{"c:22", "b:3.3.1"}],
Cell[155075, 4728, 107, 2, 23, "Text",
 CellTags->"b:3.3.1"],
Cell[155185, 4732, 90, 3, 45, "Input",
 CellTags->"b:3.3.1"],
Cell[155278, 4737, 119, 2, 23, "Text",
 CellTags->"b:3.3.1"],
Cell[155400, 4741, 90, 3, 45, "Input",
 CellTags->"b:3.3.1"],
Cell[155493, 4746, 175, 5, 23, "Text",
 CellTags->"b:3.3.1"],
Cell[155671, 4753, 301, 10, 46, "Input",
 CellTags->"b:3.3.1"],
Cell[155975, 4765, 680, 19, 78, "Input",
 CellTags->"b:3.3.1"],
Cell[CellGroupData[{
Cell[156680, 4788, 215, 7, 46, "Input",
 CellTags->"b:3.3.1"],
Cell[156898, 4797, 183, 5, 45, "Output",
 CellTags->"b:3.3.1"]
}, Open  ]],
Cell[157096, 4805, 576, 17, 111, "Text",
 CellTags->"b:3.3.1"],
Cell[CellGroupData[{
Cell[157697, 4826, 513, 16, 94, "Input",
 CellTags->"b:3.3.1"],
Cell[158213, 4844, 176, 5, 45, "Output",
 CellTags->"b:3.3.1"],
Cell[158392, 4851, 178, 5, 45, "Output",
 CellTags->"b:3.3.1"]
}, Open  ]],
Cell[158585, 4859, 248, 7, 51, "Text",
 CellTags->"b:3.3.1"],
Cell[158836, 4868, 379, 12, 46, "Input",
 CellTags->"b:3.3.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[159252, 4885, 353, 11, 60, "Subsection",
 CellTags->{"c:23", "b:3.3.2"}],
Cell[159608, 4898, 748, 24, 67, "Text",
 CellTags->"b:3.3.2"],
Cell[CellGroupData[{
Cell[160381, 4926, 503, 18, 62, "Input",
 CellTags->"b:3.3.2"],
Cell[160887, 4946, 173, 5, 45, "Output",
 CellTags->"b:3.3.2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[161109, 4957, 469, 15, 63, "Subsection",
 CellTags->{"c:24", "b:3.3.3"}],
Cell[161581, 4974, 118, 2, 23, "Text",
 CellTags->"b:3.3.3"],
Cell[CellGroupData[{
Cell[161724, 4980, 499, 16, 46, "Input",
 CellTags->"b:3.3.3"],
Cell[162226, 4998, 176, 5, 45, "Output",
 CellTags->"b:3.3.3"]
}, Open  ]],
Cell[162417, 5006, 156, 5, 51, "Text",
 CellTags->"b:3.3.3"],
Cell[162576, 5013, 1224, 41, 81, "Input",
 CellTags->"b:3.3.3"],
Cell[CellGroupData[{
Cell[163825, 5058, 267, 7, 62, "Input",
 CellTags->"b:3.3.3"],
Cell[164095, 5067, 69174, 1147, 251, "Output",
 CellTags->"b:3.3.3"]
}, Open  ]],
Cell[233284, 6217, 106, 2, 23, "Text",
 CellTags->"b:3.3.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[233427, 6224, 129, 3, 58, "Subsection",
 CellTags->{"c:25", "b:3.3.4"}],
Cell[233559, 6229, 216, 6, 51, "Text",
 CellTags->"b:3.3.4"],
Cell[CellGroupData[{
Cell[233800, 6239, 363, 12, 59, "Input",
 CellTags->"b:3.3.4"],
Cell[234166, 6253, 177, 5, 45, "Output",
 CellTags->"b:3.3.4"]
}, Open  ]],
Cell[234358, 6261, 258, 8, 23, "Text",
 CellTags->"b:3.3.4"],
Cell[CellGroupData[{
Cell[234641, 6273, 736, 23, 78, "Input",
 CellTags->"b:3.3.4"],
Cell[235380, 6298, 771, 25, 126, "Output",
 CellTags->"b:3.3.4"],
Cell[236154, 6325, 765, 25, 126, "Output",
 CellTags->"b:3.3.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[236956, 6355, 228, 7, 62, "Input",
 CellTags->"b:3.3.4"],
Cell[237187, 6364, 176, 5, 45, "Output",
 CellTags->"b:3.3.4"],
Cell[237366, 6371, 2343, 51, 251, "Output",
 CellTags->"b:3.3.4"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[239758, 6428, 138, 3, 58, "Subsection",
 CellTags->{"c:26", "b:3.3.5"}],
Cell[239899, 6433, 240, 9, 25, "Text",
 CellTags->"b:3.3.5"],
Cell[240142, 6444, 127, 4, 45, "Input",
 CellTags->"b:3.3.5"],
Cell[CellGroupData[{
Cell[240294, 6452, 905, 27, 81, "Input",
 CellTags->"b:3.3.5"],
Cell[241202, 6481, 178, 5, 45, "Output",
 CellTags->"b:3.3.5"]
}, Open  ]],
Cell[241395, 6489, 127, 4, 45, "Input",
 CellTags->"b:3.3.5"],
Cell[CellGroupData[{
Cell[241547, 6497, 180, 4, 45, "Input",
 CellTags->"b:3.3.5"],
Cell[241730, 6503, 356, 7, 21, "Print",
 CellTags->"b:3.3.5"]
}, Open  ]],
Cell[242101, 6513, 143, 4, 23, "Text",
 CellTags->"b:3.3.5"],
Cell[242247, 6519, 120, 4, 45, "Input",
 CellTags->"b:3.3.5"],
Cell[242370, 6525, 750, 23, 85, "Input",
 CellTags->"b:3.3.5"],
Cell[CellGroupData[{
Cell[243145, 6552, 496, 15, 78, "Input",
 CellTags->"b:3.3.5"],
Cell[243644, 6569, 193, 5, 45, "Output",
 CellTags->"b:3.3.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[243874, 6579, 810, 23, 78, "Input",
 CellTags->"b:3.3.5"],
Cell[244687, 6604, 2301, 53, 218, "Output",
 CellTags->"b:3.3.5"],
Cell[246991, 6659, 2720, 60, 221, "Output",
 CellTags->"b:3.3.5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[249760, 6725, 198, 6, 58, "Subsection",
 CellTags->{"c:27", "b:3.3.6"}],
Cell[249961, 6733, 201, 6, 51, "Text",
 CellTags->"b:3.3.6"],
Cell[CellGroupData[{
Cell[250187, 6743, 54, 1, 46, "Input",
 CellTags->"b:3.3.6"],
Cell[250244, 6746, 175, 5, 45, "Output",
 CellTags->"b:3.3.6"]
}, Open  ]],
Cell[250434, 6754, 89, 3, 45, "Input",
 CellTags->"b:3.3.6"],
Cell[250526, 6759, 4219, 115, 267, "Input",
 CellTags->"b:3.3.6"],
Cell[CellGroupData[{
Cell[254770, 6878, 666, 18, 94, "Input",
 CellTags->"b:3.3.6"],
Cell[255439, 6898, 877, 21, 94, "Output",
 CellTags->"b:3.3.6"],
Cell[256319, 6921, 570, 12, 22, "Print",
 CellTags->"b:3.3.6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[256926, 6938, 81, 2, 45, "Input",
 CellTags->"b:3.3.6"],
Cell[257010, 6942, 204, 5, 45, "Output",
 CellTags->"b:3.3.6"]
}, Open  ]],
Cell[257229, 6950, 52, 1, 45, "Input",
 CellTags->"b:3.3.6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[257330, 6957, 174, 7, 51, "Section",
 CellTags->"c:28"],
Cell[CellGroupData[{
Cell[257529, 6968, 180, 5, 58, "Subsection",
 CellTags->{"c:29", "b:3.4.0"}],
Cell[257712, 6975, 133, 3, 46, "Input",
 CellTags->"b:3.4.0"],
Cell[257848, 6980, 112, 3, 46, "Input",
 CellTags->"b:3.4.0"],
Cell[257963, 6985, 168, 5, 23, "Text",
 CellTags->"b:3.4.0"],
Cell[258134, 6992, 252, 7, 51, "Text",
 CellTags->"b:3.4.0"],
Cell[258389, 7001, 407, 13, 63, "Input",
 CellTags->"b:3.4.0"],
Cell[258799, 7016, 112, 2, 23, "Text",
 CellTags->"b:3.4.0"],
Cell[258914, 7020, 115, 4, 45, "Input",
 CellTags->"b:3.4.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[259066, 7029, 206, 6, 58, "Subsection",
 CellTags->{"c:30", "b:3.4.1"}],
Cell[259275, 7037, 104, 2, 23, "Text",
 CellTags->"b:3.4.1"],
Cell[259382, 7041, 95, 3, 45, "Input",
 CellTags->"b:3.4.1"],
Cell[259480, 7046, 119, 2, 23, "Text",
 CellTags->"b:3.4.1"],
Cell[259602, 7050, 90, 3, 45, "Input",
 CellTags->"b:3.4.1"],
Cell[259695, 7055, 157, 5, 23, "Text",
 CellTags->"b:3.4.1"],
Cell[259855, 7062, 327, 11, 45, "Input",
 CellTags->"b:3.4.1"],
Cell[260185, 7075, 277, 8, 46, "Input",
 CellTags->"b:3.4.1"],
Cell[260465, 7085, 680, 19, 78, "Input",
 CellTags->"b:3.4.1"],
Cell[261148, 7106, 1121, 37, 173, "Text",
 CellTags->"b:3.4.1"],
Cell[262272, 7145, 215, 7, 46, "Input",
 CellTags->"b:3.4.1"],
Cell[262490, 7154, 464, 14, 94, "Input",
 CellTags->"b:3.4.1"],
Cell[262957, 7170, 248, 7, 51, "Text",
 CellTags->"b:3.4.1"],
Cell[263208, 7179, 379, 12, 46, "Input",
 CellTags->"b:3.4.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[263624, 7196, 353, 11, 60, "Subsection",
 CellTags->{"c:31", "b:3.4.2"}],
Cell[263980, 7209, 896, 29, 97, "Text",
 CellTags->"b:3.4.2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[264913, 7243, 469, 15, 63, "Subsection",
 CellTags->{"c:32", "b:3.4.3"}],
Cell[265385, 7260, 99, 2, 23, "Text",
 CellTags->"b:3.4.3"],
Cell[265487, 7264, 499, 16, 46, "Input",
 CellTags->"b:3.4.3"],
Cell[265989, 7282, 1154, 40, 63, "Text",
 CellTags->"b:3.4.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[267180, 7327, 149, 3, 58, "Subsection",
 CellTags->{"c:33", "b:3.4.4"}],
Cell[267332, 7332, 286, 9, 55, "Text",
 CellTags->"b:3.4.4"],
Cell[267621, 7343, 363, 12, 59, "Input",
 CellTags->"b:3.4.4"],
Cell[267987, 7357, 315, 10, 82, "Input",
 CellTags->"b:3.4.4"],
Cell[268305, 7369, 425, 16, 25, "Text",
 CellTags->"b:3.4.4"],
Cell[268733, 7387, 906, 29, 62, "Input",
 CellTags->"b:3.4.4"],
Cell[269642, 7418, 287, 10, 25, "Text",
 CellTags->"b:3.4.4"],
Cell[269932, 7430, 1964, 62, 105, "Input",
 CellTags->"b:3.4.4"],
Cell[271899, 7494, 134, 4, 23, "Text",
 CellTags->"b:3.4.4"],
Cell[272036, 7500, 764, 25, 78, "Input",
 CellTags->"b:3.4.4"],
Cell[272803, 7527, 80, 2, 45, "Input",
 CellTags->"b:3.4.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[272920, 7534, 143, 3, 58, "Subsection",
 CellTags->{"c:34", "b:3.4.5"}],
Cell[273066, 7539, 160, 5, 23, "Text",
 CellTags->"b:3.4.5"],
Cell[273229, 7546, 127, 4, 45, "Input",
 CellTags->"b:3.4.5"],
Cell[273359, 7552, 132, 4, 46, "Input",
 CellTags->"b:3.4.5"],
Cell[273494, 7558, 291, 8, 61, "Input",
 CellTags->"b:3.4.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[273822, 7571, 190, 6, 58, "Subsection",
 CellTags->{"c:35", "b:3.4.6"}],
Cell[274015, 7579, 154, 5, 23, "Text",
 CellTags->"b:3.4.6"],
Cell[274172, 7586, 120, 4, 45, "Input",
 CellTags->"b:3.4.6"],
Cell[274295, 7592, 688, 22, 85, "Input",
 CellTags->"b:3.4.6"],
Cell[274986, 7616, 156, 5, 23, "Text",
 CellTags->"b:3.4.6"],
Cell[275145, 7623, 803, 22, 78, "Input",
 CellTags->"b:3.4.6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[275985, 7650, 169, 5, 58, "Subsection",
 CellTags->{"c:36", "b:3.4.7"}],
Cell[276157, 7657, 514, 12, 136, "Text",
 CellTags->"b:3.4.7"],
Cell[276674, 7671, 89, 3, 45, "Input",
 CellTags->"b:3.4.7"],
Cell[276766, 7676, 4219, 115, 267, "Input",
 CellTags->"b:3.4.7"],
Cell[280988, 7793, 767, 20, 110, "Input",
 CellTags->"b:3.4.7"],
Cell[281758, 7815, 81, 2, 45, "Input",
 CellTags->"b:3.4.7"],
Cell[281842, 7819, 264, 7, 79, "Text",
 CellTags->"b:3.4.7"],
Cell[282109, 7828, 503, 15, 62, "Input",
 CellTags->"b:3.4.7"],
Cell[282615, 7845, 43, 1, 45, "Input",
 CellTags->"b:3.4.7"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
